@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2016 - Raphael Araújo e Silva <raphael@pgmodeler.com.br>
+# Copyright 2006-2018 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -31,6 +31,7 @@ namespace ParsersAttributes {
 	ALIGNMENT=QString("alignment"),
 	ALIGN_OBJS_TO_GRID=QString("align-objs-to-grid"),
 	ALTER_CMDS=QString("alter-cmds"),
+	ALLOW_CONNS=QString("allow-conns"),
 	ANALYZE_FUNC=QString("analyze"),
 	ANCESTOR_TABLE=QString("ancestor-table"),
 	APPEND_AT_EOD=QString("append-at-eod"),
@@ -57,6 +58,7 @@ namespace ParsersAttributes {
 	BOTTOM=QString("bottom"),
 	BUFFERING=QString("buffering"),
 	BY_VALUE=QString("by-value"),
+	BYPASSRLS=QString("bypassrls"),
 	CACHE=QString("cache"),
 	CANONICAL_FUNC=QString("canonical"),
 	CANVAS_CORNER_MOVE=QString("canvas-corner-move"),
@@ -66,8 +68,10 @@ namespace ParsersAttributes {
 	CATEGORY=QString("category"),
 	CHANGE=QString("change"),
 	CHANGELOG=QString("changelog"),
+	CHECK_EXP=QString("check-exp"),
 	CHECK_UPDATE=QString("check-update"),
 	CK_CONSTR=QString("ck-constr"),
+	CLIENT_ENCODING=QString("client-encoding"),
 	CODE_COMPLETION=QString("code-completion"),
 	CODE_FONT_SIZE=QString("code-font-size"),
 	CODE_FONT=QString("code-font"),
@@ -79,9 +83,11 @@ namespace ParsersAttributes {
 	COLOR=QString("color"),
 	COLORS=QString("colors"),
 	COLS_COMMENT=QString("cols-comment"),
+	COL_IS_IDENTITY=QString("col-is-identity"),
 	COLUMN_ALIAS=QString("column-alias"),
 	COLUMN=QString("column"),
 	COLUMNS=QString("columns"),
+	COMMAND=QString("command"),
 	COMMANDS=QString("commands"),
 	COMMENT=QString("comment"),
 	COMMUTATOR_OP=QString("commutator-op"),
@@ -90,11 +96,13 @@ namespace ParsersAttributes {
 	COMPOSITE_TYPE=QString("composite"),
 	CONCURRENT=QString("concurrent"),
 	CONDITION=QString("condition"),
+	CONFIG_FILE=QString("config-file"),
 	CONFIGURATION=QString("configuration"),
 	CONFIRM_VALIDATION=QString("confirm-validation"),
 	CONN_LIMIT=QString("connlimit"),
 	CONNECT_CENTER_PNTS=QString("center-pnts"),
 	CONNECT_FK_TO_PK=QString("fk-to-pk"),
+	CONNECT_TABLE_EDGES=QString("table-edges"),
 	CONNECT_PRIV=QString("connect"),
 	CONNECTION=QString("connection"),
 	CONNECTIONS=QString("connections"),
@@ -103,6 +111,7 @@ namespace ParsersAttributes {
 	CONSTR_SQL_DISABLED=QString("constr-sql-disabled"),
 	CONSTRAINT=QString("constraint"),
 	CONSTRAINTS=QString("constraints"),
+	CONSTR_DEFS=QString("constr-defs"),
 	CONTENTS=QString("contents"),
 	COPY_MODE=QString("copy-mode"),
 	COPY_OPTIONS=QString("copy-options"),
@@ -112,13 +121,16 @@ namespace ParsersAttributes {
 	CREATE=QString("create"),
 	CREATEDB=QString("createdb"),
 	CREATEROLE=QString("createrole"),
+	CROWS_FOOT=QString("crows-foot"),
 	CTE_EXPRESSION=QString("cte-exp"),
 	CUR_VERSION=QString("cur-version"),
 	CUSTOM_COLOR=QString("custom-color"),
 	CUSTOM_FILTER=QString("custom-filter"),
 	CUSTOMIDXS=QString("customidxs"),
 	CYCLE=QString("cycle"),
+	DATA_DIRECTORY=QString("data-directory"),
 	DATE=QString("date"),
+	DATABASE=QString("database"),
 	DB_MODEL=QString("dbmodel"),
 	DDL_END_TOKEN=QString("-- ddl-end --"),
 	DECL_IN_TABLE=QString("decl-in-table"),
@@ -132,7 +144,7 @@ namespace ParsersAttributes {
 	DEFAULT=QString("default"),
 	DEFER_TYPE=QString("defer-type"),
 	DEFERRABLE=QString("deferrable"),
-	DEFINITION=QString("definition"),
+	DEFINITION=QString("definition"),	
 	DEL_ACTION=QString("del-action"),
 	DEL_EVENT=QString("del-event"),
 	DELETE_PRIV=QString("delete"),
@@ -144,6 +156,7 @@ namespace ParsersAttributes {
 	DIRECTORY=QString("directory"),
 	DISABLE_SMOOTHNESS=QString("disable-smoothness"),
 	DISPLAY_LINE_NUMBERS=QString("display-line-numbers"),
+	DOM_CONSTRAINT=QString("domconstraint"),
 	DOCK_WIDGETS=QString("dock-widgets"),
 	DROP_CMDS=QString("drop-cmds"),
 	DROP=QString("drop"),
@@ -154,6 +167,9 @@ namespace ParsersAttributes {
 	DST_LABEL=QString("dst-label"),
 	DST_REQUIRED=QString("dst-required"),
 	DST_TABLE=QString("dst-table"),
+	DYNAMIC_LIBRARY_PATH=QString("dynamic-library-path"),
+	DYNAMIC_SHARED_MEMORY=QString("dynamic-shared-memory-type"),
+	END_EXP=QString("end-exp"),
 	ELEMENT=QString("element"),
 	ELEMENTS=QString("elements"),
 	ENCODING=QString("encoding"),
@@ -178,6 +194,8 @@ namespace ParsersAttributes {
 	EXPRESSIONS=QString("expressions"),
 	EXT_OBJ_OIDS=QString("ext-obj-oids"),
 	FACTOR=QString("factor"),
+	FADEIN_OBJECTS=QString("fadein-objects"),
+	FADED_OUT=QString("faded-out"),
 	FAMILY=QString("family"),
 	FAST_UPDATE=QString("fast-update"),
 	FILE_ASSOCIATED=QString("file-associated"),
@@ -207,15 +225,18 @@ namespace ParsersAttributes {
 	HANDLES_TYPE=QString("handles-type"),
 	HAS_CHANGES=QString("has-changes"),
 	HASHES=QString("hashes"),
+	HBA_FILE=QString("hba-file"),
 	HIDE_EXT_ATTRIBS=QString("hide-ext-attribs"),
 	HIDE_REL_NAME=QString("hide-rel-name"),
 	HIDE_TABLE_TAGS=QString("hide-table-tags"),
 	HIGHLIGHT_LINES=QString("highlight-lines"),
-	HIGHLIGHT_OBJECTS=QString("highlight-objects"),
 	HIGHLIGHT_ORDER=QString("highlight-order"),
+	HISTORY_MAX_LENGTH=QString("history-max-length"),
 	ICON=QString("icon"),
 	ID=QString("id"),
 	IDENTIFIER=QString("identifier"),
+	IDENTITY_TYPE=QString("identity-type"),
+	IDENT_FILE=QString("ident-file"),
 	IGNORED_CHARS=QString("ignored-chars"),
 	IMPLICIT=QString("implicit"),
 	IMPORT=QString("import"),
@@ -238,7 +259,8 @@ namespace ParsersAttributes {
 	INSERT_PRIV=QString("insert"),
 	INTERNAL_LENGTH=QString("internal-length"),
 	INTERVAL_TYPE=QString("interval-type"),
-	INVERT_PANNING_RANGESEL=QString("invert-panning-rangesel"),
+	INVERT_RANGESEL_TRIGGER=QString("invert-rangesel-trigger"),
+	IS_TEMPLATE=QString("is-template"),
 	IO_CAST=QString("io-cast"),
 	ITALIC=QString("italic"),
 	JOIN_FUNC=QString("join"),
@@ -260,17 +282,20 @@ namespace ParsersAttributes {
 	LINE_NUMBERS_COLOR=QString("line-numbers-color"),
 	LINE_NUMBERS_BG_COLOR=QString("line-numbers-bg-color"),
 	LINE_HIGHLIGHT_COLOR=QString("line-highlight-color"),
+	LISTEN_ADDRESSES=QString("listen-addresses"),
 	LOCALE=QString("locale"),
 	LOCKER_ARC=QString("locker-arc"),
 	LOCKER_BODY=QString("locker-body"),
 	LOGIN=QString("login"),
 	LOOKAHEAD_CHAR=QString("lookahead-char"),
 	MATERIALIZED=QString("materialized"),
+	MAX_CONNECTIONS=QString("max-connections"),
 	MAX_VALUE=QString("max-value"),
 	MEMBER_ROLES=QString("member-roles"),
 	MEMBER=QString("member"),
 	MERGES=QString("merges"),
 	METADATA=QString("metadata"),
+	MIN_OBJECT_OPACITY=QString("min-object-opacity"),
 	MIN_VALUE=QString("min-value"),
 	MODE=QString("mode"),
 	MODEL_AUTHOR=QString("author"),
@@ -280,6 +305,7 @@ namespace ParsersAttributes {
 	NAMES=QString("names"),
 	NEGATOR_OP=QString("negator-op"),
 	NEW_NAME=QString("new-name"),
+	NEW_IDENTITY_TYPE=QString("new-identity-type"),
 	NEW_VERSION=QString("new-version"),
 	NN_COLUMN=QString("nn-column"),
 	NO_INHERIT=QString("no-inherit"),
@@ -322,10 +348,12 @@ namespace ParsersAttributes {
 	PARSABLE=QString("parsable"),
 	PARTIAL_MATCH=QString("partial-match"),
 	PASSWORD=QString("password"),
+	PASSWORD_ENCRYPTION=QString("password-encryption"),
 	PATH=QString("path"),
 	PATTERNS=QString("patterns"),
 	PER_ROW=QString("per-line"),
 	PERMISSION=QString("permission"),
+	PERMISSIVE=QString("permissive"),
 	PGMODELER_VERSION=QString("pgmodeler-ver"),
 	PGSQL_BASE_TYPE=QString("basetype"),
 	PGSQL_VERSION=QString("pgsql-ver"),
@@ -336,6 +364,7 @@ namespace ParsersAttributes {
 	PK_CONSTR=QString("pk-constr"),
 	PK_PATTERN=QString("pk-pattern"),
 	POINTS=QString("points"),
+	PORT=QString("port"),
 	PORTRAIT=QString("portrait"),
 	POSITION_INFO=QString("pos-info"),
 	POSITION=QString("position"),
@@ -364,8 +393,10 @@ namespace ParsersAttributes {
 	REF_TYPE=QString("ref-type"),
 	REFER=QString("refer"),
 	REFERENCE=QString("reference"),
+	REFERENCE_FK=QString("reference-fk"),
 	REFERENCES_PRIV=QString("references"),
 	REFERENCES=QString("references"),
+	REFERRERS=QString("referrers"),
 	REGULAR_EXP=QString("regexp"),
 	RELATION_TAB_VIEW=QString("reltv"),
 	RELATIONSHIP_11=QString("rel11"),
@@ -378,14 +409,20 @@ namespace ParsersAttributes {
 	RENAME=QString("rename"),
 	REPLICATION=QString("replication"),
 	RESTRICTION_FUNC=QString("restriction"),
+	RESTART_SEQ=QString("restart-seq"),
 	RETURN_TABLE=QString("return-table"),
 	RETURN_TYPE=QString("return-type"),
 	RETURNS_SETOF=QString("returns-setof"),
 	REVOKE=QString("revoke"),
 	RIGHT_TYPE=QString("right-type"),
 	RIGHT=QString("right"),
+	RLS_ENABLED=QString("rls-enabled"),
+	RLS_FORCED=QString("rls-forced"),
 	ROLE_TYPE=QString("role-type"),
 	ROLE=QString("role"),
+	ROLE_CURRENT_USER=QString("CURRENT_USER"),
+	ROLE_SESSION_USER=QString("SESSION_USER"),
+	ROLE_PUBLIC=QString("PUBLIC"),
 	ROLES=QString("roles"),
 	ROW_AMOUNT=QString("row-amount"),
 	RULES=QString("rules"),
@@ -395,8 +432,13 @@ namespace ParsersAttributes {
 	SECURITY_TYPE=QString("security-type"),
 	SELECT_EXP=QString("select-exp"),
 	SELECT_PRIV=QString("select"),
+	SELECT_OBJECTS=QString("select-objects"),
 	SEND_FUNC=QString("send"),
 	SEQUENCE=QString("sequence"),
+	SERVER_ENCODING=QString("server-encoding"),
+	SERVER_VERSION=QString("server-version"),
+	SERVER_PID=QString("server-pid"),
+	SERVER_PROTOCOL=QString("server-protocol"),
 	SET_PERMS=QString("set-perms"),
 	SHARED_OBJ=QString("shared-obj"),
 	SHELL_TYPES=QString("shell-types"),
@@ -412,6 +454,8 @@ namespace ParsersAttributes {
 	SIZE=QString("size"),
 	SNIPPET=QString("snippet"),
 	SORT_OP=QString("sort-op"),
+	SOURCE_EDITOR_APP=QString("source-editor-app"),
+	SOURCE_EDITOR_ARGS=QString("source-editor-args"),
 	SOURCE_TYPE=QString("source-type"),
 	SRC_TYPE=QString("src-type"),
 	SPATIAL_TYPE=QString("spatial-type"),
@@ -428,6 +472,11 @@ namespace ParsersAttributes {
 	SRC_REQUIRED=QString("src-required"),
 	SRC_TABLE=QString("src-table"),
 	SRID=QString("srid"),
+	SSL=QString("ssl"),
+	SSL_CA_FILE=QString("ssl-ca-file"),
+	SSL_CERT_FILE=QString("ssl-cert-file"),
+	SSL_CRL_FILE=QString("ssl-crl-file"),
+	SSL_KEY_FILE=QString("ssl-key-file"),
 	START=QString("start"),
 	STATE_TYPE=QString("state-type"),
 	STORAGE_PARAMS=QString("stg-params"),
@@ -468,6 +517,7 @@ namespace ParsersAttributes {
 	TYPE=QString("type"),
 	TYPES=QString("types"),
 	TYPE_CLASS=QString("type-class"),
+	UI_LANGUAGE=QString("ui-language"),
 	UNDERLINE=QString("underline"),
 	UNIQUE=QString("unique"),
 	UNLOGGED=QString("unlogged"),
@@ -480,9 +530,11 @@ namespace ParsersAttributes {
 	UQ_CONSTR=QString("uq-constr"),
 	UQ_PATTERN=QString("uq-pattern"),
 	USAGE_PRIV=QString("usage"),
+	USE_CURVED_LINES=QString("use-curved-lines"),
 	USE_PLACEHOLDERS=QString("use-placeholders"),
 	USE_SORTING=QString("use-sorting"),
 	USE_UNIQUE_NAMES=QString("use-unique-names"),
+	USING_EXP=QString("using-exp"),
 	VALIDATOR_FUNC=QString("validator"),
 	VALIDATOR=QString("validator"),
 	VALIDITY=QString("validity"),
