@@ -1,6 +1,6 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
-# Copyright 2006-2016 - Raphael Araújo e Silva <raphael@pgmodeler.com.br>
+# Copyright 2006-2018 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -61,12 +61,14 @@ enum ObjectType {
 	OBJ_COLLATION,
 	OBJ_EXTENSION,
 	OBJ_EVENT_TRIGGER,
+	OBJ_POLICY,
 	OBJ_RELATIONSHIP,
 	OBJ_TEXTBOX,
 	OBJ_PERMISSION,
 	OBJ_PARAMETER,
 	OBJ_TYPE_ATTRIBUTE,
 	OBJ_TAG,
+	OBJ_GENERIC_SQL,
 	BASE_RELATIONSHIP,
 	BASE_OBJECT,
 	BASE_TABLE
@@ -83,6 +85,9 @@ class BaseObject {
 		//! \brief Stores the database wich the object belongs
 		BaseObject *database;
 
+		//! \brief Stores the set of special (valid) chars that forces the object's name quoting
+		static const QByteArray special_chars;
+
 	protected:
 		SchemaParser schparser;
 
@@ -99,8 +104,8 @@ class BaseObject {
 		 in which the objects were created */
 		unsigned object_id;
 
-		//! \brief Objects type count declared on enum ObjectType (excluding BASE_OBJECT and BASE_TABLE).
-		static const int OBJECT_TYPE_COUNT=35;
+		//! \brief Objects type count declared on enum ObjectType
+		static const int OBJECT_TYPE_COUNT=37;
 
 		/*! \brief Indicates whether the object is protected or not.
 		 A protected object indicates that it can not suffer changes in position
