@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2016 - Raphael Araújo e Silva <raphael@pgmodeler.com.br>
+# Copyright 2006-2018 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -593,8 +593,8 @@ void Constraint::setColumnsNotNull(bool value)
 	{
 		for(auto &col : columns)
 		{
-			if(!col->isAddedByRelationship())
-				col->setNotNull(value);
+			//if(!col->isAddedByRelationship())
+			col->setNotNull(value);
 		}
 	}
 }
