@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2016 - Raphael Araújo e Silva <raphael@pgmodeler.com.br>
+# Copyright 2006-2018 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -53,6 +53,7 @@ Additionally, this class, saves, loads and generates the XML/SQL definition of a
 #include "extension.h"
 #include "tag.h"
 #include "eventtrigger.h"
+#include "genericsql.h"
 #include <algorithm>
 #include <locale.h>
 
@@ -86,6 +87,12 @@ class DatabaseModel:  public QObject, public BaseObject {
 		//! \brief Maximum number of connections
 		int conn_limit;
 
+		//! \brief Indicates if the database can be used as template
+		bool is_template,
+
+		//! \brief Indicates if the database accepts connection
+		allow_conns;
+
 		//! \brief Vectors that stores all the objects types
 		vector<BaseObject *> textboxes;
 		vector<BaseObject *> relationships;
@@ -111,6 +118,7 @@ class DatabaseModel:  public QObject, public BaseObject {
 		vector<BaseObject *> extensions;
 		vector<BaseObject *> tags;
 		vector<BaseObject *> eventtriggers;
+		vector<BaseObject *> genericsqls;
 
 		/*! \brief Stores the xml definition for special objects. This map is used
 		 when revalidating the relationships */
@@ -168,9 +176,12 @@ class DatabaseModel:  public QObject, public BaseObject {
 		META_OBJS_SQLDISABLED=8,	//! \brief Handle objects' sql disabled status when save/load metadata file
 		META_OBJS_CUSTOMSQL=16,	//! \brief Handle object's custom sql when save/load metadata file
 		META_OBJS_CUSTOMCOLORS=32,	//! \brief Handle object's custom colors when save/load metadata file
-		META_TEXTBOX_OBJS=64,	//! \brief Handle textboxes object when save/load metadata file
-		META_TAG_OBJS=128,	//! \brief Handle tags object when save/load metadata file
-		META_ALL_INFO=255;	//! \brief Handle all metadata information about objects when save/load metadata file
+		META_OBJS_FADEDOUT=64,	//! \brief Handle graphical object's fade out status when save/load metadata file
+		META_OBJS_EXTATTRIBS=128,	//! \brief Handle tables and views extended attributes display when save/load metadata file
+		META_TEXTBOX_OBJS=256,	//! \brief Handle textboxes object when save/load metadata file
+		META_TAG_OBJS=512,	//! \brief Handle tags object when save/load metadata file
+		META_GENERIC_SQL_OBJS=1024,	//! \brief Handle generic sql object when save/load metadata file
+		META_ALL_INFO=2047;	//! \brief Handle all metadata information about objects when save/load metadata file
 
 		DatabaseModel(void);
 
@@ -250,12 +261,20 @@ class DatabaseModel:  public QObject, public BaseObject {
 
 		void setDefaultObject(BaseObject *object, ObjectType obj_type=BASE_OBJECT);
 
+		void setIsTemplate(bool value);
+
+		void setAllowConnections(bool value);
+
 		//! \brief Returns the current state of the sql appeding at end of entire model definition
 		bool isAppendAtEOD(void);
 
 		//! \brief Returns the current state of the sql prepeding at beginning of entire model definition
 		bool isPrependedAtBOD(void);
 
+		bool isTemplate(void);
+
+		bool isAllowConnections(void);
+
 		//! \brief Destroys all the objects
 		void destroyObjects(void);
 
@@ -326,8 +345,9 @@ class DatabaseModel:  public QObject, public BaseObject {
 
 		/*! \brief Searchs and returns the relationship between the specified tables. If the second parameter
 		 is ommited (nullptr), the method returns the first relationship where the source table is
-		 participating */
-		BaseRelationship *getRelationship(BaseTable *src_tab, BaseTable *dst_tab);
+		 participating. The optional parameter ref_fk will search for foreign key relationships which the reference foreign key
+		is the one provided */
+		BaseRelationship *getRelationship(BaseTable *src_tab, BaseTable *dst_tab, Constraint *ref_fk = nullptr);
 
 		//! \brief Searchs and returns all the relationships that the specified table participates
 		vector<BaseRelationship *> getRelationships(BaseTable *tab);
@@ -437,6 +457,11 @@ class DatabaseModel:  public QObject, public BaseObject {
 		EventTrigger *getEventTrigger(unsigned obj_idx);
 		EventTrigger *getEventTrigger(const QString &name);
 
+		void addGenericSQL(GenericSQL *genericsql, int obj_idx=-1);
+		void removeGenericSQL(GenericSQL *genericsql, int obj_idx=-1);
+		GenericSQL *getGenericSQL(unsigned obj_idx);
+		GenericSQL *getGenericSQL(const QString &name);
+
 		void addPermission(Permission *perm);
 		void removePermission(Permission *perm);
 
@@ -492,7 +517,9 @@ class DatabaseModel:  public QObject, public BaseObject {
 		Rule *createRule(void);
 		Index *createIndex(void);
 		Trigger *createTrigger(void);
+		Policy *createPolicy(void);
 		EventTrigger *createEventTrigger(void);
+		GenericSQL *createGenericSQL(void);
 
 		//! \brief Creates/removes the relationship between the passed view and the referecend tables
 		void updateViewRelationships(View *view, bool force_rel_removal=false);
@@ -534,10 +561,13 @@ class DatabaseModel:  public QObject, public BaseObject {
 		meaning that ALL objects directly or inderectly linked to the 'object' are retrieved. */
 		void __getObjectReferences(BaseObject *object, vector<BaseObject *> &refs, bool exclude_perms=false);
 
-		/*! \brief Marks the graphical objects as modified forcing their redraw. User can specify only a set of
+		/*! \brief Marks the graphical objects of the provided types as modified forcing their redraw. User can specify only a set of
 	 graphical objects to be marked */
 		void setObjectsModified(vector<ObjectType> types={});
 
+		//! \brief Marks the graphical objects in the list as modified forcing their redraw.
+		void setObjectsModified(vector<BaseObject *> &objects);
+
 		/*! \brief Marks the objects with code invalidated forcing their code regeneration. User can specify only a set of
 		 graphical objects to be marked */
 		void setCodesInvalidated(vector<ObjectType> types={});
