@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2016 - Raphael Araújo e Silva <raphael@pgmodeler.com.br>
+# Copyright 2006-2018 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -106,38 +106,39 @@ QString BaseType::type_list[types_count]=
 	"tsrange","tstzrange","daterange",
 
 	//Object Identification type (OID)
-	//offsets 108 to 120
+	//offsets 108 to 122
 	"oid", "regproc", "regprocedure",
 	"regoper", "regoperator", "regclass",
-	"regtype", "regconfig", "regdictionary",
-	"xid", "cid", "tid",  "oidvector",
+	"regrole", "regnamespace", "regtype",
+	"regconfig", "regdictionary", "xid", "cid",
+	"tid",  "oidvector",
 
 	//Pseudo-types
-	//offsets 121 to 135
+	//offsets 123 to 137
 	"\"any\"","anyarray","anyelement","anyenum",
 	"anynonarray", "anyrange", "cstring","internal","language_handler",
 	"record","trigger","void","opaque", "fdw_handler", "event_trigger",
 
 	//Interval types
-	//offsets 136 to 148
+	//offsets 138 to 150
 	"YEAR", "MONTH", "DAY", "HOUR",
 	"MINUTE", "SECOND","YEAR TO MONTH",
 	"DAY TO HOUR","DAY TO MINUTE","DAY TO SECOND",
 	"HOUR TO MINUTE","HOUR TO SECOND","MINUTE TO SECOND",
 
 	//Types used by the class BehaviorType
-	//offsets 149 to 151
+	//offsets 151 to 153
 	"CALLED ON NULL INPUT",
 	"RETURNS NULL ON NULL INPUT",
 	"STRICT",
 
 	//Types used by the class SecurityType
-	//offsets 152 to 153
+	//offsets 154 to 155
 	"SECURITY INVOKER",
 	"SECURITY DEFINER",
 
 	//Types used by the class LanguageType
-	//offsets 154 to 160
+	//offsets 156 to 162
 	"sql",
 	"c",
 	"plpgsql",
@@ -147,7 +148,7 @@ QString BaseType::type_list[types_count]=
 	"internal",
 
 	//Types used by the class EncodingType
-	//offsets 161 to 202
+	//offsets 163 to 204
 	"UTF8", "BIG5", "EUC_CN", "EUC_JP", "EUC_JIS_2004",
 	"EUC_KR", "EUC_TW", "GB18030", "GBK",
 	"ISO_8859_5", "ISO_8859_6", "ISO_8859_7", "ISO_8859_8",
@@ -160,25 +161,25 @@ QString BaseType::type_list[types_count]=
 	"WIN1254", "WIN1255", "WIN1256", "WIN1257", "WIN1258",
 
 	//Types used by the class StorageType
-	//offsets 203 to 206
+	//offsets 205 to 208
 	"plain",
 	"external",
 	"extended",
 	"main",
 
 	//Types used by the class MatchType
-	//offsets 207 to 209
+	//offsets 209 to 211
 	"MATCH FULL",
 	"MATCH PARTIAL",
 	"MATCH SIMPLE",
 
 	//Types used by the class DeferralType
-	//offsets 210 to 211
+	//offsets 212 to 213
 	"INITIALLY IMMEDIATE",
 	"INITIALLY DEFERRED",
 
 	//Types used by the class CategoryType
-	//offsets 212 to 225 - See table 44-43 on PostgreSQL 8.4 documentation
+	//offsets 214 to 227 - See table 44-43 on PostgreSQL 8.4 documentation
 	"U", //User-defined types
 	"A", //Array types
 	"B", //Boolean types
@@ -195,7 +196,7 @@ QString BaseType::type_list[types_count]=
 	"X", //Unknown type
 
 	//Types used by the class FiringType
-	//offsets 226 to 228
+	//offsets 228 to 230
 	"BEFORE",
 	"AFTER",
 	"INSTEAD OF",
@@ -204,7 +205,7 @@ QString BaseType::type_list[types_count]=
 	These types accepts variations Z, M e ZM.
 	> Example: POINT, POINTZ, POINTM, POINTZM
 	Reference: http://postgis.refractions.net/documentation/manual-2.0/using_postgis_dbmanagement.html */
-	//offsets 229 to 244
+	//offsets 231 to 246
 	"POINT",
 	"LINESTRING",
 	"POLYGON",
@@ -223,16 +224,29 @@ QString BaseType::type_list[types_count]=
 	"MULTISURFACE",
 
 	//Types used by the class EventTriggerType
-	//offsets 245 to 248
+	//offsets 247 to 250
 	"ddl_command_start",
 	"ddl_command_end",
 	"sql_drop",
-	"table_rewrite"
+	"table_rewrite",
+
+	//Types used by the class IdentityType
+	//offsets 251 to 252
+	"ALWAYS",
+	"BY DEFAULT",
+
+	//Types used by the class PolicyCmdType
+	//offsets 253 to 257
+	"ALL",
+	"SELECT",
+	"INSERT",
+	"DELETE",
+	"UPDATE"
 };
 
 BaseType::BaseType(void)
 {
-	type_idx=0;
+	type_idx=BaseType::null;
 }
 
 QString BaseType::getTypeString(unsigned type_id)
@@ -258,7 +272,7 @@ void BaseType::setType(unsigned type_id,unsigned offset,unsigned count)
 bool BaseType::isTypeValid(unsigned type_id,unsigned offset,unsigned count)
 {
 	//Returns if the type id is valid according to the specified interval (offset-count)
-	return((type_id>=offset && type_id<=(offset+count-1)) || type_id==0);
+	return((type_id>=offset && type_id<=(offset+count-1)) || type_id==BaseType::null);
 }
 
 void BaseType::getTypes(QStringList &types,unsigned offset,unsigned count)
@@ -725,7 +739,7 @@ vector<UserTypeConfig> PgSQLType::user_types;
 PgSQLType::PgSQLType(void)
 {
 	type_idx=offset;
-	length=1;
+	length=0;
 	precision=-1;
 	dimension=0;
 	with_timezone=false;
@@ -778,8 +792,8 @@ PgSQLType PgSQLType::parseString(const QString &str)
 {
 	QString type_str=str.toLower().simplified(), sptype, interv;
 	bool with_tz=false;
-	unsigned len=0, dim=0, srid=0;
-	int prec=-1;
+	unsigned dim=0, srid=0;
+	int prec=-1, len = -1;
 	int start=-1, end=-1;
 	QStringList value, intervals;
 	PgSQLType type;
@@ -816,7 +830,7 @@ PgSQLType PgSQLType::parseString(const QString &str)
 	{
 		start=type_str.indexOf('(');
 		end=type_str.indexOf(')', start);
-		len=type_str.mid(start+1, end-start-1).toUInt();
+		len=type_str.mid(start+1, end-start-1).toInt();
 	}
 	//Check if the type is a numeric type, e.g, numeric(10,2)
 	else if(QRegExp(QString("(.)+\\(( )*[0-9]+( )*(,)( )*[0-9]+( )*\\)")).indexIn(type_str) >=0)
@@ -824,7 +838,7 @@ PgSQLType PgSQLType::parseString(const QString &str)
 		start=type_str.indexOf('(');
 		end=type_str.indexOf(')', start);
 		value=type_str.mid(start+1, end-start-1).split(',');
-		len=value[0].toUInt();
+		len=value[0].toInt();
 		prec=value[1].toUInt();
 	}
 	//Check if the type is a spatial type (PostGiS), e.g, geography(POINTZ, 4296)
@@ -879,7 +893,7 @@ PgSQLType PgSQLType::parseString(const QString &str)
 			type.setLength(len);
 			type.setPrecision(prec);
 		}
-		else if(type.isDateTimeType() && len > 0)
+		else if(type.isDateTimeType() && len >= 0)
 			type.setPrecision(len);
 		else if(type.hasVariableLength() && len > 0)
 			type.setLength(len);
@@ -967,8 +981,20 @@ unsigned PgSQLType::getTypeId(void)
 	return(!(*this));
 }
 
-QString PgSQLType::getTypeName(void)
+QString PgSQLType::getTypeName(bool incl_dimension)
 {
+	if(incl_dimension)
+	{
+		QString type;
+
+		type=~(*this);
+
+		if(type!=QString("void") && dimension > 0)
+			type+=QString("[]").repeated(dimension);
+
+		return(type);
+	}
+
 	return(~(*this));
 }
 
@@ -1448,6 +1474,7 @@ bool PgSQLType::isEquivalentTo(PgSQLType type)
 																		{QString("decimal"),QString("numeric")},
 																		{QString("character varying"),QString("varchar")},
 																		{QString("character"), QString("char")},
+																		{QString("bool"), QString("boolean")},
 																		{QString("bit varying"),QString("varbit")},
 																		{QString("oid"),QString("regproc"),QString("regprocedure"),
 																		 QString("regoper"),QString("regoperator"),QString("regclass"),
@@ -1510,10 +1537,6 @@ void PgSQLType::setDimension(unsigned dim)
 
 void PgSQLType::setLength(unsigned len)
 {
-	//Raises an error if the length is 0
-	if(len==0)
-		throw Exception(ERR_ASG_ZERO_LENGTH,__PRETTY_FUNCTION__,__FILE__,__LINE__);
-
 	this->length=len;
 }
 
@@ -1521,7 +1544,7 @@ void PgSQLType::setPrecision(int prec)
 {
 	if(!isUserType())
 	{
-		//Raises an error if the user tries to specify a precision > lenght
+		//Raises an error if the user tries to specify a precision > length
 		if(((BaseType::type_list[type_idx]==QString("numeric") ||
 			 BaseType::type_list[type_idx]==QString("decimal")) && prec > static_cast<int>(length)))
 			throw Exception(ERR_ASG_INV_PRECISION,__PRETTY_FUNCTION__,__FILE__,__LINE__);
@@ -1570,9 +1593,7 @@ QString PgSQLType::getCodeDefinition(unsigned def_type,QString ref_type)
 		attribs[ParsersAttributes::REF_TYPE]=ref_type;
 
 		attribs[ParsersAttributes::NAME]=(~(*this));
-
-		if(length > 1)
-			attribs[ParsersAttributes::LENGTH]=QString("%1").arg(this->length);
+		attribs[ParsersAttributes::LENGTH]=QString("%1").arg(this->length);
 
 		if(dimension > 0)
 			attribs[ParsersAttributes::DIMENSION]=QString("%1").arg(this->dimension);
@@ -1607,15 +1628,16 @@ QString PgSQLType::operator * (void)
 	//Generation the definition for the spatial types (PostGiS)
 	if(type==QString("geometry") || type==QString("geography"))
 		fmt_type=type + (*spatial_type);
-	else if(length > 1 && hasVariableLength())
+	else if(hasVariableLength())
 	{
 		//Configuring the precision
-		if((type==QString("numeric") || type==QString("decimal")) && precision>=0 &&
-				precision<=static_cast<int>(length))
+		if((type==QString("numeric") || type==QString("decimal")) && length >= 1 && precision>=0 && precision<=static_cast<int>(length))
 			aux=QString("%1(%2,%3)").arg(BaseType::type_list[type_idx]).arg(length).arg(precision);
 		//Configuring the length for the type
-		else
+		else if(length >= 1)
 			aux=QString("%1(%2)").arg(BaseType::type_list[type_idx]).arg(length);
+		else
+			aux=type;
 
 		fmt_type=aux;
 	}
@@ -2097,3 +2119,79 @@ unsigned EventTriggerType::operator = (const QString &type_name)
 	return(type_id);
 }
 
+/***************************
+ * CLASS: IdentityMode *
+ ***************************/
+IdentityType::IdentityType(void)
+{
+	type_idx=offset;
+}
+
+IdentityType::IdentityType(unsigned type_id)
+{
+	(*this)=type_id;
+}
+
+IdentityType::IdentityType(const QString &type_name)
+{
+	(*this)=type_name;
+}
+
+void IdentityType::getTypes(QStringList &tipos)
+{
+	BaseType::getTypes(tipos,offset,types_count);
+}
+
+unsigned IdentityType::operator = (unsigned type_id)
+{
+	BaseType::setType(type_id,offset,types_count);
+	return(type_idx);
+}
+
+unsigned IdentityType::operator = (const QString &type_name)
+{
+	unsigned type_id;
+
+	type_id=BaseType::getType(type_name, offset, types_count);
+	BaseType::setType(type_id,offset,types_count);
+	return(type_id);
+}
+
+/***************************
+ * CLASS: PolicyCmdType *
+ ***************************/
+PolicyCmdType::PolicyCmdType(void)
+{
+	type_idx=offset;
+}
+
+PolicyCmdType::PolicyCmdType(unsigned type_id)
+{
+	(*this)=type_id;
+}
+
+PolicyCmdType::PolicyCmdType(const QString &type_name)
+{
+	(*this)=type_name;
+}
+
+void PolicyCmdType::getTypes(QStringList &tipos)
+{
+	BaseType::getTypes(tipos,offset,types_count);
+}
+
+unsigned PolicyCmdType::operator = (unsigned type_id)
+{
+	BaseType::setType(type_id,offset,types_count);
+	return(type_idx);
+}
+
+unsigned PolicyCmdType::operator = (const QString &type_name)
+{
+	unsigned type_id;
+
+	type_id=BaseType::getType(type_name, offset, types_count);
+	BaseType::setType(type_id,offset,types_count);
+	return(type_id);
+}
+
