@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2016 - Raphael Araújo e Silva <raphael@pgmodeler.com.br>
+# Copyright 2006-2018 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -1355,7 +1355,7 @@ void Relationship::addForeignKey(Table *ref_tab, Table *recv_tab, ActionType del
 		}
 
 		//Configures the foreign key name
-		i=1;
+
 		aux.clear();
 
 		if(rel_type!=RELATIONSHIP_NN)
@@ -1423,7 +1423,7 @@ void Relationship::addAttributes(Table *recv_tab)
 	}
 }
 
-void Relationship::copyColumns(Table *ref_tab, Table *recv_tab, bool not_null)
+void Relationship::copyColumns(Table *ref_tab, Table *recv_tab, bool not_null, bool is_dst_table)
 {
 	Constraint *dst_pk=nullptr, *src_pk=nullptr, *pk=nullptr;
 	unsigned i, count;
@@ -1469,7 +1469,7 @@ void Relationship::copyColumns(Table *ref_tab, Table *recv_tab, bool not_null)
 				name=generateObjectName(SRC_COL_PATTERN, column_aux);
 			else
 			{
-				if(ref_tab==src_table)
+				if(ref_tab==src_table && (!isSelfRelationship() || (isSelfRelationship() && !is_dst_table)))
 					name=generateObjectName(SRC_COL_PATTERN, column_aux);
 				else
 					name=generateObjectName(DST_COL_PATTERN, column_aux);
@@ -1712,7 +1712,7 @@ void Relationship::addColumnsRelNn(void)
 		/* Copy the columns from the primary keys of the source and destination tables
 		 to the table that represents the n-n relationship */
 		copyColumns(tab, table_relnn, src_not_null);
-		copyColumns(tab1, table_relnn, dst_not_null);
+		copyColumns(tab1, table_relnn, dst_not_null, true);
 
 		if(single_pk_column)
 		{
@@ -1732,7 +1732,10 @@ void Relationship::addColumnsRelNn(void)
 		if(!single_pk_column)
 		{
 			for(auto &col : gen_columns)
+			{
+				col->setNotNull(true);
 				pk_tabnn->addColumn(col, Constraint::SOURCE_COLS);
+			}
 		}
 		else
 		{
@@ -2230,8 +2233,6 @@ bool Relationship::isInvalidated(void)
 				//Gets the amount of columns from the primary key
 				tab_cols_count=pk->getColumnCount(Constraint::SOURCE_COLS);
 
-				table1=getReceiverTable();
-
 				//Compares the column quantity
 				valid=(rel_cols_count==tab_cols_count);
 
@@ -2541,3 +2542,21 @@ QString Relationship::getInheritDefinition(bool undo_inherit)
 	return(BaseObject::getAlterDefinition(this->getSchemaName(), attributes));
 }
 
+
+bool Relationship::isReferenceTableMandatory(void)
+{
+	if(rel_type == BaseRelationship::RELATIONSHIP_11 && getReferenceTable() == dst_table && !src_mandatory)
+		return(dst_mandatory);
+	else
+		return((getReferenceTable() == src_table && isTableMandatory(SRC_TABLE)) ||
+					 (getReferenceTable() == dst_table && isTableMandatory(DST_TABLE)));
+}
+
+bool Relationship::isReceiverTableMandatory(void)
+{
+	if(rel_type == BaseRelationship::RELATIONSHIP_11 && getReferenceTable() == dst_table && !src_mandatory)
+		return(false);
+	else
+		return((getReceiverTable() == src_table && isTableMandatory(SRC_TABLE)) ||
+					 (getReceiverTable() == dst_table && isTableMandatory(DST_TABLE)));
+}
