@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2016 - Raphael Araújo e Silva <raphael@pgmodeler.com.br>
+# Copyright 2006-2018 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -38,28 +38,30 @@ ConstraintWidget::ConstraintWidget(QWidget *parent): BaseObjectWidget(parent, OB
 		expression_hl=new SyntaxHighlighter(expression_txt, false, true);
 		expression_hl->loadConfiguration(GlobalAttributes::SQL_HIGHLIGHT_CONF_PATH);
 
-		columns_tab=new ObjectTableWidget(ObjectTableWidget::ALL_BUTTONS ^
-										  (ObjectTableWidget::EDIT_BUTTON |
-										   ObjectTableWidget::UPDATE_BUTTON), true, this);
+		columns_tab=new ObjectsTableWidget(ObjectsTableWidget::ALL_BUTTONS ^
+										  (ObjectsTableWidget::EDIT_BUTTON |
+											 ObjectsTableWidget::UPDATE_BUTTON |
+											 ObjectsTableWidget::DUPLICATE_BUTTON), true, this);
 
-		ref_columns_tab=new ObjectTableWidget(ObjectTableWidget::ALL_BUTTONS ^
-											  (ObjectTableWidget::EDIT_BUTTON |
-											   ObjectTableWidget::UPDATE_BUTTON), true, this);
+		ref_columns_tab=new ObjectsTableWidget(ObjectsTableWidget::ALL_BUTTONS ^
+											  (ObjectsTableWidget::EDIT_BUTTON |
+												 ObjectsTableWidget::UPDATE_BUTTON |
+												ObjectsTableWidget::DUPLICATE_BUTTON), true, this);
 
 		ref_table_sel=new ObjectSelectorWidget(OBJ_TABLE, true, this);
 
 		columns_tab->setColumnCount(2);
 		columns_tab->setHeaderLabel(trUtf8("Column"), 0);
-		columns_tab->setHeaderIcon(QPixmap(QString(":/icones/icones/column.png")),0);
+		columns_tab->setHeaderIcon(QPixmap(PgModelerUiNS::getIconPath("column")),0);
 		columns_tab->setHeaderLabel(trUtf8("Type"), 1);
-		columns_tab->setHeaderIcon(QPixmap(QString(":/icones/icones/usertype.png")),1);
+		columns_tab->setHeaderIcon(QPixmap(PgModelerUiNS::getIconPath("usertype")),1);
 
 		ref_columns_tab->setEnabled(false);
 		ref_columns_tab->setColumnCount(2);
 		ref_columns_tab->setHeaderLabel(trUtf8("Column"), 0);
-		ref_columns_tab->setHeaderIcon(QPixmap(QString(":/icones/icones/column.png")),0);
+		ref_columns_tab->setHeaderIcon(QPixmap(PgModelerUiNS::getIconPath("column")),0);
 		ref_columns_tab->setHeaderLabel(trUtf8("Type"), 1);
-		ref_columns_tab->setHeaderIcon(QPixmap(QString(":/icones/icones/usertype.png")),1);
+		ref_columns_tab->setHeaderIcon(QPixmap(PgModelerUiNS::getIconPath("usertype")),1);
 
 		dynamic_cast<QGridLayout *>(columns_tbw->widget(0)->layout())->addWidget(columns_tab, 1,0,1,3);
 		dynamic_cast<QGridLayout *>(columns_tbw->widget(1)->layout())->addWidget(ref_table_sel, 0,1,1,2);
@@ -124,7 +126,7 @@ ConstraintWidget::ConstraintWidget(QWidget *parent): BaseObjectWidget(parent, OB
 void ConstraintWidget::addColumn(int row)
 {
 	QObject *sender_obj=sender();
-	ObjectTableWidget *aux_col_tab=nullptr;
+	ObjectsTableWidget *aux_col_tab=nullptr;
 	QComboBox *combo=nullptr;
 	Column *column=nullptr;
 	unsigned col_id;
@@ -154,7 +156,7 @@ void ConstraintWidget::addColumn(int row)
 		addColumn(column, col_id, row);
 
 		//When there is no items con the combo the insert button of the table is disabled
-		aux_col_tab->setButtonsEnabled(ObjectTableWidget::ADD_BUTTON, (combo->count()!=0));
+		aux_col_tab->setButtonsEnabled(ObjectsTableWidget::ADD_BUTTON, (combo->count()!=0));
 	}
 	catch(Exception &e)
 	{
@@ -181,7 +183,7 @@ void ConstraintWidget::removeColumns(void)
 
 void ConstraintWidget::addColumn(Column *column, unsigned col_id, int row)
 {
-	ObjectTableWidget *table_wgt=nullptr;
+	ObjectsTableWidget *table_wgt=nullptr;
 
 	if(column && row >= 0)
 	{
@@ -211,7 +213,7 @@ void ConstraintWidget::addColumn(Column *column, unsigned col_id, int row)
 
 void ConstraintWidget::updateColumnsCombo(unsigned col_id)
 {
-	ObjectTableWidget *aux_col_tab=nullptr;
+	ObjectsTableWidget *aux_col_tab=nullptr;
 	Column *column=nullptr;
 	Table *table=nullptr;
 	QComboBox *combo=nullptr;
@@ -265,7 +267,7 @@ void ConstraintWidget::updateColumnsCombo(unsigned col_id)
 							   QString(")"), QVariant::fromValue<void *>(column));
 		}
 
-		aux_col_tab->setButtonsEnabled(ObjectTableWidget::ADD_BUTTON, (combo->count()!=0));
+		aux_col_tab->setButtonsEnabled(ObjectsTableWidget::ADD_BUTTON, (combo->count()!=0));
 	}
 	catch(Exception &e)
 	{
@@ -292,33 +294,6 @@ void ConstraintWidget::selectReferencedTable(void)
 	}
 }
 
-void ConstraintWidget::hideEvent(QHideEvent *event)
-{
-	expression_txt->clear();
-	column_cmb->clear();
-	ref_column_cmb->clear();
-
-	no_inherit_chk->setChecked(false);
-	deferrable_chk->setChecked(false);
-	constr_type_lbl->setEnabled(true);
-	constr_type_cmb->setEnabled(true);
-	constr_type_cmb->setCurrentIndex(0);
-	match_cmb->setCurrentIndex(0);
-	deferral_cmb->setCurrentIndex(0);
-
-	columns_tab->blockSignals(true);
-	ref_columns_tab->blockSignals(true);
-	columns_tab->removeRows();
-	ref_columns_tab->removeRows();
-	columns_tab->blockSignals(false);
-	ref_columns_tab->blockSignals(false);
-
-	ref_table_sel->clearSelector();
-	excl_elems_wgt->clear();
-
-	BaseObjectWidget::hideEvent(event);
-}
-
 void ConstraintWidget::selectConstraintType(void)
 {
 	ConstraintType constr_type=ConstraintType(constr_type_cmb->currentText());
@@ -418,7 +393,7 @@ void ConstraintWidget::setAttributes(DatabaseModel *model, OperationList *op_lis
 	}
 
 	updateColumnsCombo(Constraint::SOURCE_COLS);
-	columns_tab->setButtonsEnabled(ObjectTableWidget::ADD_BUTTON, (column_cmb->count()!=0));
+	columns_tab->setButtonsEnabled(ObjectsTableWidget::ADD_BUTTON, (column_cmb->count()!=0));
 	columns_tab->blockSignals(false);
 
 	if(constr)
@@ -463,7 +438,7 @@ void ConstraintWidget::setAttributes(DatabaseModel *model, OperationList *op_lis
 			}
 
 			updateColumnsCombo(Constraint::REFERENCED_COLS);
-			ref_columns_tab->setButtonsEnabled(ObjectTableWidget::ADD_BUTTON, (column_cmb->count()!=0));
+			ref_columns_tab->setButtonsEnabled(ObjectsTableWidget::ADD_BUTTON, (column_cmb->count()!=0));
 			ref_columns_tab->blockSignals(false);
 		}
 	}
@@ -478,7 +453,7 @@ void ConstraintWidget::applyConfiguration(void)
 		Constraint *constr=nullptr;
 		unsigned i, col_id, count;
 		Column *column=nullptr;
-		ObjectTableWidget *aux_col_tab=nullptr;
+		ObjectsTableWidget *aux_col_tab=nullptr;
 		vector<ExcludeElement> excl_elems;
 
 		startConfiguration<Constraint>();
