@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2016 - Raphael Araújo e Silva <raphael@pgmodeler.com.br>
+# Copyright 2006-2018 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -311,11 +311,11 @@ void DatabaseImportHelper::createObjects(void)
 				 in order to be created later */
 			if(obj_type!=OBJ_CONSTRAINT)
 			{
-				emit s_progressUpdated(progress,
-										 trUtf8("Creating object `%1' (%2)...")
-									   .arg(attribs[ParsersAttributes::NAME])
-						.arg(BaseObject::getTypeName(obj_type)),
-						obj_type);
+				emit s_progressUpdated(progress, trUtf8("Creating object `%1' (%2), oid `%3'...")
+															.arg(attribs[ParsersAttributes::NAME])
+															.arg(BaseObject::getTypeName(obj_type))
+															.arg(attribs[ParsersAttributes::OID]),
+															obj_type);
 
 				createObject(attribs);
 			}
@@ -361,9 +361,10 @@ void DatabaseImportHelper::createObjects(void)
 				itr++;
 
 				emit s_progressUpdated(progress,
-										 trUtf8("Trying to recreate object `%1' (%2)...")
-									   .arg(attribs[ParsersAttributes::NAME])
-						.arg(BaseObject::getTypeName(obj_type)),
+										 trUtf8("Trying to recreate object `%1' (%2), oid `%3'...")
+										.arg(attribs[ParsersAttributes::NAME])
+										.arg(BaseObject::getTypeName(obj_type))
+										.arg(attribs[ParsersAttributes::OID]),
 						obj_type);
 
 				try
@@ -656,6 +657,10 @@ void DatabaseImportHelper::createObject(attribs_map &attribs)
 	ObjectType obj_type=static_cast<ObjectType>(attribs[ParsersAttributes::OBJECT_TYPE].toUInt());
 	QString obj_name=getObjectName(attribs[ParsersAttributes::OID], (obj_type==OBJ_FUNCTION || obj_type==OBJ_OPERATOR));
 
+	//Avoiding the creation of pgModeler's temp objects created in database during the catalog reading
+	if(obj_name.contains(Catalog::PGMODELER_TEMP_DB_OBJ))
+		return;
+
 	try
 	{
 		if(!import_canceled &&
@@ -669,7 +674,7 @@ void DatabaseImportHelper::createObject(attribs_map &attribs)
 				attribs[ParsersAttributes::DECL_IN_TABLE]=QString();
 
 			//System objects will have the sql disabled by default
-			attribs[ParsersAttributes::SQL_DISABLED]=(oid > catalog.getLastSysObjectOID() ? QString() : ParsersAttributes::_TRUE_);
+			attribs[ParsersAttributes::SQL_DISABLED]=(catalog.isSystemObject(oid) || catalog.isExtensionObject(oid) ? ParsersAttributes::_TRUE_ : QString());
 			attribs[ParsersAttributes::COMMENT]=getComment(attribs);
 
 			if(attribs.count(ParsersAttributes::OWNER))
@@ -681,7 +686,11 @@ void DatabaseImportHelper::createObject(attribs_map &attribs)
 			if(attribs.count(ParsersAttributes::SCHEMA))
 				attribs[ParsersAttributes::SCHEMA]=getDependencyObject(attribs[ParsersAttributes::SCHEMA], OBJ_SCHEMA, false, auto_resolve_deps);
 
-			if(!attribs[ParsersAttributes::PERMISSION].isEmpty())
+			/* Due to the object recreation mechanism there are some situations when pgModeler fails to recreate
+			them due to the duplication of permissions. So, to avoid this problem we need to check if the OID of the
+			object was previously registered in the vector of permissions to be created */
+			if(!attribs[ParsersAttributes::PERMISSION].isEmpty() &&
+				 std::find(obj_perms.begin(), obj_perms.end(), oid)==obj_perms.end())
 				obj_perms.push_back(oid);
 
 			if(debug_mode)
@@ -715,6 +724,7 @@ void DatabaseImportHelper::createObject(attribs_map &attribs)
 				case OBJ_TRIGGER: createTrigger(attribs); break;
 				case OBJ_INDEX: createIndex(attribs); break;
 				case OBJ_CONSTRAINT: createConstraint(attribs); break;
+				case OBJ_POLICY: createPolicy(attribs); break;
 				case OBJ_EVENT_TRIGGER: createEventTrigger(attribs); break;
 
 				default:
@@ -733,8 +743,8 @@ void DatabaseImportHelper::createObject(attribs_map &attribs)
 	catch(Exception &e)
 	{
 		throw Exception(Exception::getErrorMessage(ERR_OBJECT_NOT_IMPORTED)
-						.arg(obj_name).arg(BaseObject::getTypeName(obj_type)),
-						ERR_OBJECT_NOT_IMPORTED,__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
+										.arg(obj_name).arg(BaseObject::getTypeName(obj_type)).arg(attribs[ParsersAttributes::OID]),
+										ERR_OBJECT_NOT_IMPORTED,__PRETTY_FUNCTION__,__FILE__,__LINE__, &e, dumpObjectAttributes(attribs));
 	}
 }
 
@@ -885,6 +895,7 @@ void DatabaseImportHelper::resetImportParameters(void)
 	col_perms.clear();
 	connection.close();
 	catalog.closeConnection();
+	inherited_cols.clear();
 }
 
 QString DatabaseImportHelper::dumpObjectAttributes(attribs_map &attribs)
@@ -972,9 +983,31 @@ void DatabaseImportHelper::createRole(attribs_map &attribs)
 void DatabaseImportHelper::createDomain(attribs_map &attribs)
 {
 	Domain *dom=nullptr;
+	QStringList constraints, constr_attrs;
+	attribs_map aux_attribs;
+	QString expr;
 
 	try
 	{
+		constraints = Catalog::parseArrayValues(attribs[ParsersAttributes::CONSTRAINTS]);
+		attribs[ParsersAttributes::CONSTRAINTS].clear();
+
+		for(auto constr : constraints)
+		{
+			constr.remove(0, 1);
+			constr.remove(constr.length() - 1, 1);
+			constr_attrs = constr.split(QChar(':'));
+
+			aux_attribs[ParsersAttributes::NAME] = constr_attrs.at(0);
+
+			expr = constr_attrs.at(1);
+			expr.remove(0,1);
+			expr.remove(expr.length() - 1,1);
+			aux_attribs[ParsersAttributes::EXPRESSION] = expr;
+
+			attribs[ParsersAttributes::CONSTRAINTS]+= schparser.getCodeDefinition(ParsersAttributes::DOM_CONSTRAINT, aux_attribs, SchemaParser::XML_DEFINITION);
+		}
+
 		attribs[ParsersAttributes::TYPE]=getType(attribs[ParsersAttributes::TYPE], true, attribs);
 		attribs[ParsersAttributes::COLLATION]=getDependencyObject(attribs[ParsersAttributes::COLLATION], OBJ_COLLATION);
 		loadObjectXML(OBJ_DOMAIN, attribs);
@@ -1013,9 +1046,9 @@ void DatabaseImportHelper::createFunction(attribs_map &attribs)
 	Parameter param;
 	PgSQLType type;
 	unsigned dim=0;
-	QStringList param_types, param_names, param_modes, param_def_vals;
-	int def_val_idx=0;
+	QStringList param_types, param_names, param_modes, param_def_vals, param_xmls;
 	QString param_tmpl_name=QString("_param%1");
+	vector<Parameter> parameters;
 
 	try
 	{
@@ -1068,15 +1101,36 @@ void DatabaseImportHelper::createFunction(attribs_map &attribs)
 				param.setVariadic(param_modes[i]==QString("v"));
 			}
 
-			//Setting the default value for the current paramenter. OUT parameter doesn't receive default values.
-			if(def_val_idx < param_def_vals.size() && (!param.isOut() || (param.isIn() && param.isOut())))
-				param.setDefaultValue(param_def_vals[def_val_idx++]);
-
 			//If the mode is 't' indicates that the current parameter will be used as a return table colum
 			if(!param_modes.isEmpty() && param_modes[i]==QString("t"))
 				attribs[ParsersAttributes::RETURN_TABLE]+=param.getCodeDefinition(SchemaParser::XML_DEFINITION);
 			else
-				attribs[ParsersAttributes::PARAMETERS]+=param.getCodeDefinition(SchemaParser::XML_DEFINITION);
+				parameters.push_back(param);
+		}
+
+		if(!parameters.empty())
+		{
+			vector<Parameter>::reverse_iterator ritr, ritr_end;
+
+			ritr = parameters.rbegin();
+			ritr_end = parameters.rend();
+
+			while(ritr != ritr_end)
+			{
+				param = *ritr;
+				ritr++;
+
+				//Setting the default value for the current paramenter. OUT parameter doesn't receive default values.
+				if(!param_def_vals.isEmpty() && (!param.isOut() || (param.isIn() && param.isOut())))
+				{
+					param.setDefaultValue(param_def_vals.back());
+					param_def_vals.pop_back();
+				}
+
+				param_xmls.push_front(param.getCodeDefinition(SchemaParser::XML_DEFINITION));
+			}
+
+			attribs[ParsersAttributes::PARAMETERS]+=param_xmls.join(QChar('\n'));
 		}
 
 		//Case the function's language is C the symbol is the 'definition' attribute
@@ -1371,6 +1425,7 @@ void DatabaseImportHelper::createConversion(attribs_map &attribs)
 void DatabaseImportHelper::createSequence(attribs_map &attribs)
 {
 	Sequence *seq=nullptr;
+	Column *col = nullptr;
 
 	try
 	{
@@ -1382,11 +1437,32 @@ void DatabaseImportHelper::createSequence(attribs_map &attribs)
 
 		attribs[ParsersAttributes::OWNER_COLUMN]=QString();
 
-		/* If there are owner columns and the oid of sequence is greater that the owner column's table oid
+		/* If there are owner columns and the oid of sequence is greater than the owner column's table oid
 		stores the oid of both (sequence and table) in order to swap it's ids at the end of import to
 		avoid reference breaking when generation SQL code */
-		if(owner_col.size()==2 && attribs[ParsersAttributes::OID].toUInt() > owner_col[0].toUInt())
-			seq_tab_swap[attribs[ParsersAttributes::OID]]=owner_col[0];
+		if(owner_col.size()==2)
+		{
+			Table *tab = nullptr;
+			QString col_name, tab_name;
+			attribs_map pos_attrib={
+				{ ParsersAttributes::X_POS, QString("0") },
+				{ ParsersAttributes::Y_POS, QString("0") }};
+
+			if(attribs[ParsersAttributes::OID].toUInt() > owner_col[0].toUInt())
+				seq_tab_swap[attribs[ParsersAttributes::OID]]=owner_col[0];
+
+			/* Get the table and the owner column instances so the sequence code can be disabled if the
+				column is an identity one */
+			tab_name = getDependencyObject(owner_col[0], OBJ_TABLE, true, auto_resolve_deps, false,
+			{{ ParsersAttributes::POSITION,
+				 schparser.getCodeDefinition(ParsersAttributes::POSITION, pos_attrib, SchemaParser::XML_DEFINITION)}});
+
+			col_name=getColumnName(owner_col[0], owner_col[1]);
+			tab = dbmodel->getTable(tab_name);
+
+			if(tab)
+				col = tab->getColumn(col_name);
+		}
 
 		for(int i=0; i < seq_attribs.size(); i++)
 			attribs[attr[i]]=seq_attribs[i];
@@ -1394,6 +1470,10 @@ void DatabaseImportHelper::createSequence(attribs_map &attribs)
 		loadObjectXML(OBJ_SEQUENCE, attribs);
 		seq=dbmodel->createSequence();
 		dbmodel->addSequence(seq);
+
+		//Disable the sequence's SQL when the owner column is identity
+		if(col && col->isIdentity())
+			seq->setSQLDisabled(true);
 	}
 	catch(Exception &e)
 	{
@@ -1418,9 +1498,10 @@ void DatabaseImportHelper::createAggregate(attribs_map &attribs)
 			attribs[func_types[i]]=getDependencyObject(attribs[func_types[i]], OBJ_FUNCTION, true, auto_resolve_deps, true, {{ParsersAttributes::REF_TYPE, func_types[i]}});
 
 		types=getTypes(attribs[ParsersAttributes::TYPES], true);
+		attribs[ParsersAttributes::TYPES]=QString();
+
 		if(!types.isEmpty())
 		{
-			attribs[ParsersAttributes::TYPES]=QString();
 			for(int i=0; i < types.size(); i++)
 				attribs[ParsersAttributes::TYPES]+=types[i];
 		}
@@ -1507,13 +1588,18 @@ void DatabaseImportHelper::createType(attribs_map &attribs)
 
 			attribs[ParsersAttributes::ELEMENT]=getType(attribs[ParsersAttributes::ELEMENT], false);
 
-			for(i=0; i < count; i++)
+			/* Workaround: if importing a datatype that is part of an extension we avoid the importing of
+			 * its supporting functions (since they will not be necessary here because the type will be sql-disabled)*/
+			if(!catalog.isExtensionObject(attribs[ParsersAttributes::OID].toUInt()))
 			{
-				attribs[func_types[i]]=getDependencyObject(attribs[func_types[i]], OBJ_FUNCTION, true, true, true, {{ParsersAttributes::REF_TYPE, func_types[i]}});
+				for(i=0; i < count; i++)
+				{
+					attribs[func_types[i]]=getDependencyObject(attribs[func_types[i]], OBJ_FUNCTION, true, true, true, {{ParsersAttributes::REF_TYPE, func_types[i]}});
 
-				/* Since pgModeler requires that type functions refers to the constructing type as "any"
-					 it's necessary to replace the function parameter types names */
-				attribs[func_types[i]].replace(QString("IN ") + type_name, QString("IN any"));
+					/* Since pgModeler requires that type functions refers to the constructing type as "any"
+						 it's necessary to replace the function parameter types names */
+					attribs[func_types[i]].replace(QString("IN ") + type_name, QString("IN any"));
+				}
 			}
 		}
 
@@ -1539,11 +1625,11 @@ void DatabaseImportHelper::createTable(attribs_map &attribs)
 		bool is_type_registered=false;
 		Column col;
 		vector<unsigned> inh_cols;
-		QString type_def, unknown_obj_xml, type_name;
+		QString type_def, unknown_obj_xml, type_name, def_val;
 		map<unsigned, attribs_map>::iterator itr, itr1, itr_end;
-		attribs_map pos_attrib={{ ParsersAttributes::X_POS, QString("0") },
-								{ ParsersAttributes::Y_POS, QString("0") }};
-
+		attribs_map pos_attrib={
+			{ ParsersAttributes::X_POS, QString("0") },
+			{ ParsersAttributes::Y_POS, QString("0") }};
 
 		attribs[ParsersAttributes::COLUMNS]=QString();
 		attribs[ParsersAttributes::POSITION]=schparser.getCodeDefinition(ParsersAttributes::POSITION, pos_attrib, SchemaParser::XML_DEFINITION);
@@ -1584,8 +1670,8 @@ void DatabaseImportHelper::createTable(attribs_map &attribs)
 			{
 				/* Building the type name prepending the schema name in order to search it on
 		   the user defined types list at PgSQLType class */
-				type_name=getObjectName(types[type_oid][ParsersAttributes::SCHEMA], true);
-				type_name+=QString(".") + types[type_oid][ParsersAttributes::NAME];
+				type_name=BaseObject::formatName(getObjectName(types[type_oid][ParsersAttributes::SCHEMA], true), false);
+				type_name+=QString(".") + BaseObject::formatName(types[type_oid][ParsersAttributes::NAME], false);
 				is_type_registered=PgSQLType::isRegistered(type_name, dbmodel);
 			}
 			else
@@ -1609,10 +1695,44 @@ void DatabaseImportHelper::createTable(attribs_map &attribs)
 					type_def=getDependencyObject(itr->second[ParsersAttributes::TYPE_OID], OBJ_DOMAIN);
 			}
 
+			col.setIdentityType(BaseType::null);
 			col.setType(PgSQLType::parseString(type_name));
 			col.setNotNull(!itr->second[ParsersAttributes::NOT_NULL].isEmpty());
 			col.setComment(itr->second[ParsersAttributes::COMMENT]);
-			col.setDefaultValue(itr->second[ParsersAttributes::DEFAULT_VALUE]);
+
+			//Overriding the default value if the column is identity
+			if(!itr->second[ParsersAttributes::IDENTITY_TYPE].isEmpty())
+				col.setIdentityType(itr->second[ParsersAttributes::IDENTITY_TYPE]);
+			else
+			{
+				/* Removing extra/forced type casting in the retrieved default value.
+				 This is done in order to avoid unnecessary entries in the diff results.
+
+				 For instance: say in the model we have a column with the following configutation:
+				 > varchar(3) default 'foo'
+
+				 Now, when importing the same column the default value for it will be something like:
+				 > varchar(3) default 'foo'::character varying
+
+				 Since the extra chars in the default value of the imported column are redundant (casting
+				 varchar to character varying) we remove the '::character varying'. The idea here is to eliminate
+				 the cast if the casting is equivalent to the column type. */
+
+				def_val = itr->second[ParsersAttributes::DEFAULT_VALUE];
+
+				if(!def_val.startsWith(QString("nextval(")) && def_val.contains(QString("::")))
+				{
+					QStringList values = def_val.split(QString("::"));
+
+					if(values.size() > 1 &&
+						 ((~col.getType() == values[1]) ||
+							(~col.getType() == QString("char") && values[1] == QString("bpchar")) ||
+							(col.getType().isUserType() && (~col.getType()).endsWith(values[1]))))
+						def_val=values[0];
+				}
+
+				col.setDefaultValue(def_val);
+			}
 
 			//Checking if the collation used by the column exists, if not it'll be created when auto_resolve_deps is checked
 			if(auto_resolve_deps && !itr->second[ParsersAttributes::COLLATION].isEmpty())
@@ -1659,6 +1779,8 @@ void DatabaseImportHelper::createView(attribs_map &attribs)
 		loadObjectXML(OBJ_VIEW, attribs);
 		view=dbmodel->createView();
 		dbmodel->addView(view);
+
+		retrieveTableColumns(view->getSchema()->getName(), view->getName());
 	}
 	catch(Exception &e)
 	{
@@ -1726,61 +1848,121 @@ void DatabaseImportHelper::createTrigger(attribs_map &attribs)
 	}
 }
 
+QStringList DatabaseImportHelper::parseIndexExpressions(const QString &expr)
+{
+	int open_paren = 0, close_paren = 0, pos = 0;
+	QStringList expressions;
+	QChar chr;
+	QString word;
+	bool open_apos = false;
+
+	if(!expr.isEmpty())
+	{
+		while(pos < expr.length())
+		{
+			chr = expr[pos++];
+			word += chr;
+
+			if(chr == QChar('\''))
+				open_apos = !open_apos;
+
+			if(!open_apos && chr == QChar('('))
+				open_paren++;
+			else if(!open_apos && chr == QChar(')'))
+				close_paren++;
+
+			if(chr == QChar(',') || pos == expr.length())
+			{
+				if(open_paren == close_paren)
+				{
+					if(word.endsWith(QChar(',')))
+						word.remove(word.length() - 1, 1);
+
+					if(word.contains('(') && word.contains(')'))
+						expressions.push_back(word.trimmed());
+
+					word.clear();
+					open_paren = close_paren = 0;
+				}
+			}
+		}
+	}
+
+	return(expressions);
+}
+
 void DatabaseImportHelper::createIndex(attribs_map &attribs)
 {
 	try
 	{
-		QStringList cols, exprs, opclasses, collations;
+		QStringList cols, opclasses, collations, exprs;
 		IndexElement elem;
-		Table *table=nullptr;
+		BaseTable *parent_tab=nullptr;
 		Collation *coll=nullptr;
 		OperatorClass *opclass=nullptr;
 		QString tab_name, coll_name, opc_name;
-		int i, id_expr;
+		int i;
 
 		attribs[ParsersAttributes::FACTOR]=QString("90");
 		tab_name=getDependencyObject(attribs[ParsersAttributes::TABLE], OBJ_TABLE, true, auto_resolve_deps, false);
-		table=dynamic_cast<Table *>(dbmodel->getObject(tab_name, OBJ_TABLE));
+		parent_tab=dynamic_cast<BaseTable *>(dbmodel->getObject(tab_name, OBJ_TABLE));
 
-		if(!table)
-			throw Exception(Exception::getErrorMessage(ERR_REF_OBJ_INEXISTS_MODEL)
-							.arg(attribs[ParsersAttributes::NAME]).arg(BaseObject::getTypeName(OBJ_INDEX))
-				.arg(tab_name).arg(BaseObject::getTypeName(OBJ_TABLE))
-				,ERR_REF_OBJ_INEXISTS_MODEL ,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+		if(!parent_tab)
+		{
+			tab_name=getDependencyObject(attribs[ParsersAttributes::TABLE], OBJ_VIEW, true, auto_resolve_deps, false);
+			parent_tab=dynamic_cast<BaseTable *>(dbmodel->getObject(tab_name, OBJ_VIEW));
+
+			if(!parent_tab)
+				throw Exception(Exception::getErrorMessage(ERR_REF_OBJ_INEXISTS_MODEL)
+												.arg(attribs[ParsersAttributes::NAME]).arg(BaseObject::getTypeName(OBJ_INDEX))
+												.arg(tab_name).arg(BaseObject::getTypeName(OBJ_TABLE)),
+												ERR_REF_OBJ_INEXISTS_MODEL ,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+		}
 
 		cols=Catalog::parseArrayValues(attribs[ParsersAttributes::COLUMNS]);
-		exprs=Catalog::parseArrayValues(attribs[ParsersAttributes::EXPRESSIONS]);
 		collations=Catalog::parseArrayValues(attribs[ParsersAttributes::COLLATIONS]);
 		opclasses=Catalog::parseArrayValues(attribs[ParsersAttributes::OP_CLASSES]);
+		exprs = parseIndexExpressions(attribs[ParsersAttributes::EXPRESSIONS]);
 
-		for(i=0, id_expr=0; i < cols.size(); i++)
+		for(i=0; i < cols.size(); i++)
 		{
 			elem=IndexElement();
 
 			if(cols[i]!=QString("0"))
-				elem.setColumn(table->getColumn(getColumnName(attribs[ParsersAttributes::TABLE], cols[i])));
-			else if(id_expr < exprs.size())
-				elem.setExpression(exprs[id_expr++]);
+			{
+				if(parent_tab->getObjectType() == OBJ_TABLE)
+					elem.setColumn(dynamic_cast<Table *>(parent_tab)->getColumn(getColumnName(attribs[ParsersAttributes::TABLE], cols[i])));
+				else
+					elem.setExpression(getColumnName(attribs[ParsersAttributes::TABLE], cols[i]));
+			}
+			else if(!exprs.isEmpty())
+			{
+				elem.setExpression(exprs.front());
+				exprs.pop_front();
+			}
 
 			if(i < collations.size() && collations[i]!=QString("0"))
 			{
 				coll_name=getDependencyObject(collations[i], OBJ_COLLATION, false, true, false);
 				coll=dynamic_cast<Collation *>(dbmodel->getObject(coll_name, OBJ_COLLATION));
 
-				if(coll)
+				//Even if the collation exists we'll ignore it when it is the "pg_catalog.default"
+				if(coll && (!coll->isSystemObject() ||
+										(coll->isSystemObject() && coll->getName() != QString("default"))))
 					elem.setCollation(coll);
 			}
 
 			if(i < opclasses.size() && opclasses[i]!=QString("0"))
 			{
-				opc_name=getDependencyObject(opclasses[i], OBJ_OPCLASS, false, true, false);
+				opc_name=getDependencyObject(opclasses[i], OBJ_OPCLASS, true, true, false);
 				opclass=dynamic_cast<OperatorClass *>(dbmodel->getObject(opc_name, OBJ_OPCLASS));
 
 				if(opclass)
 					elem.setOperatorClass(opclass);
 			}
 
-			attribs[ParsersAttributes::ELEMENTS]+=elem.getCodeDefinition(SchemaParser::XML_DEFINITION);
+			if(elem.getColumn() || !elem.getExpression().isEmpty())
+				attribs[ParsersAttributes::ELEMENTS]+=elem.getCodeDefinition(SchemaParser::XML_DEFINITION);
 		}
 
 		attribs[ParsersAttributes::TABLE]=tab_name;
@@ -1821,7 +2003,7 @@ void DatabaseImportHelper::createConstraint(attribs_map &attribs)
 
 			if(attribs[ParsersAttributes::TYPE]==ParsersAttributes::EX_CONSTR)
 			{
-				QStringList cols, exprs, opclasses, opers;
+				QStringList cols, opclasses, opers, exprs;
 				ExcludeElement elem;
 				QString opc_name, op_name;
 				OperatorClass *opclass=nullptr;
@@ -1831,22 +2013,34 @@ void DatabaseImportHelper::createConstraint(attribs_map &attribs)
 				attribs[ParsersAttributes::EXPRESSION]=attribs[ParsersAttributes::CONDITION];
 
 				cols=Catalog::parseArrayValues(attribs[ParsersAttributes::COLUMNS]);
-				exprs=Catalog::parseArrayValues(attribs[ParsersAttributes::EXPRESSIONS]);
 				opers=Catalog::parseArrayValues(attribs[ParsersAttributes::OPERATORS]);
 				opclasses=Catalog::parseArrayValues(attribs[ParsersAttributes::OP_CLASSES]);
 
-				for(int i=0, id_expr=0; i < cols.size(); i++)
+				/* Due to the way exclude constraints are constructed (similar to indexes),
+				 * we get the constraint's definition in for of expressions. Internally we use pg_get_constraintdef.
+				 * This way we will get EXCLUDE USING [index type](elements). The elements in this case is a set of expression
+				 * which we work to separate column only references from complex expression. Only complex expression will be used
+				 * and assigned to their exclude constraint elements. Column references are used in exclude elements but relying in
+				 * the cols list above */
+				exprs=attribs[ParsersAttributes::EXPRESSIONS]
+							.replace(QString("EXCLUDE USING %1 (").arg(attribs[ParsersAttributes::INDEX_TYPE]), QString())
+							.split(QRegExp("(WITH )(\\+|\\-|\\*|\\/|\\<|\\>|\\=|\\~|\\!|\\@|\\#|\\%|\\^|\\&|\\||\\'|\\?)+((,)?|(\\))?)"),
+										 QString::SkipEmptyParts);
+
+				for(int i=0; i < cols.size(); i++)
 				{
 					elem=ExcludeElement();
 
-					if(cols[i]!=QString("0"))
+					if(cols[i] != QString("0"))
 						elem.setColumn(table->getColumn(getColumnName(table_oid, cols[i])));
-					else if(id_expr < exprs.size())
-						elem.setExpression(exprs[id_expr++]);
+					else
+						elem.setExpression(exprs.front().trimmed());
+
+					exprs.pop_front();
 
 					if(i < opclasses.size() && opclasses[i]!=QString("0"))
 					{
-						opc_name=getDependencyObject(opclasses[i], OBJ_OPCLASS, false, true, false);
+						opc_name=getDependencyObject(opclasses[i], OBJ_OPCLASS, true, true, false);
 						opclass=dynamic_cast<OperatorClass *>(dbmodel->getObject(opc_name, OBJ_OPCLASS));
 
 						if(opclass)
@@ -1864,7 +2058,6 @@ void DatabaseImportHelper::createConstraint(attribs_map &attribs)
 
 					attribs[ParsersAttributes::ELEMENTS]+=elem.getCodeDefinition(SchemaParser::XML_DEFINITION);
 				}
-
 			}
 			else
 			{
@@ -1896,6 +2089,22 @@ void DatabaseImportHelper::createConstraint(attribs_map &attribs)
 	}
 }
 
+void DatabaseImportHelper::createPolicy(attribs_map &attribs)
+{
+	try
+	{
+		attribs[ParsersAttributes::TABLE]=getDependencyObject(attribs[ParsersAttributes::TABLE], OBJ_TABLE, true, auto_resolve_deps, false);
+		attribs[ParsersAttributes::ROLES]=getObjectNames(attribs[ParsersAttributes::ROLES]).join(',');
+		loadObjectXML(OBJ_POLICY, attribs);
+		dbmodel->createPolicy();
+	}
+	catch(Exception &e)
+	{
+		throw Exception(e.getErrorMessage(), e.getErrorType(),
+						__PRETTY_FUNCTION__,__FILE__,__LINE__, &e, xmlparser->getXMLBuffer());
+	}
+}
+
 void DatabaseImportHelper::createEventTrigger(attribs_map &attribs)
 {
 	try
@@ -1967,6 +2176,13 @@ void DatabaseImportHelper::createPermission(attribs_map &attribs)
 			{
 				role=dynamic_cast<Role *>(dbmodel->getObject(role_name, OBJ_ROLE));
 
+				if(auto_resolve_deps && !role_name.isEmpty() && !role)
+				{
+					QString oid = catalog.getObjectOID(role_name, OBJ_ROLE);
+					getDependencyObject(oid, OBJ_ROLE);
+					role=dynamic_cast<Role *>(dbmodel->getObject(role_name, OBJ_ROLE));
+				}
+
 				/* If the role doesn't exists and there is a name defined, throws an error because
 				the roles wasn't found on the model */
 				if(!role && !role_name.isEmpty())
@@ -2047,7 +2263,7 @@ void DatabaseImportHelper::destroyDetachedColumns(void)
 
 	dbmodel->disconnectRelationships();
 
-	emit s_progressUpdated(95,
+	emit s_progressUpdated(100,
 						   trUtf8("Destroying unused detached columns..."),
 						   OBJ_COLUMN);
 
@@ -2084,7 +2300,7 @@ void DatabaseImportHelper::assignSequencesToColumns(void)
 {
 	Table *table=nullptr;
 	Column *col=nullptr;
-	emit s_progressUpdated(95,
+	emit s_progressUpdated(100,
 							 trUtf8("Assigning sequences to columns..."),
 						   OBJ_SEQUENCE);
 
@@ -2171,6 +2387,8 @@ void DatabaseImportHelper::__createTableInheritances(void)
 
 					//Create the inheritance relationship
 					rel=new Relationship(Relationship::RELATIONSHIP_GEN, child_tab, parent_tab);
+					rel->setName(PgModelerNS::generateUniqueName(rel, (*dbmodel->getObjectList(OBJ_RELATIONSHIP))));
+
 					dbmodel->addRelationship(rel);
 					rel=nullptr;
 				}
@@ -2193,10 +2411,6 @@ void DatabaseImportHelper::configureDatabase(attribs_map &attribs)
 	try
 	{
 		attribs[ParsersAttributes::APPEND_AT_EOD]=QString();
-
-		//Removing the encoding suffix from LC_COLLATE and LC_CTYPE attribs
-		attribs[ParsersAttributes::_LC_COLLATE_].remove(QRegExp(QString("(\\.)(.)+")));
-		attribs[ParsersAttributes::_LC_CTYPE_].remove(QRegExp(QString("(\\.)(.)+")));
 		loadObjectXML(OBJ_DATABASE, attribs);
 		dbmodel->configureDatabase(attribs);
 	}
@@ -2239,7 +2453,7 @@ QString DatabaseImportHelper::getObjectName(const QString &oid, bool signature_f
 				obj_name.prepend(sch_name + QString("."));
 
 			//Formatting the name in form of signature (only for functions and operators)
-			if(signature_form && (obj_type==OBJ_FUNCTION || obj_type==OBJ_OPERATOR || obj_type==OBJ_AGGREGATE || obj_type==OBJ_OPFAMILY))
+			if(signature_form && (obj_type==OBJ_FUNCTION || obj_type==OBJ_OPERATOR || obj_type==OBJ_AGGREGATE || obj_type==OBJ_OPFAMILY || obj_type==OBJ_OPCLASS))
 			{
 				QStringList params;
 
@@ -2287,7 +2501,7 @@ QString DatabaseImportHelper::getObjectName(const QString &oid, bool signature_f
 					obj_name += QString(" USING %1").arg(obj_attr[ParsersAttributes::INDEX_TYPE]);
 				}
 
-				if(obj_type != OBJ_OPFAMILY)
+				if(obj_type != OBJ_OPFAMILY && obj_type != OBJ_OPCLASS)
 					obj_name+=QString("(") + params.join(',') + QString(")");
 			}
 
@@ -2355,8 +2569,10 @@ QString DatabaseImportHelper::getType(const QString &oid_str, bool generate_xml,
 	try
 	{
 		attribs_map type_attr;
-		QString xml_def, sch_name, obj_name;
-		unsigned type_oid=oid_str.toUInt(), dimension=0, object_id=type_attr[ParsersAttributes::OBJECT_ID].toUInt();
+		QString xml_def, sch_name, obj_name, aux_name;
+		unsigned type_oid=oid_str.toUInt(), elem_tp_oid = 0,
+				dimension=0, object_id=type_attr[ParsersAttributes::OBJECT_ID].toUInt();
+		bool is_derivated_from_obj = false;
 
 		if(type_oid > 0)
 		{
@@ -2368,6 +2584,8 @@ QString DatabaseImportHelper::getType(const QString &oid_str, bool generate_xml,
 					type_attr[ParsersAttributes::NAME].contains(QString("[]")))
 			{
 				obj_name=type_attr[ParsersAttributes::NAME];
+				elem_tp_oid=type_attr[ParsersAttributes::ELEMENT].toUInt();
+
 				if(generate_xml)
 				{
 					dimension=type_attr[ParsersAttributes::NAME].count(QString("[]"));
@@ -2393,6 +2611,7 @@ QString DatabaseImportHelper::getType(const QString &oid_str, bool generate_xml,
 				else
 					obj_type=OBJ_SEQUENCE;
 
+				is_derivated_from_obj = true;
 				getDependencyObject(type_attr[ParsersAttributes::OBJECT_ID], obj_type, true, true, false);
 			}
 
@@ -2405,16 +2624,27 @@ QString DatabaseImportHelper::getType(const QString &oid_str, bool generate_xml,
 			/* Prepend the schema name only if it is not a system schema ('pg_catalog' or 'information_schema') and
 		 if the schema's names is already present in the type's name (in case of table types) */
 			sch_name=getObjectName(type_attr[ParsersAttributes::SCHEMA]);
-			if(!sch_name.isEmpty() && sch_name!=QString("pg_catalog") &&
-					sch_name!=QString("information_schema") &&
-					!obj_name.contains(QRegExp(QString("^(\\\")?(%1)(\\\")?(.)").arg(sch_name))))
+			if(!sch_name.isEmpty() &&
+				 ((sch_name!=QString("pg_catalog") && sch_name!=QString("information_schema")) ||
+					type_oid > catalog.getLastSysObjectOID()) &&
+				 !obj_name.contains(QRegExp(QString("^(\\\")?(%1)(\\\")?(\\.)").arg(sch_name))))
 				obj_name.prepend(sch_name + QString("."));
 
-			/* In case of auto resolve dependencies, if the type is a user defined one and was not created in the database
-					model but its attributes were retrieved the object will be created to avoid reference errors */
-			if(auto_resolve_deps && !type_attr.empty() &&
-				 type_oid > catalog.getLastSysObjectOID() && !dbmodel->getType(obj_name))
-				createObject(type_attr);
+			/* In case of auto resolve dependencies, if the type is a user defined one and is not derivated from table/view/sequence and
+			 was not created in the database model but its attributes were retrieved, the object will be created to avoid reference errors */
+			aux_name = obj_name;
+			aux_name.remove(QString("[]"));
+			if(auto_resolve_deps && !type_attr.empty() && !is_derivated_from_obj &&
+				 type_oid > catalog.getLastSysObjectOID() && !dbmodel->getType(aux_name))
+			{
+				//If the type is not an array one we simply use the current type attributes map
+				if(type_attr[ParsersAttributes::CATEGORY] != QString("A"))
+					createObject(type_attr);
+				 /* In case the type is an array one we should use the oid held by "element" attribute to
+				 create the type related to current one */
+				else if(elem_tp_oid > catalog.getLastSysObjectOID() &&	types.count(elem_tp_oid))
+					createObject(types[elem_tp_oid]);
+			}
 
 			if(generate_xml)
 			{
