@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2016 - Raphael Araújo e Silva <raphael@pgmodeler.com.br>
+# Copyright 2006-2018 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -29,18 +29,18 @@ ModelDatabaseDiffForm::ModelDatabaseDiffForm(QWidget *parent, Qt::WindowFlags f)
 		sqlcode_txt=PgModelerUiNS::createNumberedTextEditor(sqlcode_wgt);
 		sqlcode_txt->setReadOnly(true);
 
-		htmlitem_del=new HtmlItemDelegate;
+		htmlitem_del=new HtmlItemDelegate(this);
 		output_trw->setItemDelegateForColumn(0, htmlitem_del);
 
-		imported_model=nullptr;
-		import_helper=nullptr;
+		imported_model=loaded_model=source_model=nullptr;
+		src_import_helper=import_helper=nullptr;
 		diff_helper=nullptr;
 		export_helper=nullptr;
-		import_thread=diff_thread=export_thread=nullptr;
-		import_item=diff_item=export_item=nullptr;
+		src_import_thread=import_thread=diff_thread=export_thread=nullptr;
+		src_import_item=import_item=diff_item=export_item=nullptr;
 		export_conn=nullptr;
 		process_paused=false;
-		diff_progress=0;
+		diff_progress=curr_step=total_steps=0;
 
 		apply_on_server_ht=new HintTextWidget(apply_on_server_hint, this);
 		apply_on_server_ht->setText(apply_on_server_rb->statusTip());
@@ -88,8 +88,14 @@ ModelDatabaseDiffForm::ModelDatabaseDiffForm(QWidget *parent, Qt::WindowFlags f)
 		preserve_db_name_ht=new HintTextWidget(preserve_db_name_hint, this);
 		preserve_db_name_ht->setText(preserve_db_name_chk->statusTip());
 
-		keep_not_imported_objs_ht=new HintTextWidget(keep_not_imported_objs_hint, this);
-		keep_not_imported_objs_ht->setText(keep_not_imported_objs_chk->statusTip());
+		dont_drop_missing_objs_ht=new HintTextWidget(dont_drop_missing_objs_hint, this);
+		dont_drop_missing_objs_ht->setText(dont_drop_missing_objs_chk->statusTip());
+
+		drop_missing_cols_constr_ht=new HintTextWidget(drop_missing_cols_constr_hint, this);
+		drop_missing_cols_constr_ht->setText(drop_missing_cols_constr_chk->statusTip());
+
+		ignore_error_codes_ht=new HintTextWidget(ignore_extra_errors_hint, this);
+		ignore_error_codes_ht->setText(ignore_error_codes_chk->statusTip());
 
 		sqlcode_hl=new SyntaxHighlighter(sqlcode_txt);
 		sqlcode_hl->loadConfiguration(GlobalAttributes::SQL_HIGHLIGHT_CONF_PATH);
@@ -98,7 +104,7 @@ ModelDatabaseDiffForm::ModelDatabaseDiffForm(QWidget *parent, Qt::WindowFlags f)
 
 		PgModelerUiNS::configureWidgetFont(message_lbl, PgModelerUiNS::MEDIUM_FONT_FACTOR);
 
-		connect(cancel_btn, &QToolButton::clicked, [=](){ cancelOperation(true); });
+		connect(cancel_btn, &QToolButton::clicked, [&](){ cancelOperation(true); });
 		connect(pgsql_ver_chk, SIGNAL(toggled(bool)), pgsql_ver_cmb, SLOT(setEnabled(bool)));
 		connect(connections_cmb, SIGNAL(activated(int)), this, SLOT(listDatabases()));
 		connect(store_in_file_rb, SIGNAL(clicked()), this, SLOT(enableDiffMode()));
@@ -111,16 +117,27 @@ ModelDatabaseDiffForm::ModelDatabaseDiffForm(QWidget *parent, Qt::WindowFlags f)
 		connect(select_file_tb, SIGNAL(clicked()), this, SLOT(selectOutputFile()));
 		connect(file_edt, SIGNAL(textChanged(QString)), this, SLOT(enableDiffMode()));
 		connect(force_recreation_chk, SIGNAL(toggled(bool)), recreate_unmod_chk, SLOT(setEnabled(bool)));
+		connect(dont_drop_missing_objs_chk, SIGNAL(toggled(bool)), drop_missing_cols_constr_chk, SLOT(setEnabled(bool)));
 		connect(create_tb, SIGNAL(toggled(bool)), this, SLOT(filterDiffInfos()));
 		connect(drop_tb, SIGNAL(toggled(bool)), this, SLOT(filterDiffInfos()));
 		connect(alter_tb, SIGNAL(toggled(bool)), this, SLOT(filterDiffInfos()));
 		connect(ignore_tb, SIGNAL(toggled(bool)), this, SLOT(filterDiffInfos()));
+		connect(ignore_error_codes_chk, SIGNAL(toggled(bool)), error_codes_edt, SLOT(setEnabled(bool)));
+		connect(src_database_rb, SIGNAL(toggled(bool)), src_database_wgt, SLOT(setEnabled(bool)));
+		connect(src_model_rb, SIGNAL(toggled(bool)), src_model_name_lbl, SLOT(setEnabled(bool)));
+		connect(src_connections_cmb, SIGNAL(activated(int)), this, SLOT(listDatabases()));
+		connect(src_database_cmb, SIGNAL(currentIndexChanged(int)), this, SLOT(enableDiffMode()));
+		connect(src_model_rb, SIGNAL(toggled(bool)), this, SLOT(enableDiffMode()));
+		connect(src_database_rb, SIGNAL(toggled(bool)), this, SLOT(enableDiffMode()));
 
 #ifdef DEMO_VERSION
 	#warning "DEMO VERSION: forcing ignore errors in diff due to the object count limit."
 	ignore_errors_chk->setChecked(true);
 	ignore_errors_chk->setEnabled(false);
 
+	ignore_error_codes_chk->setChecked(false);
+	ignore_error_codes_chk->setEnabled(false);
+
 	apply_on_server_rb->setChecked(false);
 	apply_on_server_rb->setEnabled(false);
 #endif
@@ -139,16 +156,33 @@ ModelDatabaseDiffForm::~ModelDatabaseDiffForm(void)
 	destroyModel();
 }
 
-void ModelDatabaseDiffForm::setDatabaseModel(DatabaseModel *model)
+void ModelDatabaseDiffForm::setModelWidget(ModelWidget *model_wgt)
 {
-	source_model=model;
+	if(model_wgt)
+	{
+		source_model=loaded_model=model_wgt->getDatabaseModel();
+		src_model_name_lbl->setText(QString("%1 [%2]").arg(source_model->getName()).arg(QFileInfo(model_wgt->getFilename()).fileName()));
+		src_model_name_lbl->setToolTip(model_wgt->getFilename().isEmpty() ? trUtf8("model not saved yet") : model_wgt->getFilename());
+	}
+	else
+	{
+		src_model_name_lbl->setText(trUtf8("(none)"));
+		src_model_name_lbl->setToolTip("");
+		src_database_rb->setChecked(true);
+		src_model_rb->setEnabled(false);
+	}
 }
 
 void ModelDatabaseDiffForm::resetForm(void)
 {
+	ConnectionsConfigWidget::fillConnectionsComboBox(src_connections_cmb, true);
+	src_connections_cmb->setEnabled(src_connections_cmb->count() > 0);
+	src_connection_lbl->setEnabled(src_connections_cmb->isEnabled());
+
 	ConnectionsConfigWidget::fillConnectionsComboBox(connections_cmb, true, Connection::OP_DIFF);
 	connections_cmb->setEnabled(connections_cmb->count() > 0);
 	connection_lbl->setEnabled(connections_cmb->isEnabled());
+
 	enableDiffMode();
 	settings_tbw->setTabEnabled(1, false);
 	settings_tbw->setTabEnabled(2, false);
@@ -174,12 +208,27 @@ void ModelDatabaseDiffForm::showEvent(QShowEvent *)
 
 void ModelDatabaseDiffForm::createThread(unsigned thread_id)
 {
-	if(thread_id==IMPORT_THREAD)
+	if(thread_id==SRC_IMPORT_THREAD)
+	{
+		src_import_thread=new QThread;
+		src_import_helper=new DatabaseImportHelper;
+		src_import_helper->moveToThread(src_import_thread);
+
+		output_trw->setUniformRowHeights(true);
+
+		connect(src_import_thread, SIGNAL(started(void)), src_import_helper, SLOT(importDatabase()));
+		connect(src_import_helper, SIGNAL(s_progressUpdated(int,QString,ObjectType)), this, SLOT(updateProgress(int,QString,ObjectType)), Qt::BlockingQueuedConnection);
+		connect(src_import_helper, SIGNAL(s_importFinished(Exception)), this, SLOT(handleImportFinished(Exception)));
+		connect(src_import_helper, SIGNAL(s_importAborted(Exception)), this, SLOT(captureThreadError(Exception)));
+	}
+	else if(thread_id==IMPORT_THREAD)
 	{
 		import_thread=new QThread;
 		import_helper=new DatabaseImportHelper;
 		import_helper->moveToThread(import_thread);
 
+		output_trw->setUniformRowHeights(true);
+
 		connect(import_thread, SIGNAL(started(void)), import_helper, SLOT(importDatabase()));
 		connect(import_helper, SIGNAL(s_progressUpdated(int,QString,ObjectType)), this, SLOT(updateProgress(int,QString,ObjectType)), Qt::BlockingQueuedConnection);
 		connect(import_helper, SIGNAL(s_importFinished(Exception)), this, SLOT(handleImportFinished(Exception)));
@@ -205,10 +254,18 @@ void ModelDatabaseDiffForm::createThread(unsigned thread_id)
 		export_helper->moveToThread(export_thread);
 
 		connect(apply_on_server_btn, &QPushButton::clicked,
-				[=](){ apply_on_server_btn->setEnabled(false);
-			exportDiff(false); });
+			[&](){
+						apply_on_server_btn->setEnabled(false);
+						if(!export_thread->isRunning())
+							exportDiff(false);
+			});
 
 		connect(export_thread, SIGNAL(started(void)), export_helper, SLOT(exportToDBMS()));
+
+		connect(export_thread, &QThread::finished, [&](){
+			output_trw->setUniformRowHeights(false);
+		});
+
 		connect(export_helper, SIGNAL(s_progressUpdated(int,QString,ObjectType,QString)), this, SLOT(updateProgress(int,QString,ObjectType,QString)), Qt::BlockingQueuedConnection);
 		connect(export_helper, SIGNAL(s_errorIgnored(QString,QString, QString)), this, SLOT(handleErrorIgnored(QString,QString,QString)));
 		connect(export_helper, SIGNAL(s_exportFinished()), this, SLOT(handleExportFinished()));
@@ -218,7 +275,14 @@ void ModelDatabaseDiffForm::createThread(unsigned thread_id)
 
 void ModelDatabaseDiffForm::destroyThread(unsigned thread_id)
 {
-	if(thread_id==IMPORT_THREAD && import_thread)
+	if(thread_id==SRC_IMPORT_THREAD && src_import_thread)
+	{
+		delete(src_import_thread);
+		delete(src_import_helper);
+		src_import_thread=nullptr;
+		src_import_helper=nullptr;
+	}
+	else if(thread_id==IMPORT_THREAD && import_thread)
 	{
 		delete(import_thread);
 		delete(import_helper);
@@ -252,13 +316,19 @@ void ModelDatabaseDiffForm::destroyModel(void)
 	if(imported_model)
 		delete(imported_model);
 
+	if(source_model && source_model != loaded_model && src_database_rb->isChecked())
+	{
+		delete(source_model);
+		source_model = nullptr;
+	}
+
 	imported_model=nullptr;
 }
 
 void ModelDatabaseDiffForm::clearOutput(void)
 {
 	output_trw->clear();
-	import_item=diff_item=export_item=nullptr;
+	src_import_item=import_item=diff_item=export_item=nullptr;
 
 	step_lbl->setText(trUtf8("Waiting process to start..."));
 	step_ico_lbl->setPixmap(QPixmap());
@@ -276,35 +346,38 @@ void ModelDatabaseDiffForm::clearOutput(void)
 
 void ModelDatabaseDiffForm::listDatabases(void)
 {
+	QComboBox *conn_cmb = (sender() == src_connections_cmb ? src_connections_cmb : connections_cmb),
+			*db_cmb = (conn_cmb == src_connections_cmb ? src_database_cmb : database_cmb);
+	QLabel *db_lbl = (conn_cmb == src_connections_cmb ? src_database_lbl : database_lbl);
+
 	try
 	{
-		if(connections_cmb->currentIndex()==connections_cmb->count()-1 &&
-				ConnectionsConfigWidget::openConnectionsConfiguration(connections_cmb, true))
+		if(conn_cmb->currentIndex()==conn_cmb->count()-1 &&
+			 ConnectionsConfigWidget::openConnectionsConfiguration(conn_cmb, true))
 		{
 			emit s_connectionsUpdateRequest();
+			resetForm();
+		}
+
+		Connection *conn=reinterpret_cast<Connection *>(conn_cmb->itemData(conn_cmb->currentIndex()).value<void *>());
+
+		if(conn)
+		{
+			DatabaseImportHelper imp_helper;
+			imp_helper.setConnection(*conn);
+			DatabaseImportForm::listDatabases(imp_helper, db_cmb);
 		}
 		else
-		{
-			Connection *conn=reinterpret_cast<Connection *>(connections_cmb->itemData(connections_cmb->currentIndex()).value<void *>());
+			db_cmb->clear();
 
-			if(conn)
-			{
-				DatabaseImportHelper imp_helper;
-				imp_helper.setConnection(*conn);
-				DatabaseImportForm::listDatabases(imp_helper, database_cmb);
-			}
-			else
-				database_cmb->clear();
-
-			database_cmb->setEnabled(database_cmb->count() > 0);
-			database_lbl->setEnabled(database_cmb->isEnabled());
-		}
+		db_cmb->setEnabled(db_cmb->count() > 0);
+		db_lbl->setEnabled(db_cmb->isEnabled());
 	}
 	catch(Exception &e)
 	{
-		database_cmb->clear();
-		database_cmb->setEnabled(false);
-		database_lbl->setEnabled(false);
+		db_cmb->clear();
+		db_cmb->setEnabled(false);
+		db_lbl->setEnabled(false);
 		throw Exception(e.getErrorMessage(), e.getErrorType(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
 	}
 }
@@ -314,20 +387,33 @@ void ModelDatabaseDiffForm::enableDiffMode(void)
 	store_in_file_wgt->setEnabled(store_in_file_rb->isChecked());
 
 	generate_btn->setEnabled(database_cmb->currentIndex() > 0 &&
-							 ((store_in_file_rb->isChecked() && !file_edt->text().isEmpty()) ||
-							  (apply_on_server_rb->isChecked())));
+													 ((src_database_rb->isChecked() && src_database_cmb->currentIndex() > 0) ||
+														(src_model_rb->isChecked() && loaded_model)) &&
+													 ((store_in_file_rb->isChecked() && !file_edt->text().isEmpty()) ||
+														(apply_on_server_rb->isChecked())));
 }
 
 void ModelDatabaseDiffForm::generateDiff(void)
 {
 	//Destroy previously allocated threads and helper before start over.
 	destroyModel();
+	destroyThread(SRC_IMPORT_THREAD);
 	destroyThread(IMPORT_THREAD);
 	destroyThread(DIFF_THREAD);
 	destroyThread(EXPORT_THREAD);
 
 	clearOutput();
-	importDatabase();
+	curr_step = 1;
+
+	if(src_model_rb->isChecked())
+	{
+		source_model = loaded_model;
+		total_steps=3;
+	}
+	else
+		total_steps=4;
+
+	importDatabase(src_database_rb->isChecked() ? SRC_IMPORT_THREAD : IMPORT_THREAD);
 
 	buttons_wgt->setEnabled(false);
 	cancel_btn->setEnabled(true);
@@ -339,24 +425,39 @@ void ModelDatabaseDiffForm::generateDiff(void)
 	settings_tbw->setCurrentIndex(1);
 }
 
-void ModelDatabaseDiffForm::importDatabase(void)
+void ModelDatabaseDiffForm::importDatabase(unsigned thread_id)
 {
 	try
 	{
-		createThread(IMPORT_THREAD);
+		if(thread_id != SRC_IMPORT_THREAD && thread_id != IMPORT_THREAD)
+			throw Exception(ERR_ALOC_OBJECT_INV_TYPE,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 
-		Connection conn=(*reinterpret_cast<Connection *>(connections_cmb->itemData(connections_cmb->currentIndex()).value<void *>())), conn1;
+		createThread(thread_id);
+
+		QThread *thread = (thread_id == SRC_IMPORT_THREAD ? src_import_thread : import_thread);
+		DatabaseImportHelper *import_hlp = (thread_id == SRC_IMPORT_THREAD ? src_import_helper : import_helper);
+		QComboBox *conn_cmb = (thread_id == SRC_IMPORT_THREAD ? src_connections_cmb : connections_cmb),
+				*db_cmb = (thread_id == SRC_IMPORT_THREAD ? src_database_cmb : database_cmb);
+		Connection conn=(*reinterpret_cast<Connection *>(conn_cmb->itemData(conn_cmb->currentIndex()).value<void *>())), conn1;
 		map<ObjectType, vector<unsigned>> obj_oids;
 		map<unsigned, vector<unsigned>> col_oids;
 		Catalog catalog;
+		DatabaseModel *db_model = nullptr;
 
 		conn1=conn;
-		step_lbl->setText(trUtf8("Importing database <strong>%1</strong>...").arg(database_cmb->currentText()));
-		step_ico_lbl->setPixmap(QPixmap(QString(":/icones/icones/import.png")));
+		step_lbl->setText(trUtf8("Step %1/%2: Importing database <strong>%3</strong>...")
+											.arg(curr_step)
+											.arg(total_steps)
+											.arg(db_cmb->currentText()));
 
-		import_item=PgModelerUiNS::createOutputTreeItem(output_trw, step_lbl->text(), *step_ico_lbl->pixmap(), nullptr);
+		step_ico_lbl->setPixmap(QPixmap(PgModelerUiNS::getIconPath("import")));
 
-		conn.switchToDatabase(database_cmb->currentText());
+		if(thread_id == SRC_IMPORT_THREAD)
+			src_import_item=PgModelerUiNS::createOutputTreeItem(output_trw, step_lbl->text(), *step_ico_lbl->pixmap(), nullptr);
+		else
+			import_item=PgModelerUiNS::createOutputTreeItem(output_trw, step_lbl->text(), *step_ico_lbl->pixmap(), nullptr);
+
+		conn.switchToDatabase(db_cmb->currentText());
 		pgsql_ver=conn.getPgSQLVersion(true);
 
 		catalog.setConnection(conn);
@@ -365,18 +466,27 @@ void ModelDatabaseDiffForm::importDatabase(void)
 		catalog.setFilter(Catalog::LIST_ALL_OBJS | Catalog::EXCL_BUILTIN_ARRAY_TYPES |
 						  Catalog::EXCL_EXTENSION_OBJS | Catalog::EXCL_SYSTEM_OBJS);
 		catalog.getObjectsOIDs(obj_oids, col_oids, {{ParsersAttributes::FILTER_TABLE_TYPES, ParsersAttributes::_TRUE_}});
-		obj_oids[OBJ_DATABASE].push_back(database_cmb->currentData().value<unsigned>());
+		obj_oids[OBJ_DATABASE].push_back(db_cmb->currentData().value<unsigned>());
 
-		imported_model=new DatabaseModel;
-		imported_model->createSystemObjects(true);
+		if(thread_id == SRC_IMPORT_THREAD)
+		{
+			source_model=new DatabaseModel;
+			source_model->createSystemObjects(true);
+			db_model = source_model;
+		}
+		else
+		{
+			imported_model=new DatabaseModel;
+			imported_model->createSystemObjects(true);
+			db_model = imported_model;
+		}
 
-		import_helper->setConnection(conn1);
-		import_helper->setSelectedOIDs(imported_model, obj_oids, col_oids);
-		import_helper->setCurrentDatabase(database_cmb->currentText());
-		import_helper->setImportOptions(import_sys_objs_chk->isChecked(), import_ext_objs_chk->isChecked(), true,
+		import_hlp->setConnection(conn1);
+		import_hlp->setSelectedOIDs(db_model, obj_oids, col_oids);
+		import_hlp->setCurrentDatabase(db_cmb->currentText());
+		import_hlp->setImportOptions(import_sys_objs_chk->isChecked(), import_ext_objs_chk->isChecked(), true,
 										ignore_errors_chk->isChecked(), false, false, false);
-
-		import_thread->start();
+		thread->start();
 	}
 	catch(Exception &e)
 	{
@@ -388,10 +498,12 @@ void ModelDatabaseDiffForm::diffModels(void)
 {
 	createThread(DIFF_THREAD);
 
-	step_lbl->setText(trUtf8("Comparing the model <strong>%1</strong> and database <strong>%2</strong>...")
+	step_lbl->setText(trUtf8("Step %1/%2: Comparing <strong>%3</strong> and <strong>%4</strong>...")
+						.arg(curr_step)
+						.arg(total_steps)
 					  .arg(source_model->getName())
 					  .arg(imported_model->getName()));
-	step_ico_lbl->setPixmap(QPixmap(QString(":/icones/icones/diff.png")));
+	step_ico_lbl->setPixmap(QPixmap(PgModelerUiNS::getIconPath("diff")));
 
 	output_trw->collapseItem(import_item);
 	diff_progress=step_pb->value();
@@ -406,7 +518,8 @@ void ModelDatabaseDiffForm::diffModels(void)
 	diff_helper->setDiffOption(ModelsDiffHelper::OPT_KEEP_OBJ_PERMS, keep_obj_perms_chk->isChecked());
 	diff_helper->setDiffOption(ModelsDiffHelper::OPT_REUSE_SEQUENCES, reuse_sequences_chk->isChecked());
 	diff_helper->setDiffOption(ModelsDiffHelper::OPT_PRESERVE_DB_NAME, preserve_db_name_chk->isChecked());
-	diff_helper->setDiffOption(ModelsDiffHelper::OPT_KEEP_NOT_IMPORTED_OBJS, keep_not_imported_objs_chk->isChecked());
+	diff_helper->setDiffOption(ModelsDiffHelper::OPT_DONT_DROP_MISSING_OBJS, dont_drop_missing_objs_chk->isChecked());
+	diff_helper->setDiffOption(ModelsDiffHelper::OPT_DROP_MISSING_COLS_CONSTR, drop_missing_cols_constr_chk->isChecked());
 
 	diff_helper->setModels(source_model, imported_model);
 
@@ -429,16 +542,18 @@ void ModelDatabaseDiffForm::exportDiff(bool confirm)
 					 trUtf8(" <strong>WARNING:</strong> The generated diff is ready to be exported! Once started this process will cause irreversible changes on the database. Do you really want to proceed?"),
 					 Messagebox::ALERT_ICON, Messagebox::ALL_BUTTONS,
 					 trUtf8("Apply diff"), trUtf8("Preview diff"), QString(),
-					 QString(":/icones/icones/diff.png"), QString(":/icones/icones/codigosql.png"));
+					 PgModelerUiNS::getIconPath("diff"), PgModelerUiNS::getIconPath("codigosql"));
 
 	if(!confirm || msg_box.result()==QDialog::Accepted)
 	{
 		settings_tbw->setCurrentIndex(1);
 		apply_on_server_btn->setEnabled(true);
 
-		step_lbl->setText(trUtf8("Exporting diff to database <strong>%1</strong>...")
-						  .arg(imported_model->getName()));
-		step_ico_lbl->setPixmap(QPixmap(QString(":/icones/icones/exportar.png")));
+		step_lbl->setText(trUtf8("Step %1/%2: Exporting diff to database <strong>%3</strong>...")
+											.arg(curr_step)
+											.arg(total_steps)
+											.arg(imported_model->getName()));
+		step_ico_lbl->setPixmap(QPixmap(PgModelerUiNS::getIconPath("exportar")));
 
 		output_trw->collapseItem(diff_item);
 		diff_progress=step_pb->value();
@@ -447,7 +562,10 @@ void ModelDatabaseDiffForm::exportDiff(bool confirm)
 		export_conn=new Connection;
 		*export_conn=*reinterpret_cast<Connection *>(connections_cmb->itemData(connections_cmb->currentIndex()).value<void *>());
 		export_helper->setExportToDBMSParams(sqlcode_txt->toPlainText(), export_conn,
-											 database_cmb->currentText(), ignore_duplic_chk->isChecked());
+																				 database_cmb->currentText(), ignore_duplic_chk->isChecked());
+		if(ignore_error_codes_chk->isChecked())
+			export_helper->setIgnoredErrors(error_codes_edt->text().simplified().split(' '));
+
 		export_thread->start();
 	}
 	else if(msg_box.isCancelled())
@@ -460,7 +578,7 @@ void ModelDatabaseDiffForm::exportDiff(bool confirm)
 		output_trw->collapseItem(diff_item);
 		PgModelerUiNS::createOutputTreeItem(output_trw,
 											trUtf8("Diff process paused. Waiting user action..."),
-											QPixmap(QString(":/icones/icones/msgbox_alerta.png")), nullptr);
+											QPixmap(PgModelerUiNS::getIconPath("msgbox_alerta")), nullptr);
 	}
 }
 
@@ -495,7 +613,7 @@ void ModelDatabaseDiffForm::saveDiffToFile(void)
 		QFile output;
 
 		step_lbl->setText(trUtf8("Saving diff to file <strong>%1</strong>").arg(file_edt->text()));
-		step_ico_lbl->setPixmap(QPixmap(QString(":/icones/icones/salvar.png")));
+		step_ico_lbl->setPixmap(QPixmap(PgModelerUiNS::getIconPath("salvar")));
 		import_item=PgModelerUiNS::createOutputTreeItem(output_trw, step_lbl->text(), *step_ico_lbl->pixmap(), nullptr);
 		step_pb->setValue(90);
 		progress_pb->setValue(100);
@@ -520,8 +638,8 @@ void ModelDatabaseDiffForm::finishDiff(void)
 	step_lbl->setText(trUtf8("Diff process sucessfully ended!"));
 	progress_lbl->setText(trUtf8("No operations left."));
 
-	step_ico_lbl->setPixmap(QPixmap(QString(":/icones/icones/msgbox_info.png")));
-	progress_ico_lbl->setPixmap(QPixmap(QString(":/icones/icones/msgbox_info.png")));
+	step_ico_lbl->setPixmap(QPixmap(PgModelerUiNS::getIconPath("msgbox_info")));
+	progress_ico_lbl->setPixmap(QPixmap(PgModelerUiNS::getIconPath("msgbox_info")));
 
 	import_item=PgModelerUiNS::createOutputTreeItem(output_trw, step_lbl->text(), *step_ico_lbl->pixmap(), nullptr);
 	step_pb->setValue(100);
@@ -535,12 +653,18 @@ void ModelDatabaseDiffForm::cancelOperation(bool cancel_by_user)
 		step_lbl->setText(trUtf8("Operation cancelled by the user."));
 		progress_lbl->setText(trUtf8("No operations left."));
 
-		step_ico_lbl->setPixmap(QPixmap(QString(":/icones/icones/msgbox_alerta.png")));
-		progress_ico_lbl->setPixmap(QPixmap(QString(":/icones/icones/msgbox_alerta.png")));
+		step_ico_lbl->setPixmap(QPixmap(PgModelerUiNS::getIconPath("msgbox_alerta")));
+		progress_ico_lbl->setPixmap(QPixmap(PgModelerUiNS::getIconPath("msgbox_alerta")));
 
 		PgModelerUiNS::createOutputTreeItem(output_trw, step_lbl->text(), *step_ico_lbl->pixmap(), nullptr);
 	}
 
+	if(src_import_helper && src_import_thread->isRunning())
+	{
+		src_import_helper->cancelImport();
+		src_import_thread->quit();
+	}
+
 	if(import_helper && import_thread->isRunning())
 	{
 		import_helper->cancelImport();
@@ -569,12 +693,10 @@ void ModelDatabaseDiffForm::captureThreadError(Exception e)
 
 	cancelOperation(false);
 	progress_lbl->setText(trUtf8("Process aborted due to errors!"));
-	progress_ico_lbl->setPixmap(QPixmap(QString(":/icones/icones/msgbox_erro.png")));
+	progress_ico_lbl->setPixmap(QPixmap(PgModelerUiNS::getIconPath("msgbox_erro")));
 
-	item=PgModelerUiNS::createOutputTreeItem(output_trw, PgModelerUiNS::formatMessage(e.getErrorMessage()), *progress_ico_lbl->pixmap(), nullptr, true, true);
-
-	if(!e.getExtraInfo().isEmpty())
-		PgModelerUiNS::createOutputTreeItem(output_trw, PgModelerUiNS::formatMessage(e.getExtraInfo()), *progress_ico_lbl->pixmap(), item, true, true);
+	item=PgModelerUiNS::createOutputTreeItem(output_trw, PgModelerUiNS::formatMessage(e.getErrorMessage()), *progress_ico_lbl->pixmap(), nullptr, false, true);
+	PgModelerUiNS::createExceptionsTree(output_trw, e, item);
 
 	throw Exception(e.getErrorMessage(), e.getErrorType(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
 }
@@ -587,13 +709,24 @@ void ModelDatabaseDiffForm::handleImportFinished(Exception e)
 		msgbox.show(e, e.getErrorMessage(), Messagebox::ALERT_ICON);
 	}
 
-	import_thread->quit();
-	step_pb->setValue(30);
-	diffModels();
+	curr_step++;
+
+	if(src_import_thread && src_import_thread->isRunning())
+	{
+		src_import_thread->quit();
+		src_import_item->setExpanded(false);
+		importDatabase(IMPORT_THREAD);
+	}
+	else
+	{
+		import_thread->quit();
+		diffModels();
+	}
 }
 
 void ModelDatabaseDiffForm::handleDiffFinished(void)
 {
+	curr_step++;
 	sqlcode_txt->setPlainText(diff_helper->getDiffDefinition());
 
 #ifdef DEMO_VERSION
@@ -634,11 +767,11 @@ void ModelDatabaseDiffForm::handleErrorIgnored(QString err_code, QString err_msg
 	QTreeWidgetItem *item=nullptr;
 
 	item=PgModelerUiNS::createOutputTreeItem(output_trw, trUtf8("Error code <strong>%1</strong> found and ignored. Proceeding with export.").arg(err_code),
-											 QPixmap(QString(":/icones/icones/msgbox_alerta.png")),
+											 QPixmap(PgModelerUiNS::getIconPath("msgbox_alerta")),
 											 export_item, false);
 
 	PgModelerUiNS::createOutputTreeItem(output_trw, PgModelerUiNS::formatMessage(err_msg),
-										QPixmap(QString(":/icones/icones/msgbox_alerta.png")),
+										QPixmap(QString("msgbox_alerta")),
 										item, false);
 
 	PgModelerUiNS::createOutputTreeItem(output_trw, cmd,
@@ -648,15 +781,28 @@ void ModelDatabaseDiffForm::handleErrorIgnored(QString err_code, QString err_msg
 
 void ModelDatabaseDiffForm::updateProgress(int progress, QString msg, ObjectType obj_type, QString cmd)
 {
+	int progress_aux = 0;
+
 	msg=PgModelerUiNS::formatMessage(msg);
 
-	if(import_thread && import_thread->isRunning())
+
+	if(src_import_thread && src_import_thread->isRunning())
 	{
-		if(progress > 90)
-			step_pb->setValue(step_pb->value() + 5);
+		progress_aux = progress/5;
 
 		PgModelerUiNS::createOutputTreeItem(output_trw, msg,
-											QPixmap(QString(":/icones/icones/") + BaseObject::getSchemaName(obj_type) + QString(".png")),
+											QPixmap(PgModelerUiNS::getIconPath(obj_type)),
+											src_import_item);
+	}
+	else if(import_thread && import_thread->isRunning())
+	{
+		if(src_model_rb->isChecked())
+			progress_aux = progress/4;
+		else
+			progress_aux = 20 + (progress/5);
+
+		PgModelerUiNS::createOutputTreeItem(output_trw, msg,
+											QPixmap(PgModelerUiNS::getIconPath(obj_type)),
 											import_item);
 	}
 	else if(diff_thread && diff_thread->isRunning())
@@ -664,22 +810,23 @@ void ModelDatabaseDiffForm::updateProgress(int progress, QString msg, ObjectType
 		if((progress == 0 || progress == 100) && obj_type==BASE_OBJECT)
 		{
 			PgModelerUiNS::createOutputTreeItem(output_trw, msg,
-												QPixmap(QString(":/icones/icones/msgbox_info.png")),
+												QPixmap(PgModelerUiNS::getIconPath("msgbox_info")),
 												diff_item);
 		}
 
-		step_pb->setValue(diff_progress + (progress/3));
+		progress_aux = diff_progress + (progress/3);
 	}
 	else if(export_thread && export_thread->isRunning())
 	{
 		QTreeWidgetItem *item=nullptr;
 		QPixmap ico;
-		step_pb->setValue(diff_progress + (progress/3));
+
+		progress_aux = diff_progress + (progress/3);
 
 		if(obj_type==BASE_OBJECT)
-			ico=QPixmap(QString(":/icones/icones/codigosql.png"));
+			ico=QPixmap(PgModelerUiNS::getIconPath("codigosql"));
 		else
-			ico=QPixmap(QString(":/icones/icones/") + BaseObject::getSchemaName(obj_type) + QString(".png"));
+			ico=QPixmap(PgModelerUiNS::getIconPath(obj_type));
 
 		item=PgModelerUiNS::createOutputTreeItem(output_trw, msg, ico, export_item, false);
 
@@ -687,13 +834,16 @@ void ModelDatabaseDiffForm::updateProgress(int progress, QString msg, ObjectType
 			PgModelerUiNS::createOutputTreeItem(output_trw, cmd, QPixmap(), item, false);
 	}
 
+	if(progress_aux > step_pb->value())
+		step_pb->setValue(progress_aux);
+
 	progress_lbl->setText(msg);
 	progress_pb->setValue(progress);
 
 	if(obj_type!=BASE_OBJECT)
-		progress_ico_lbl->setPixmap(QPixmap(QString(":/icones/icones/") + BaseObject::getSchemaName(obj_type) + QString(".png")));
+		progress_ico_lbl->setPixmap(QPixmap(PgModelerUiNS::getIconPath(obj_type)));
 	else
-		progress_ico_lbl->setPixmap(QPixmap(QString(":/icones/icones/msgbox_info.png")));
+		progress_ico_lbl->setPixmap(QPixmap(PgModelerUiNS::getIconPath("msgbox_info")));
 
 	this->repaint();
 }
@@ -711,7 +861,7 @@ void ModelDatabaseDiffForm::updateDiffInfo(ObjectsDiffInfo diff_info)
 
 	item=PgModelerUiNS::createOutputTreeItem(output_trw,
 											 PgModelerUiNS::formatMessage(diff_info.getInfoMessage()),
-											 QPixmap(QString(":/icones/icones/%1.png").arg(diff_info.getObject()->getSchemaName())), diff_item);
+											 QPixmap(PgModelerUiNS::getIconPath(diff_info.getObject()->getSchemaName())), diff_item);
 	item->setData(0, Qt::UserRole, diff_info.getDiffType());
 
 	if(diff_helper)
@@ -729,7 +879,9 @@ void ModelDatabaseDiffForm::selectOutputFile(void)
 	file_dlg.setAcceptMode(QFileDialog::AcceptSave);
 	file_dlg.setModal(true);
 	file_dlg.setNameFilter(trUtf8("SQL code (*.sql);;All files (*.*)"));
-	file_dlg.selectFile(source_model->getName() + QString("-diff.sql"));
+
+	if(source_model)
+		file_dlg.selectFile(source_model->getName() + QString("-diff.sql"));
 
 	if(file_dlg.exec()==QFileDialog::Accepted)
 	{
@@ -741,4 +893,3 @@ void ModelDatabaseDiffForm::selectOutputFile(void)
 		file_edt->setText(file);
 	}
 }
-
