@@ -31,10 +31,32 @@ void ModelExportHelper::abortExport(Exception &e)
 		throw Exception(e.getErrorMessage(),e.getErrorType(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
 }
 
+void ModelExportHelper::handleSQLError(Exception &e, const QString &sql_cmd, bool ignore_dup)
+{
+	//Ignoring the error if it is in the ignored list
+	if(ignored_errors.indexOf(e.getExtraInfo()) >= 0 ||
+			(ignore_dup && isDuplicationError(e.getExtraInfo())))
+		emit s_errorIgnored(e.getExtraInfo(), e.getErrorMessage(), sql_cmd);
+	//Raises an excpetion if the error returned by the database is not listed in the ignored list of errors
+	else if(ignored_errors.indexOf(e.getExtraInfo()) < 0)
+		throw Exception(e.getErrorMessage(), e.getErrorType(),__PRETTY_FUNCTION__,__FILE__,__LINE__,&e, sql_cmd);
+	else
+		errors.push_back(e);
+}
+
 void ModelExportHelper::setIgnoredErrors(const QStringList &err_codes)
 {
-	ignored_errors=err_codes;
-	ignored_errors.removeDuplicates();
+	QRegExp valid_code = QRegExp("([a-z]|[A-Z]|[0-9])+");
+	QStringList error_codes=err_codes;
+
+	ignored_errors.clear();
+	error_codes.removeDuplicates();
+
+	for(QString code : error_codes)
+	{
+		if(valid_code.exactMatch(code))
+			ignored_errors.push_back(code);
+	}
 }
 
 void ModelExportHelper::exportToSQL(DatabaseModel *db_model, const QString &filename, const QString &pgsql_ver)
@@ -225,7 +247,7 @@ void ModelExportHelper::exportToSVG(ObjectsScene *scene, const QString &filename
 
 	bool shw_dlm=false, shw_grd=false, align_objs=false;
 	QSvgGenerator svg_gen;
-	QRectF scene_rect=scene->itemsBoundingRect();
+	QRectF scene_rect=scene->itemsBoundingRect(true);
 	QFileInfo fi(filename);
 
 	//Making a backup of the current scene options
@@ -239,8 +261,6 @@ void ModelExportHelper::exportToSVG(ObjectsScene *scene, const QString &filename
 	emit s_progressUpdated(0, trUtf8("Exporting model to SVG file."));
 
 	svg_gen.setFileName(filename);
-	svg_gen.setSize(scene_rect.size().toSize());
-	svg_gen.setViewBox(scene_rect.toRect());
 	svg_gen.setTitle(trUtf8("SVG representation of database model"));
 	svg_gen.setDescription(trUtf8("SVG file generated by pgModeler"));
 
@@ -290,12 +310,14 @@ void ModelExportHelper::exportToSVG(ObjectsScene *scene, const QString &filename
 
 void ModelExportHelper::exportToDBMS(DatabaseModel *db_model, Connection conn, const QString &pgsql_ver, bool ignore_dup, bool drop_db, bool drop_objs, bool simulate, bool use_tmp_names)
 {
-	int type_id;
-	QString  version, sql_cmd, buf;
+	int type_id = 0, pos = -1;
+	QString  version, sql_cmd, buf, sql_cmd_comment;
 	Connection new_db_conn;
 	unsigned i, count;
 	ObjectType types[]={OBJ_ROLE, OBJ_TABLESPACE};
 	BaseObject *object=nullptr;
+	QString tmpl_comm_regexp = QString("(COMMENT)( )+(ON)( )+(%1)(.)+(\n)(") + ParsersAttributes::DDL_END_TOKEN + QString(")");
+	QRegExp comm_regexp;
 
 	try
 	{
@@ -367,7 +389,18 @@ void ModelExportHelper::exportToDBMS(DatabaseModel *db_model, Connection conn, c
 		if(drop_db)
 		{
 			emit s_progressUpdated(progress, trUtf8("Trying to drop database `%1'.").arg(db_model->getName()));
-			conn.executeDDLCommand(QString("DROP DATABASE IF EXISTS %1;").arg(db_model->getName(true)));
+
+			try
+			{
+				sql_cmd = QString("DROP DATABASE IF EXISTS %1;").arg(db_model->getName(true));
+				conn.executeDDLCommand(sql_cmd);
+			}
+			catch(Exception &e)
+			{
+				handleSQLError(e, sql_cmd, ignore_dup);
+			}
+
+			sql_cmd.clear();
 		}
 
 		if(simulate)
@@ -395,21 +428,31 @@ void ModelExportHelper::exportToDBMS(DatabaseModel *db_model, Connection conn, c
 											   object->getObjectType());
 
 						sql_cmd=object->getCodeDefinition(SchemaParser::SQL_DEFINITION);
+
+						if(types[type_id] == OBJ_TABLESPACE)
+						{
+							comm_regexp = QRegExp(tmpl_comm_regexp.arg(object->getSQLName()));
+							pos = comm_regexp.indexIn(sql_cmd);
+
+							/* If we find a comment on statement we should strip it from the tablespace definition in
+							 * order to execute it after creating the db */
+							if(pos >= 0)
+							{
+								sql_cmd_comment = sql_cmd.mid(pos, comm_regexp.matchedLength());
+								sql_cmd.remove(pos, comm_regexp.matchedLength());
+								pos = -1;
+							}
+						}
+
 						conn.executeDDLCommand(sql_cmd);
+
+						if(!sql_cmd_comment.isEmpty())
+							conn.executeDDLCommand(sql_cmd_comment);
 					}
 				}
 				catch(Exception &e)
 				{
-					//Ignoring the error if it is in the ignored list
-					if(ignored_errors.indexOf(e.getExtraInfo()) >= 0 ||
-							(ignore_dup && isDuplicationError(e.getExtraInfo())))
-						emit s_errorIgnored(e.getExtraInfo(), e.getErrorMessage(), sql_cmd);
-					//Raises an excpetion if the error returned by the database is not listed in the ignored list of errors
-					else if(ignored_errors.indexOf(e.getExtraInfo()) < 0)
-						throw Exception(e.getErrorMessage(),
-										e.getErrorType(),__PRETTY_FUNCTION__,__FILE__,__LINE__,&e, sql_cmd);
-					else
-						errors.push_back(e);
+					handleSQLError(e, sql_cmd, ignore_dup);
 				}
 
 				created_objs[types[type_id]]++;
@@ -420,6 +463,8 @@ void ModelExportHelper::exportToDBMS(DatabaseModel *db_model, Connection conn, c
 		{
 			if(!db_model->isSQLDisabled() && !export_canceled)
 			{
+				comm_regexp = QRegExp(tmpl_comm_regexp.arg(db_model->getSQLName()));
+
 				//Creating the database on the DBMS
 				emit s_progressUpdated(progress,
 									   trUtf8("Creating database `%1'")
@@ -427,22 +472,26 @@ void ModelExportHelper::exportToDBMS(DatabaseModel *db_model, Connection conn, c
 									   OBJ_DATABASE);
 
 				sql_cmd=db_model->__getCodeDefinition(SchemaParser::SQL_DEFINITION);
+				pos = comm_regexp.indexIn(sql_cmd);
+
+				/* If we find a comment on statment we should strip it from the DB definition in
+				 * order to execute it after creating the db */
+				if(pos >= 0)
+				{
+					sql_cmd_comment = sql_cmd.mid(pos, comm_regexp.matchedLength());
+					sql_cmd.remove(pos, comm_regexp.matchedLength());
+				}
+
 				conn.executeDDLCommand(sql_cmd);
 				db_created=true;
+
+				if(!sql_cmd_comment.isEmpty())
+					conn.executeDDLCommand(sql_cmd_comment);
 			}
 		}
 		catch(Exception &e)
 		{
-			//Ignoring the error if it is in the ignored list
-			if(ignored_errors.indexOf(e.getExtraInfo()) >= 0 ||
-					(ignore_dup && isDuplicationError(e.getExtraInfo())))
-				emit s_errorIgnored(e.getExtraInfo(), e.getErrorMessage(), sql_cmd);
-			//Raises an excpetion if the error returned by the database is not listed in the ignored list of errors
-			else if(ignored_errors.indexOf(e.getExtraInfo()) < 0)
-				throw Exception(e.getErrorMessage(),
-								e.getErrorType(),__PRETTY_FUNCTION__,__FILE__,__LINE__,&e, sql_cmd);
-			else
-				errors.push_back(e);
+			handleSQLError(e, sql_cmd, ignore_dup);
 		}
 
 		if(!export_canceled)
@@ -459,8 +508,7 @@ void ModelExportHelper::exportToDBMS(DatabaseModel *db_model, Connection conn, c
 			progress=30;
 
 			//Creating the other object types
-			emit s_progressUpdated(progress,
-								   trUtf8("Generating SQL for `%1' objects...").arg(db_model->getObjectCount()));
+			emit s_progressUpdated(progress, trUtf8("Generating SQL for `%1' objects...").arg(db_model->getObjectCount()));
 
 			//Exporting the database model definition using the opened connection
 			buf=db_model->getCodeDefinition(SchemaParser::SQL_DEFINITION, false);
@@ -683,7 +731,8 @@ void ModelExportHelper::restoreObjectNames(void)
 
 	/* Invalidates the codes of all objects on database model in order to generate the SQL referencing the
 		 object's with their original names */
-	db_model->setCodesInvalidated();
+	if(db_model)
+		db_model->setCodesInvalidated();
 }
 
 bool ModelExportHelper::isDuplicationError(const QString &error_code)
@@ -710,9 +759,8 @@ void ModelExportHelper::exportBufferToDBMS(const QString &buffer, Connection &co
 {
 	Connection aux_conn;
 	QString sql_buf=buffer, sql_cmd, aux_cmd, lin, msg,
-			obj_name, obj_tp_name, tab_name,
+			obj_name, obj_tp_name, tab_name, orig_conn_db_name,
 			alter_tab=QString("ALTER TABLE");
-	//vector<Exception> errors;
 	vector<QString> db_sql_cmds;
 	QTextStream ts;
 	ObjectType obj_type=BASE_OBJECT;
@@ -728,7 +776,7 @@ void ModelExportHelper::exportBufferToDBMS(const QString &buffer, Connection &co
 			drop_tab_obj_reg(QString("^((\\-\\-)+( )*)+(%1)(.)+(DROP)(.)+").arg(alter_tab)),
 			reg_aux;
 
-	vector<ObjectType> obj_types={ OBJ_ROLE, OBJ_FUNCTION, OBJ_TRIGGER, OBJ_INDEX,
+	vector<ObjectType> obj_types={ OBJ_ROLE, OBJ_FUNCTION, OBJ_TRIGGER, OBJ_INDEX, OBJ_POLICY,
 								   OBJ_RULE,	OBJ_TABLE, OBJ_VIEW, OBJ_DOMAIN,
 								   OBJ_SCHEMA,	OBJ_AGGREGATE, OBJ_OPFAMILY,
 								   OBJ_OPCLASS, OBJ_OPERATOR,  OBJ_SEQUENCE,
@@ -743,6 +791,8 @@ void ModelExportHelper::exportBufferToDBMS(const QString &buffer, Connection &co
 
 	if(!conn.isStablished())
 	{
+		orig_conn_db_name = conn.getConnectionParam(Connection::PARAM_DB_NAME);
+
 		if(!db_name.isEmpty())
 			conn.setConnectionParam(Connection::PARAM_DB_NAME, db_name);
 
@@ -798,9 +848,6 @@ void ModelExportHelper::exportBufferToDBMS(const QString &buffer, Connection &co
 					obj_type=(aux_cmd.contains(QString("COLUMN")) ? OBJ_COLUMN : OBJ_CONSTRAINT);
 					reg_aux=QRegExp(QString("(COLUMN|CONSTRAINT)( )+"));
 
-					pos+=tab_obj_reg.matchedLength();
-					pos1=aux_cmd.indexOf(' ', pos);
-
 					//Extracting the table name
 					pos=aux_cmd.indexOf(alter_tab) + alter_tab.size();
 					pos1=aux_cmd.indexOf(QString("ADD"));
@@ -944,6 +991,7 @@ void ModelExportHelper::exportBufferToDBMS(const QString &buffer, Connection &co
 			{
 				conn.close();
 				aux_conn=conn;
+				aux_conn.setConnectionParam(Connection::PARAM_DB_NAME, orig_conn_db_name);
 				aux_conn.connect();
 				for(QString cmd : db_sql_cmds)
 					aux_conn.executeDDLCommand(cmd);
@@ -952,18 +1000,7 @@ void ModelExportHelper::exportBufferToDBMS(const QString &buffer, Connection &co
 		catch(Exception &e)
 		{
 			if(ddl_tk_found) ddl_tk_found=false;
-
-			//Ignoring the error if it is in the ignored list
-			if(ignored_errors.indexOf(e.getExtraInfo()) >= 0 ||
-					(ignore_dup && isDuplicationError(e.getExtraInfo())))
-				emit s_errorIgnored(e.getExtraInfo(), e.getErrorMessage(), sql_cmd);
-			//Raises an excpetion if the error returned by the database is not listed in the ignored list of errors
-			else if(ignored_errors.indexOf(e.getExtraInfo()) < 0)
-				throw Exception(e.getErrorMessage(),
-								e.getErrorType(),__PRETTY_FUNCTION__,__FILE__,__LINE__,&e, sql_cmd);
-			else
-				errors.push_back(e);
-
+			handleSQLError(e, sql_cmd, ignore_dup);
 			sql_cmd.clear();
 		}
 	}
