@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2016 - Raphael Araújo e Silva <raphael@pgmodeler.com.br>
+# Copyright 2006-2018 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -18,6 +18,7 @@
 
 #include "modelobjectswidget.h"
 #include "databaseimportform.h"
+#include "pgmodeleruins.h"
 
 ModelObjectsWidget::ModelObjectsWidget(bool simplified_view, QWidget *parent) : QWidget(parent)
 {
@@ -35,6 +36,7 @@ ModelObjectsWidget::ModelObjectsWidget(bool simplified_view, QWidget *parent) :
 	cancel_tb->setVisible(simplified_view);
 	options_tb->setVisible(!simplified_view);
 	visibleobjects_grp->setVisible(false);
+	filter_wgt->setVisible(simplified_view);
 
 	selected_object=nullptr;
 	splitter->handle(1)->setEnabled(false);
@@ -119,8 +121,7 @@ void ModelObjectsWidget::editObject(void)
 		//If the user double-clicked the item "Permission (n)" on tree view
 		if(sender()==objectstree_tw && objectstree_tw->currentItem() &&
 				objectstree_tw->currentItem()->data(1, Qt::UserRole).toUInt()==OBJ_PERMISSION)
-			model_wgt->showObjectForm(OBJ_PERMISSION,
-									  reinterpret_cast<BaseObject *>(objectstree_tw->currentItem()->data(0, Qt::UserRole).value<void *>()));
+			model_wgt->showObjectForm(OBJ_PERMISSION, reinterpret_cast<BaseObject *>(objectstree_tw->currentItem()->data(0, Qt::UserRole).value<void *>()));
 		//If the user double-clicked a permission on  list view
 		else if(sender()==objectslist_tbw && objectslist_tbw->currentRow() >= 0)
 		{
@@ -161,7 +162,7 @@ void ModelObjectsWidget::selectObject(void)
 				obj_type!=OBJ_COLUMN && obj_type!=OBJ_CONSTRAINT && obj_type!=OBJ_RULE &&
 				obj_type!=OBJ_INDEX && obj_type!=OBJ_TRIGGER && obj_type!=OBJ_PERMISSION)
 		{
-			QAction act(QPixmap(QString(":/icones/icones/") + BaseObject::getSchemaName(obj_type) + QString(".png")),
+			QAction act(QPixmap(PgModelerUiNS::getIconPath(obj_type)),
 						trUtf8("New") + QString(" ") + BaseObject::getTypeName(obj_type), nullptr);
 			QMenu popup;
 
@@ -314,8 +315,7 @@ QTreeWidgetItem *ModelObjectsWidget::createItemForObject(BaseObject *object, QTr
 			str_aux=QString("_%1").arg(TableObjectView::TXT_EXCLUDE);
 	}
 
-	str_aux=BaseObject::getSchemaName(obj_type) + str_aux;
-	item->setIcon(0,QPixmap(QString(":/icones/icones/") + str_aux + QString(".png")));
+	item->setIcon(0, QPixmap(PgModelerUiNS::getIconPath(BaseObject::getSchemaName(obj_type) + str_aux)));
 
 	return(item);
 }
@@ -381,6 +381,7 @@ void ModelObjectsWidget::changeObjectsView(void)
 	}
 	else if(sender()==options_tb)
 	{
+		filter_wgt->setVisible(options_tb->isChecked());
 		visibleobjects_grp->setVisible(options_tb->isChecked());
 		splitter->handle(1)->setEnabled(options_tb->isChecked());
 
@@ -460,20 +461,17 @@ void ModelObjectsWidget::updateSchemaTree(QTreeWidgetItem *root)
 {
 	if(db_model && visible_objs_map[OBJ_SCHEMA])
 	{
-		BaseObject *object=nullptr, *schema=nullptr;
+		BaseObject *schema=nullptr;
 		vector<BaseObject *> obj_list;
 		QFont font;
 		QTreeWidgetItem *item=nullptr, *item1=nullptr, *item2=nullptr, *item3=nullptr;
-		ObjectType types[]={ OBJ_FUNCTION, OBJ_AGGREGATE,
-							 OBJ_DOMAIN, OBJ_TYPE, OBJ_CONVERSION,
-							 OBJ_OPERATOR, OBJ_OPFAMILY, OBJ_OPCLASS,
-							 OBJ_SEQUENCE, OBJ_COLLATION, OBJ_EXTENSION };
-		int count, count2, type_cnt=sizeof(types)/sizeof(ObjectType), i, i1, i2;
+		vector<ObjectType> types = BaseObject::getChildObjectTypes(OBJ_SCHEMA);
+		int count, count2, i;
+		QPixmap group_icon=QPixmap(PgModelerUiNS::getIconPath(QString(BaseObject::getSchemaName(OBJ_SCHEMA)) + QString("_grp")));
 
-		QPixmap group_icon=QPixmap(QString(":/icones/icones/") +
-								   QString(BaseObject::getSchemaName(OBJ_SCHEMA)) +
-								   QString("_grp") +
-								   QString(".png"));
+		//Removing the OBJ_TABLE and OBJ_VIEW types since they are handled separetedly
+		types.erase(std::find(types.begin(), types.end(), OBJ_TABLE));
+		types.erase(std::find(types.begin(), types.end(), OBJ_VIEW));
 
 		//Get the current schema count on database
 		count=(db_model->getObjectCount(OBJ_SCHEMA));
@@ -482,8 +480,7 @@ void ModelObjectsWidget::updateSchemaTree(QTreeWidgetItem *root)
 		item->setData(1, Qt::UserRole, QVariant::fromValue<unsigned>(OBJ_SCHEMA));
 
 		//Create the schema group item
-		item->setText(0,BaseObject::getTypeName(OBJ_SCHEMA) +
-					  QString(" (%1)").arg(count));
+		item->setText(0, QString("%1 (%2)").arg(BaseObject::getTypeName(OBJ_SCHEMA)).arg(count));
 		font=item->font(0);
 		font.setItalic(true);
 		item->setFont(0, font);
@@ -501,7 +498,7 @@ void ModelObjectsWidget::updateSchemaTree(QTreeWidgetItem *root)
 				}
 				else
 				{
-					schema=db_model->getObject(i,OBJ_SCHEMA);
+					schema=db_model->getObject(i, OBJ_SCHEMA);
 					item2=createItemForObject(schema, item);
 				}
 
@@ -512,34 +509,26 @@ void ModelObjectsWidget::updateSchemaTree(QTreeWidgetItem *root)
 				updateViewTree(item2, schema);
 
 				//Creates the object group at schema level (function, domain, sequences, etc)
-				for(i1=0; i1 < type_cnt; i1++)
+				for(auto type : types)
 				{
-					if(visible_objs_map[types[i1]])
+					if(visible_objs_map[type])
 					{
 						item3=new QTreeWidgetItem(item2);
-						item3->setIcon(0,QPixmap(QString(":/icones/icones/") +
-												 QString(BaseObject::getSchemaName(types[i1])) +
-												 QString("_grp") +
-												 QString(".png")));
+						item3->setIcon(0,QPixmap(PgModelerUiNS::getIconPath(BaseObject::getSchemaName(type) + QString("_grp"))));
 
 						//Get the objects that belongs to the current schema
-						obj_list=db_model->getObjects(types[i1], schema);
+						obj_list=db_model->getObjects(type, schema);
 
 						count2=obj_list.size();
-						item3->setText(0,
-									   BaseObject::getTypeName(types[i1]) +
-									   QString(" (%1)").arg(count2));
-						item3->setData(1, Qt::UserRole, QVariant::fromValue<unsigned>(types[i1]));
+						item3->setText(0, QString("%1 (%2)").arg(BaseObject::getTypeName(type)).arg(count2));
+						item3->setData(1, Qt::UserRole, QVariant::fromValue<unsigned>(type));
 
 						font=item3->font(0);
 						font.setItalic(true);
 						item3->setFont(0, font);
 
-						for(i2=0; i2 < count2; i2++)
-						{
-							object=obj_list[i2];
-							createItemForObject(object, item3);
-						}
+						for(auto obj : obj_list)
+							createItemForObject(obj, item3);
 					}
 				}
 			}
@@ -555,17 +544,12 @@ void ModelObjectsWidget::updateTableTree(QTreeWidgetItem *root, BaseObject *sche
 {
 	if(db_model && visible_objs_map[OBJ_TABLE])
 	{
-		BaseObject *object=nullptr;
 		vector<BaseObject *> obj_list;
 		Table *table=nullptr;
 		QTreeWidgetItem *item=nullptr, *item1=nullptr, *item2=nullptr;
 		QFont font;
-		ObjectType types[]={ OBJ_COLUMN, OBJ_CONSTRAINT, OBJ_RULE,
-							 OBJ_TRIGGER, OBJ_INDEX };
-		int count, count1, type_cnt=sizeof(types)/sizeof(ObjectType), i, i1, i2;
-		QPixmap group_icon=QPixmap(QString(":/icones/icones/") +
-								   QString(BaseObject::getSchemaName(OBJ_TABLE)) +
-								   QString("_grp") + QString(".png"));
+		vector<ObjectType> types = BaseObject::getChildObjectTypes(OBJ_TABLE);
+		QPixmap group_icon=QPixmap(PgModelerUiNS::getIconPath(BaseObject::getSchemaName(OBJ_TABLE) + QString("_grp")));
 
 		try
 		{
@@ -583,35 +567,26 @@ void ModelObjectsWidget::updateTableTree(QTreeWidgetItem *root, BaseObject *sche
 			font.setItalic(true);
 			item->setFont(0, font);
 
-			count=obj_list.size();
-			for(i=0; i < count; i++)
+			for(auto obj : obj_list)
 			{
-				table=dynamic_cast<Table *>(obj_list[i]);
+				table=dynamic_cast<Table *>(obj);
 				item1=createItemForObject(table, item);
 
 				//Creating the group for the child objects (column, rules, triggers, indexes and constraints)
-				for(i1=0; i1 < type_cnt; i1++)
+				for(auto type : types)
 				{
-					if(visible_objs_map[types[i1]])
+					if(visible_objs_map[type])
 					{
 						item2=new QTreeWidgetItem(item1);
-						item2->setIcon(0,QPixmap(QString(":/icones/icones/") +
-												 QString(BaseObject::getSchemaName(types[i1])) +
-												 QString("_grp") +
-												 QString(".png")));
+						item2->setIcon(0,QPixmap(PgModelerUiNS::getIconPath(BaseObject::getSchemaName(type) + QString("_grp"))));
 						font=item2->font(0);
 						font.setItalic(true);
 						item2->setFont(0, font);
 
-						count1=table->getObjectCount(types[i1]);
-						item2->setText(0,BaseObject::getTypeName(types[i1]) +
-									   QString(" (%1)").arg(count1));
+						item2->setText(0, QString("%1 (%2)").arg(BaseObject::getTypeName(type)).arg(table->getObjectCount(type)));
 
-						for(i2=0; i2 < count1; i2++)
-						{
-							object=table->getObject(i2,types[i1]);
-							createItemForObject(object, item2);
-						}
+						for(auto tab_obj : *table->getObjectList(type))
+							createItemForObject(tab_obj, item2);
 					}
 				}
 			}
@@ -632,11 +607,9 @@ void ModelObjectsWidget::updateViewTree(QTreeWidgetItem *root, BaseObject *schem
 		View *view=nullptr;
 		QTreeWidgetItem *item=nullptr, *item1=nullptr, *item2=nullptr;
 		QFont font;
-		ObjectType types[]={ OBJ_RULE, OBJ_TRIGGER };
+		ObjectType types[]={ OBJ_RULE, OBJ_TRIGGER, OBJ_INDEX };
 		int count, count1, type_cnt=sizeof(types)/sizeof(ObjectType), i, i1, i2;
-		QPixmap group_icon=QPixmap(QString(":/icones/icones/") +
-								   QString(BaseObject::getSchemaName(OBJ_VIEW)) +
-								   QString("_grp") + QString(".png"));
+		QPixmap group_icon=QPixmap(PgModelerUiNS::getIconPath(QString(BaseObject::getSchemaName(OBJ_VIEW)) + QString("_grp")));
 
 		try
 		{
@@ -666,10 +639,7 @@ void ModelObjectsWidget::updateViewTree(QTreeWidgetItem *root, BaseObject *schem
 					if(visible_objs_map[types[i1]])
 					{
 						item2=new QTreeWidgetItem(item1);
-						item2->setIcon(0,QPixmap(QString(":/icones/icones/") +
-												 QString(BaseObject::getSchemaName(types[i1])) +
-												 QString("_grp") +
-												 QString(".png")));
+						item2->setIcon(0,QPixmap(PgModelerUiNS::getIconPath(BaseObject::getSchemaName(types[i1]) + QString("_grp"))));
 						font=item2->font(0);
 						font.setItalic(true);
 						item2->setFont(0, font);
@@ -706,7 +676,7 @@ void ModelObjectsWidget::updatePermissionTree(QTreeWidgetItem *root, BaseObject
 			QFont font=item->font(0);
 
 			db_model->getPermissions(object, perms);
-			item->setIcon(0,QPixmap(QString(":/icones/icones/permission_grp.png")));
+			item->setIcon(0,QPixmap(PgModelerUiNS::getIconPath("permission_grp")));
 
 			font.setItalic(true);
 			item->setFont(0, font);
@@ -737,7 +707,8 @@ void ModelObjectsWidget::updateDatabaseTree(void)
 		vector<BaseObject *> ref_list, tree_state, obj_list;
 		ObjectType types[]={ OBJ_ROLE, OBJ_TABLESPACE,
 							 OBJ_LANGUAGE, OBJ_CAST, OBJ_TEXTBOX,
-							 OBJ_RELATIONSHIP, OBJ_EVENT_TRIGGER, OBJ_TAG };
+							 OBJ_RELATIONSHIP, OBJ_EVENT_TRIGGER,
+							 OBJ_TAG, OBJ_GENERIC_SQL, OBJ_EXTENSION };
 		unsigned count, i, i1, type_cnt=sizeof(types)/sizeof(ObjectType);
 
 		try
@@ -761,8 +732,7 @@ void ModelObjectsWidget::updateDatabaseTree(void)
 						item1=new QTreeWidgetItem(root);
 						str_aux=QString(BaseObject::getSchemaName(types[i]));
 
-						item1->setIcon(0,QPixmap(QString(":/icones/icones/") +
-												 str_aux + QString("_grp") + QString(".png")));
+						item1->setIcon(0,QPixmap(PgModelerUiNS::getIconPath(str_aux + QString("_grp"))));
 						item1->setData(1, Qt::UserRole, QVariant::fromValue<unsigned>(types[i]));
 
 						obj_list=(*db_model->getObjectList(types[i]));
