@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2016 - Raphael Araújo e Silva <raphael@pgmodeler.com.br>
+# Copyright 2006-2018 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -30,12 +30,13 @@ ModelsDiffHelper::ModelsDiffHelper(void)
 	diff_opts[OPT_KEEP_CLUSTER_OBJS]=true;
 	diff_opts[OPT_CASCADE_MODE]=true;
 	diff_opts[OPT_TRUCANTE_TABLES]=false;
-	diff_opts[OPT_FORCE_RECREATION]=false;
+	diff_opts[OPT_FORCE_RECREATION]=true;
 	diff_opts[OPT_RECREATE_UNCHANGEBLE]=true;
 	diff_opts[OPT_KEEP_OBJ_PERMS]=true;
 	diff_opts[OPT_REUSE_SEQUENCES]=true;
 	diff_opts[OPT_PRESERVE_DB_NAME]=true;
-	diff_opts[OPT_KEEP_NOT_IMPORTED_OBJS]=false;
+	diff_opts[OPT_DONT_DROP_MISSING_OBJS]=false;
+	diff_opts[OPT_DROP_MISSING_COLS_CONSTR]=false;
 }
 
 ModelsDiffHelper::~ModelsDiffHelper(void)
@@ -45,10 +46,13 @@ ModelsDiffHelper::~ModelsDiffHelper(void)
 
 void ModelsDiffHelper::setDiffOption(unsigned opt_id, bool value)
 {
-	if(opt_id > OPT_KEEP_NOT_IMPORTED_OBJS)
+	if(opt_id > OPT_DROP_MISSING_COLS_CONSTR)
 		throw Exception(ERR_REF_ELEM_INV_INDEX,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 
-	diff_opts[opt_id]=value;
+	if(opt_id == OPT_DROP_MISSING_COLS_CONSTR)
+		diff_opts[opt_id]=value & !diff_opts[OPT_DROP_MISSING_COLS_CONSTR];
+	else
+		diff_opts[opt_id]=value;
 }
 
 void ModelsDiffHelper::setPgSQLVersion(const QString pgsql_ver)
@@ -149,20 +153,22 @@ void ModelsDiffHelper::diffTables(Table *src_table, Table *imp_table, unsigned d
 			aux_obj=comp_tab->getObject(tab_obj->getName(), tab_obj->getObjectType());
 			constr=dynamic_cast<Constraint *>(tab_obj);
 
-			//Ignoring check constraints added by generalizations
-			if(constr && constr->isAddedByGeneralization() &&
-					constr->getConstraintType()==ConstraintType::check)
+			//Ignoring object with sql disabled or check constraints added by generalizations
+			if(tab_obj->isSQLDisabled() ||
+				 (constr && constr->isAddedByGeneralization() &&
+					constr->getConstraintType()==ConstraintType::check))
 			{
-				generateDiffInfo(ObjectsDiffInfo::IGNORE_OBJECT, constr);
+				generateDiffInfo(ObjectsDiffInfo::IGNORE_OBJECT, tab_obj);
 			}
 			else
 			{
 				/* If the current info is ALTER or CREATE, only objects created by generalization or
-		 not created by common relationships will be considered on the comparison. Also,
-		 foreign keys are discarded here, since they will be compared on the main comparison
-		 at diffModels() */
+					 columns created by common relationships will be considered on the comparison. Also,
+					foreign keys are discarded here, since they will be compared on the main comparison
+					at diffModels() */
 				if(aux_obj && diff_type!=ObjectsDiffInfo::DROP_OBJECT &&
-						((tab_obj->isAddedByGeneralization() || !tab_obj->isAddedByLinking()) ||
+						((tab_obj->isAddedByGeneralization() || !tab_obj->isAddedByLinking() ||
+							(aux_obj->getObjectType()==OBJ_COLUMN && tab_obj->isAddedByLinking())) ||
 						 (constr && constr->getConstraintType()!=ConstraintType::foreign_key)))
 				{
 					//If there are some differences on the XML code of the objects
@@ -170,12 +176,13 @@ void ModelsDiffHelper::diffTables(Table *src_table, Table *imp_table, unsigned d
 						generateDiffInfo(ObjectsDiffInfo::ALTER_OBJECT, tab_obj, aux_obj);
 
 				}
-				/* If the object does not exists it will generate a drop info and the original
-		 one (tab_obj) was not included by generalization (to avoid drop inherited columns) */
+				/*	If the object does not exists it will generate a drop info and the original
+						one (tab_obj) was not included by generalization (to avoid drop inherited columns) */
 				else if(!aux_obj && !tab_obj->isAddedByGeneralization())
 				{
 					if(diff_type!=ObjectsDiffInfo::DROP_OBJECT ||
-						 (diff_type==ObjectsDiffInfo::DROP_OBJECT && !diff_opts[OPT_KEEP_NOT_IMPORTED_OBJS]))
+						 (diff_type==ObjectsDiffInfo::DROP_OBJECT && !diff_opts[OPT_DONT_DROP_MISSING_OBJS]) ||
+						 (diff_type==ObjectsDiffInfo::DROP_OBJECT && diff_opts[OPT_DROP_MISSING_COLS_CONSTR]))
 						generateDiffInfo(diff_type, tab_obj);
 					else
 						generateDiffInfo(ObjectsDiffInfo::IGNORE_OBJECT, tab_obj);
@@ -286,7 +293,7 @@ void ModelsDiffHelper::diffModels(unsigned diff_type)
 						if(obj_type==OBJ_TABLE && !aux_object)
 							aux_object=getRelNNTable(obj_name, aux_model);
 
-						if(diff_type!=ObjectsDiffInfo::DROP_OBJECT && aux_object)
+						if(diff_type != ObjectsDiffInfo::DROP_OBJECT && aux_object)
 						{
 							/* Try to get a diff from the retrieve object and the current object,
 				 comparing only basic attributes like schema, tablespace and owner
@@ -299,7 +306,9 @@ void ModelsDiffHelper::diffModels(unsigned diff_type)
 								{ ParsersAttributes::PROTECTED,
 								  ParsersAttributes::SQL_DISABLED,
 								  ParsersAttributes::RECT_VISIBLE,
-								  ParsersAttributes::FILL_COLOR },
+									ParsersAttributes::FILL_COLOR,
+									ParsersAttributes::FADED_OUT,
+									ParsersAttributes::HIDE_EXT_ATTRIBS},
 								{ ParsersAttributes::ROLE,
 								  ParsersAttributes::TABLESPACE,
 								  ParsersAttributes::COLLATION,
@@ -325,7 +334,8 @@ void ModelsDiffHelper::diffModels(unsigned diff_type)
 						}
 						else if(!aux_object)
 						{
-							if(!diff_opts[OPT_KEEP_NOT_IMPORTED_OBJS])
+							if(diff_type != ObjectsDiffInfo::DROP_OBJECT ||
+								 (diff_type == ObjectsDiffInfo::DROP_OBJECT && !diff_opts[OPT_DONT_DROP_MISSING_OBJS]))
 								generateDiffInfo(diff_type, object);
 							else
 								generateDiffInfo(ObjectsDiffInfo::IGNORE_OBJECT, object);
@@ -383,8 +393,7 @@ void ModelsDiffHelper::diffTableObject(TableObject *tab_obj, unsigned diff_type)
 		if(!aux_base_tab)
 			aux_base_tab=dynamic_cast<BaseTable *>(getRelNNTable(tab_name, source_model));
 	}
-	else if(diff_type==ObjectsDiffInfo::CREATE_OBJECT ||
-			diff_type==ObjectsDiffInfo::ALTER_OBJECT)
+	else if(diff_type==ObjectsDiffInfo::CREATE_OBJECT || diff_type==ObjectsDiffInfo::ALTER_OBJECT)
 	{
 		aux_base_tab=dynamic_cast<BaseTable *>(imported_model->getObject(tab_name, base_tab->getObjectType()));
 
@@ -395,7 +404,7 @@ void ModelsDiffHelper::diffTableObject(TableObject *tab_obj, unsigned diff_type)
 
 	if(aux_base_tab)
 	{
-		if(obj_type==OBJ_INDEX || obj_type==OBJ_CONSTRAINT)
+		if(obj_type==OBJ_CONSTRAINT)
 		{
 			Table *aux_table=dynamic_cast<Table *>(aux_base_tab);
 			aux_tab_obj=aux_table->getObject(obj_name, obj_type);
@@ -406,7 +415,8 @@ void ModelsDiffHelper::diffTableObject(TableObject *tab_obj, unsigned diff_type)
 
 	if(!aux_tab_obj)
 	{
-		if(!diff_opts[OPT_KEEP_NOT_IMPORTED_OBJS])
+		if(diff_type!=ObjectsDiffInfo::DROP_OBJECT ||
+			 (diff_type==ObjectsDiffInfo::DROP_OBJECT && !diff_opts[OPT_DONT_DROP_MISSING_OBJS]))
 			generateDiffInfo(diff_type, tab_obj);
 		else
 			generateDiffInfo(ObjectsDiffInfo::IGNORE_OBJECT, tab_obj);
@@ -590,14 +600,14 @@ void ModelsDiffHelper::processDiffInfos(void)
 {
 	BaseObject *object=nullptr;
 	Relationship *rel=nullptr;
-	map<unsigned, QString> drop_objs, create_objs, alter_objs, truncate_tabs;
+	map<unsigned, QString> drop_objs, create_objs, alter_objs, truncate_tabs, create_fks, create_constrs;
 	vector<BaseObject *> drop_vect, create_vect, drop_cols;
 	unsigned diff_type, schema_id=0, idx=0;
 	ObjectType obj_type;
 	map<unsigned, QString>::reverse_iterator ritr, ritr_end;
 	attribs_map attribs;
 	QString alter_def, no_inherit_def, inherit_def, set_perms,
-			unset_perms, col_drop_def;
+			unset_perms, col_drop_def, curr_pgsql_ver = BaseObject::getPgSQLVersion();
 	SchemaParser schparser;
 	Type *type=nullptr;
 	vector<Type *> types;
@@ -606,10 +616,12 @@ void ModelsDiffHelper::processDiffInfos(void)
 	Table *parent_tab=nullptr;
 	bool skip_obj=false;
 	QStringList sch_names;
-	map<unsigned, QString> create_fks;
 
 	try
 	{
+		//Overriding the global PostgreSQL version so the diff code can match the destination server version
+		BaseObject::setPgSQLVersion(pgsql_version);
+
 		if(!diff_infos.empty())
 			emit s_progressUpdated(0, trUtf8("Processing diff infos..."));
 
@@ -711,10 +723,15 @@ void ModelsDiffHelper::processDiffInfos(void)
 					set_perms+=object->getCodeDefinition(SchemaParser::SQL_DEFINITION);
 				else
 				{
-					//Generating fks definitions in a separated variable in order to append them at create commands maps
-					if(object->getObjectType()==OBJ_CONSTRAINT &&
-							dynamic_cast<Constraint *>(object)->getConstraintType()==ConstraintType::foreign_key)
-						create_fks[object->getObjectId()]=getCodeDefinition(object, false);
+					/* Special case for constaints: the creation commands for these objects are appended at the very end of create commands secion.
+						Primary keys, unique keys, check constraints and exclude constraints are created after foreign keys */
+					if(object->getObjectType()==OBJ_CONSTRAINT)
+					{
+						if(dynamic_cast<Constraint *>(object)->getConstraintType()==ConstraintType::foreign_key)
+							create_fks[object->getObjectId()]=getCodeDefinition(object, false);
+						else
+							create_constrs[object->getObjectId()]=getCodeDefinition(object, false);
+					}
 					else
 					{
 						create_objs[object->getObjectId()]=getCodeDefinition(object, false);
@@ -747,10 +764,14 @@ void ModelsDiffHelper::processDiffInfos(void)
 						if(!isDiffInfoExists(ObjectsDiffInfo::ALTER_OBJECT, nullptr, obj, false))
 						{
 							/* Special case for constraints, their code will be appeded to a separated variable in order to
-				 create them at the end of diff buffer */
-							if(obj->getObjectType()==OBJ_CONSTRAINT &&
-									dynamic_cast<Constraint *>(obj)->getConstraintType()==ConstraintType::foreign_key)
-								create_fks[obj->getObjectId()]=getCodeDefinition(obj, false);
+							 create them at the end of diff buffer */
+							if(obj->getObjectType()==OBJ_CONSTRAINT)
+							{
+								if(dynamic_cast<Constraint *>(obj)->getConstraintType()==ConstraintType::foreign_key)
+									create_fks[obj->getObjectId()]=getCodeDefinition(obj, false);
+								else
+									create_constrs[obj->getObjectId()]=getCodeDefinition(obj, false);
+							}
 							else
 								create_objs[obj->getObjectId()]=getCodeDefinition(obj, false);
 						}
@@ -764,22 +785,27 @@ void ModelsDiffHelper::processDiffInfos(void)
 					if(diff.getOldObject())
 						alter_def=diff.getOldObject()->getAlterDefinition(object);
 
+					if(obj_type == OBJ_DATABASE && diff_opts[OPT_PRESERVE_DB_NAME])
+						alter_def.remove(QRegExp(QString("(ALTER)( )+(DATABASE)( )+(%1)( )+(RENAME)( )+(TO)(.)*(\\n)").arg(diff.getOldObject()->getSignature())));
+
 					if(!alter_def.isEmpty())
 					{
-						if(obj_type!=OBJ_DATABASE || !diff_opts[OPT_PRESERVE_DB_NAME])
-							alter_objs[object->getObjectId()]=alter_def;
+						alter_objs[object->getObjectId()]=alter_def;
 
 						/* If the object is a column checks if the types of the columns are differents,
 							generating a TRUNCATE TABLE for the parent table */
 						if(obj_type==OBJ_COLUMN && diff_opts[OPT_TRUCANTE_TABLES])
 						{
 							Column *src_col=dynamic_cast<Column *>(object),
-									*imp_col=dynamic_cast<Column *>(diff.getOldObject());
+									*old_col=dynamic_cast<Column *>(diff.getOldObject());
 							Table *tab=dynamic_cast<Table *>(src_col->getParentTable());
 
-							//If the truncate was not generated previously
-							if((*src_col->getType())!=(*imp_col->getType()) && truncate_tabs.count(tab->getObjectId())==0)
+							if(((old_col->getType().isSerialType() && !src_col->getType().isEquivalentTo(old_col->getType().getAliasType())) ||
+									(!old_col->getType().isSerialType() && !src_col->getType().isEquivalentTo(old_col->getType()))) &&
+								 truncate_tabs.count(tab->getObjectId())==0)
+							{
 								truncate_tabs[tab->getObjectId()]=tab->getTruncateDefinition(diff_opts[OPT_CASCADE_MODE]);
+							}
 						}
 					}
 				}
@@ -805,11 +831,11 @@ void ModelsDiffHelper::processDiffInfos(void)
 
 		diff_def.clear();
 
-		if(!drop_objs.empty() || !create_objs.empty() || !alter_objs.empty() || !create_fks.empty() ||
-				!inherit_def.isEmpty() || !no_inherit_def.isEmpty() || !set_perms.isEmpty() ||
-				!col_drop_def.isEmpty())
+		if(!drop_objs.empty() || !create_objs.empty() || !alter_objs.empty() ||
+			 !create_fks.empty() || !create_constrs.empty() || !inherit_def.isEmpty() ||
+			 !no_inherit_def.isEmpty() || !set_perms.isEmpty() || !col_drop_def.isEmpty())
 		{
-			unsigned create_objs_count=create_objs.size() + create_fks.size();
+			unsigned create_objs_count=create_objs.size() + create_constrs.size() + create_fks.size();
 			bool has_diffs=false;
 
 			sch_names.removeDuplicates();
@@ -818,6 +844,10 @@ void ModelsDiffHelper::processDiffInfos(void)
 			//Attributes used on the diff schema file
 			attribs[ParsersAttributes::HAS_CHANGES]=ParsersAttributes::_TRUE_;
 			attribs[ParsersAttributes::PGMODELER_VERSION]=GlobalAttributes::PGMODELER_VERSION;
+			attribs[ParsersAttributes::DB_MODEL]=source_model->getName();
+			attribs[ParsersAttributes::DATABASE]=imported_model->getName();
+			attribs[ParsersAttributes::DATE]=QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss");
+			attribs[ParsersAttributes::CONNECTION]=imported_model->getName();
 			attribs[ParsersAttributes::CHANGE]=QString::number(alter_objs.size());
 			attribs[ParsersAttributes::CREATE]=QString::number(create_objs_count);
 			attribs[ParsersAttributes::DROP]=QString::number(drop_objs.size());
@@ -826,6 +856,7 @@ void ModelsDiffHelper::processDiffInfos(void)
 			attribs[ParsersAttributes::DROP_CMDS]=QString();
 			attribs[ParsersAttributes::CREATE_CMDS]=QString();
 			attribs[ParsersAttributes::TRUNCATE_CMDS]=QString();
+			attribs[ParsersAttributes::CONSTR_DEFS]=QString();
 			attribs[ParsersAttributes::FK_DEFS]=QString();
 			attribs[ParsersAttributes::UNSET_PERMS]=unset_perms;
 			attribs[ParsersAttributes::SET_PERMS]=set_perms;
@@ -851,6 +882,9 @@ void ModelsDiffHelper::processDiffInfos(void)
 
 			attribs[ParsersAttributes::CREATE_CMDS]+=inherit_def;
 
+			for(auto &itr : create_constrs)
+				attribs[ParsersAttributes::CONSTR_DEFS]+=itr.second;
+
 			for(auto &itr : create_fks)
 				attribs[ParsersAttributes::FK_DEFS]+=itr.second;
 
@@ -871,9 +905,14 @@ void ModelsDiffHelper::processDiffInfos(void)
 			emit s_progressUpdated(100, trUtf8("No differences between the model and database."));
 		else
 			emit s_progressUpdated(100, trUtf8("Preparing diff code..."));
+
+		//Restoring the global PostgreSQL version
+		BaseObject::setPgSQLVersion(curr_pgsql_ver);
 	}
 	catch(Exception &e)
 	{
+		BaseObject::setPgSQLVersion(curr_pgsql_ver);
+
 		for(Type *type : types)
 			type->convertFunctionParameters(true);
 
@@ -939,7 +978,8 @@ void ModelsDiffHelper::recreateObject(BaseObject *object, vector<BaseObject *> &
 {
 	if(object &&
 			object->getObjectType()!=BASE_RELATIONSHIP &&
-			object->getObjectType()!=OBJ_RELATIONSHIP)
+			object->getObjectType()!=OBJ_RELATIONSHIP &&
+			object->getObjectType()!=OBJ_DATABASE)
 	{
 		vector<BaseObject *> ref_objs;
 		BaseObject *aux_obj=nullptr;
@@ -955,8 +995,8 @@ void ModelsDiffHelper::recreateObject(BaseObject *object, vector<BaseObject *> &
 			if(tab_obj->getParentTable())
 			{
 				/* If the object is a table's child object, first we get a table from the database which
-		   name is the same as the 'object' parent table's name. Sencond, we try to get a child
-		   object from that table using the same name as the 'object' */
+				name is the same as the 'object' parent table's name. Sencond, we try to get a child
+				object from that table using the same name as the 'object' */
 				BaseTable *tab=dynamic_cast<BaseTable *>(imported_model->getObject(tab_obj->getParentTable()->getSignature(), tab_obj->getParentTable()->getObjectType()));
 				aux_obj=tab->getObject(tab_obj->getName(true), tab_obj->getObjectType());
 			}
@@ -966,7 +1006,7 @@ void ModelsDiffHelper::recreateObject(BaseObject *object, vector<BaseObject *> &
 		imported_model->getObjectReferences(aux_obj, ref_objs, false, true);
 
 		/* If the to-be recreate object is a constraint check if it's a pk,
-	   if so, the fk's linked to it need to be recreated as well */
+		 if so, the fk's linked to it need to be recreated as well */
 		if(aux_obj->getObjectType()==OBJ_CONSTRAINT)
 		{
 			Constraint *constr=dynamic_cast<Constraint *>(aux_obj);
