@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2016 - Raphael Araújo e Silva <raphael@pgmodeler.com.br>
+# Copyright 2006-2018 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -43,9 +43,6 @@ class OperatorWidget: public BaseObjectWidget, public Ui::OperatorWidget {
 
 		void setAttributes(DatabaseModel *model, OperationList *op_list, Schema *schema, Operator *oper);
 
-	private slots:
-		void hideEvent(QHideEvent *event);
-
 	public slots:
 		void applyConfiguration(void);
 };
