@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2016 - Raphael Araújo e Silva <raphael@pgmodeler.com.br>
+# Copyright 2006-2018 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -65,33 +65,33 @@ RelationshipWidget::RelationshipWidget(QWidget *parent): BaseObjectWidget(parent
 											  GlobalAttributes::CONFIGURATION_EXT);
 		}
 
-		attributes_tab=new ObjectTableWidget(ObjectTableWidget::ALL_BUTTONS ^
-											 (ObjectTableWidget::UPDATE_BUTTON |
-											  ObjectTableWidget::MOVE_BUTTONS), true, this);
+		attributes_tab=new ObjectsTableWidget(ObjectsTableWidget::ALL_BUTTONS ^
+											 (ObjectsTableWidget::UPDATE_BUTTON |
+											  ObjectsTableWidget::MOVE_BUTTONS), true, this);
 
-		constraints_tab=new ObjectTableWidget(ObjectTableWidget::ALL_BUTTONS  ^
-											  (ObjectTableWidget::UPDATE_BUTTON |
-											   ObjectTableWidget::MOVE_BUTTONS), true, this);
+		constraints_tab=new ObjectsTableWidget(ObjectsTableWidget::ALL_BUTTONS  ^
+											  (ObjectsTableWidget::UPDATE_BUTTON |
+											   ObjectsTableWidget::MOVE_BUTTONS), true, this);
 
-		advanced_objs_tab=new ObjectTableWidget(ObjectTableWidget::EDIT_BUTTON, true, this);
+		advanced_objs_tab=new ObjectsTableWidget(ObjectsTableWidget::EDIT_BUTTON, true, this);
 
 		attributes_tab->setColumnCount(2);
 		attributes_tab->setHeaderLabel(trUtf8("Attribute"), 0);
-		attributes_tab->setHeaderIcon(QPixmap(QString(":/icones/icones/column.png")),0);
+		attributes_tab->setHeaderIcon(QPixmap(PgModelerUiNS::getIconPath("column")),0);
 		attributes_tab->setHeaderLabel(trUtf8("Type"), 1);
-		attributes_tab->setHeaderIcon(QPixmap(QString(":/icones/icones/usertype.png")),1);
+		attributes_tab->setHeaderIcon(QPixmap(PgModelerUiNS::getIconPath("usertype")),1);
 
 		constraints_tab->setColumnCount(2);
 		constraints_tab->setHeaderLabel(trUtf8("Constraint"), 0);
-		constraints_tab->setHeaderIcon(QPixmap(QString(":/icones/icones/constraint.png")),0);
+		constraints_tab->setHeaderIcon(QPixmap(PgModelerUiNS::getIconPath("constraint")),0);
 		constraints_tab->setHeaderLabel(trUtf8("Type"), 1);
-		constraints_tab->setHeaderIcon(QPixmap(QString(":/icones/icones/usertype.png")),1);
+		constraints_tab->setHeaderIcon(QPixmap(PgModelerUiNS::getIconPath("usertype")),1);
 
 		advanced_objs_tab->setColumnCount(2);
 		advanced_objs_tab->setHeaderLabel(trUtf8("Name"), 0);
-		advanced_objs_tab->setHeaderIcon(QPixmap(QString(":/icones/icones/column.png")),0);
+		advanced_objs_tab->setHeaderIcon(QPixmap(PgModelerUiNS::getIconPath("column")),0);
 		advanced_objs_tab->setHeaderLabel(trUtf8("Type"), 1);
-		advanced_objs_tab->setHeaderIcon(QPixmap(QString(":/icones/icones/usertype.png")),1);
+		advanced_objs_tab->setHeaderIcon(QPixmap(PgModelerUiNS::getIconPath("usertype")),1);
 
 		connect(advanced_objs_tab, SIGNAL(s_rowEdited(int)), this, SLOT(showAdvancedObject(int)));
 
@@ -157,18 +157,24 @@ RelationshipWidget::RelationshipWidget(QWidget *parent): BaseObjectWidget(parent
 		connect(deferrable_chk, SIGNAL(toggled(bool)), deferral_cmb, SLOT(setEnabled(bool)));
 		connect(deferrable_chk, SIGNAL(toggled(bool)), deferral_lbl, SLOT(setEnabled(bool)));
 
-		connect(identifier_chk, SIGNAL(toggled(bool)), table1_mand_chk, SLOT(setDisabled(bool)));
-		connect(identifier_chk, SIGNAL(toggled(bool)), table2_mand_chk, SLOT(setDisabled(bool)));
+		connect(identifier_chk, &QCheckBox::toggled, [&](){
+			table1_mand_chk->setDisabled(identifier_chk->isChecked());
+			table2_mand_chk->setEnabled(!identifier_chk->isChecked() &&
+																	this->object &&
+																	dynamic_cast<BaseRelationship *>(this->object)->getRelationshipType() != BaseRelationship::RELATIONSHIP_1N);
+		});
 
 		connect(attributes_tab, SIGNAL(s_rowsRemoved(void)), this, SLOT(removeObjects(void)));
 		connect(attributes_tab, SIGNAL(s_rowAdded(int)), this, SLOT(addObject(void)));
 		connect(attributes_tab, SIGNAL(s_rowEdited(int)), this, SLOT(editObject(int)));
 		connect(attributes_tab, SIGNAL(s_rowRemoved(int)), this, SLOT(removeObject(int)));
+		connect(attributes_tab, SIGNAL(s_rowDuplicated(int,int)), this, SLOT(duplicateObject(int,int)));
 
 		connect(constraints_tab, SIGNAL(s_rowsRemoved(void)), this, SLOT(removeObjects(void)));
 		connect(constraints_tab, SIGNAL(s_rowAdded(int)), this, SLOT(addObject(void)));
 		connect(constraints_tab, SIGNAL(s_rowEdited(int)), this, SLOT(editObject(int)));
 		connect(constraints_tab, SIGNAL(s_rowRemoved(int)), this, SLOT(removeObject(int)));
+		connect(constraints_tab, SIGNAL(s_rowDuplicated(int,int)), this, SLOT(duplicateObject(int,int)));
 
 		connect(defaults_rb, SIGNAL(toggled(bool)), this, SLOT(selectCopyOptions(void)));
 		connect(including_rb, SIGNAL(toggled(bool)), this, SLOT(selectCopyOptions(void)));
@@ -186,7 +192,7 @@ RelationshipWidget::RelationshipWidget(QWidget *parent): BaseObjectWidget(parent
 		connect(fk_gconf_chk, SIGNAL(toggled(bool)), this, SLOT(useFKGlobalSettings(bool)));
 		connect(patterns_gconf_chk, SIGNAL(toggled(bool)), this, SLOT(usePatternGlobalSettings(bool)));
 
-		setMinimumSize(600, 0);
+		setMinimumSize(620, 670);
 	}
 	catch(Exception &e)
 	{
@@ -194,36 +200,6 @@ RelationshipWidget::RelationshipWidget(QWidget *parent): BaseObjectWidget(parent
 	}
 }
 
-void RelationshipWidget::hideEvent(QHideEvent *event)
-{
-	BaseRelationship *rel=dynamic_cast<BaseRelationship *>(this->object);
-
-	custom_color_chk->setChecked(false);
-	identifier_chk->setChecked(false);
-	table1_mand_chk->setChecked(false);
-	table2_mand_chk->setChecked(false);
-	relnn_tab_name_edt->clear();
-	deferrable_chk->setChecked(false);
-	deferral_cmb->setCurrentIndex(0);
-	rel_attribs_tbw->setCurrentIndex(0);
-	del_action_cmb->setCurrentIndex(0);
-	upd_action_cmb->setCurrentIndex(0);
-
-	attributes_tab->blockSignals(true);
-	constraints_tab->blockSignals(true);
-	attributes_tab->removeRows();
-	constraints_tab->removeRows();
-	attributes_tab->blockSignals(false);
-	constraints_tab->blockSignals(false);
-
-	rel_columns_lst->clear();
-
-	if(rel && !rel->isModified())
-		this->cancelConfiguration();
-
-	BaseObjectWidget::hideEvent(event);
-}
-
 void RelationshipWidget::setAttributes(DatabaseModel *model, OperationList *op_list, Table *src_tab, Table *dst_tab, unsigned rel_type)
 {
 	Relationship *rel=nullptr;
@@ -332,8 +308,8 @@ void RelationshipWidget::setAttributes(DatabaseModel *model, OperationList *op_l
 		table2_mand_chk->setChecked(aux_rel->isTableMandatory(BaseRelationship::DST_TABLE));
 		identifier_chk->setChecked(aux_rel->isIdentifier());
 		relnn_tab_name_edt->setText(aux_rel->getTableNameRelNN());
-		attributes_tab->setButtonsEnabled(ObjectTableWidget::ALL_BUTTONS, !aux_rel->isProtected());
-		constraints_tab->setButtonsEnabled(ObjectTableWidget::ALL_BUTTONS, !aux_rel->isProtected());
+		attributes_tab->setButtonsEnabled(ObjectsTableWidget::ALL_BUTTONS, !aux_rel->isProtected());
+		constraints_tab->setButtonsEnabled(ObjectsTableWidget::ALL_BUTTONS, !aux_rel->isProtected());
 
 		//Lists the relationship attributes
 		listObjects(OBJ_COLUMN);
@@ -524,7 +500,7 @@ void RelationshipWidget::usePatternGlobalSettings(bool value)
 
 void RelationshipWidget::listObjects(ObjectType obj_type)
 {
-	ObjectTableWidget *tab=nullptr;
+	ObjectsTableWidget *tab=nullptr;
 	Relationship *rel=nullptr;
 	unsigned count, i;
 
@@ -549,7 +525,7 @@ void RelationshipWidget::listObjects(ObjectType obj_type)
 		tab->clearSelection();
 		tab->blockSignals(false);
 
-		constraints_tab->setButtonsEnabled(ObjectTableWidget::ADD_BUTTON,
+		constraints_tab->setButtonsEnabled(ObjectsTableWidget::ADD_BUTTON,
 										   attributes_tab->getRowCount() > 0);
 	}
 	catch(Exception &e)
@@ -615,23 +591,18 @@ void RelationshipWidget::listAdvancedObjects(void)
 		}
 		else if(base_rel->getRelationshipType()==BaseRelationship::RELATIONSHIP_FK)
 		{
-			tab=dynamic_cast<Table *>(base_rel->getTable(BaseRelationship::DST_TABLE));
-			dynamic_cast<Table *>(base_rel->getTable(BaseRelationship::SRC_TABLE))->getForeignKeys(constrs,false,tab);
+			Constraint *fk = base_rel->getReferenceForeignKey();
 
-			if(!base_rel->isSelfRelationship())
+			if(fk)
 			{
-				tab=dynamic_cast<Table *>(base_rel->getTable(BaseRelationship::SRC_TABLE));
-				dynamic_cast<Table *>(base_rel->getTable(BaseRelationship::DST_TABLE))->getForeignKeys(constrs,false,tab);
-			}
+				int row = 0;
 
-			count=constrs.size();
-
-			for(i=0, i1=advanced_objs_tab->getRowCount(); i < count; i++, i1++)
-			{
 				advanced_objs_tab->addRow();
-				advanced_objs_tab->setCellText(constrs[i]->getName(),i1,0);
-				advanced_objs_tab->setCellText(constrs[i]->getTypeName(),i1,1);
-				advanced_objs_tab->setRowData(QVariant::fromValue<void *>(constrs[i]), i1);
+				row = advanced_objs_tab->getRowCount() - 1;
+
+				advanced_objs_tab->setCellText(fk->getName(), row ,0);
+				advanced_objs_tab->setCellText(fk->getTypeName(), row, 1);
+				advanced_objs_tab->setRowData(QVariant::fromValue<void *>(fk), row);
 			}
 		}
 
@@ -648,52 +619,57 @@ void RelationshipWidget::listAdvancedObjects(void)
 void RelationshipWidget::showAdvancedObject(int row)
 {
 	BaseObject *object=reinterpret_cast<BaseObject *>(advanced_objs_tab->getRowData(row).value<void *>());
-	bool prot=true;
 	Table *tab=nullptr;
 	Constraint *constr=nullptr;
 	Column *col=nullptr;
 	ObjectType obj_type=object->getObjectType();
+	bool is_protected = false;
 
 	if(obj_type==OBJ_COLUMN)
 	{
-		ColumnWidget column_wgt(this);
 		col=dynamic_cast<Column *>(object);
-		column_wgt.setAttributes(this->model, this->op_list, col->getParentTable(), col);
-		column_wgt.show();
+		is_protected = col->isProtected();
+		openEditingForm<Column,ColumnWidget>(col, col->getParentTable());
 	}
 	else if(obj_type==OBJ_CONSTRAINT)
 	{
-		ConstraintWidget constraint_wgt(this);
 		constr=dynamic_cast<Constraint *>(object);
 
 		if(!constr->isAddedByRelationship())
 		{
-			prot=constr->isProtected();
+			is_protected = constr->isProtected();
 			constr->setProtected(true);
 		}
 
-		constraint_wgt.setAttributes(this->model, this->op_list, constr->getParentTable(), constr);
-		constraint_wgt.show();
-		constr->setProtected(prot);
+		openEditingForm<Constraint, ConstraintWidget>(constr, constr->getParentTable());
+
+		if(!constr->isAddedByRelationship())
+			constr->setProtected(is_protected);
 	}
 	else
 	{
-		TableWidget table_wgt(this);
+		TableWidget *table_wgt=new TableWidget;
+		BaseForm editing_form(this);
+
 		tab=dynamic_cast<Table *>(object);
 		tab->setProtected(true);
-		table_wgt.setAttributes(this->model, this->op_list, dynamic_cast<Schema *>(tab->getSchema()),
-								tab,	tab->getPosition().x(), tab->getPosition().y());
-		table_wgt.show();
+
+		table_wgt->setAttributes(this->model, this->op_list, dynamic_cast<Schema *>(tab->getSchema()),
+														 tab,	tab->getPosition().x(), tab->getPosition().y());
+
+		editing_form.setMainWidget(table_wgt);
+		editing_form.exec();
 		tab->setProtected(false);
 	}
 }
 
 template<class Class, class WidgetClass>
-int RelationshipWidget::openEditingForm(TableObject *object)
+int RelationshipWidget::openEditingForm(TableObject *object, BaseObject *parent)
 {
 	BaseForm editing_form(this);
 	WidgetClass *object_wgt=new WidgetClass;
-	object_wgt->setAttributes(this->model, this->op_list, this->object, dynamic_cast<Class *>(object));
+
+	object_wgt->setAttributes(this->model, this->op_list, (!parent ? this->object : parent), dynamic_cast<Class *>(object));
 	editing_form.setMainWidget(object_wgt);
 
 	return(editing_form.exec());
@@ -730,6 +706,60 @@ void RelationshipWidget::addObject(void)
 	}
 }
 
+void RelationshipWidget::duplicateObject(int curr_row, int new_row)
+{
+	ObjectType obj_type=BASE_OBJECT;
+	BaseObject *object = nullptr, *dup_object = nullptr;
+	Relationship *rel = dynamic_cast<Relationship *>(this->object);
+	vector<TableObject *> obj_list;
+	ObjectsTableWidget *tab = nullptr;
+	int op_id = -1;
+
+	if(!rel)
+		return;
+
+	try
+	{
+		if(sender()==attributes_tab)
+		{
+			obj_type=OBJ_COLUMN;
+			tab=attributes_tab;
+			obj_list = rel->getAttributes();
+		}
+		else
+		{
+			obj_type=OBJ_CONSTRAINT;
+			tab=constraints_tab;
+			obj_list = rel->getConstraints();
+		}
+
+		//Gets the object reference if there is an item select on table
+		if(curr_row >= 0)
+			object = reinterpret_cast<BaseObject *>(tab->getRowData(curr_row).value<void *>());
+
+		PgModelerNS::copyObject(&dup_object, object, obj_type);
+		dup_object->setName(PgModelerNS::generateUniqueName(dup_object, obj_list, false, QString("_cp")));
+
+		op_id=op_list->registerObject(dup_object, Operation::OBJECT_CREATED, new_row, rel);
+
+		rel->addObject(dynamic_cast<TableObject *>(dup_object));
+		listObjects(obj_type);
+	}
+	catch(Exception &e)
+	{
+		//If operation was registered
+		if(op_id >= 0)
+		{
+			op_list->ignoreOperationChain(true);
+			op_list->removeLastOperation();
+			op_list->ignoreOperationChain(false);
+		}
+
+		listObjects(obj_type);
+		throw Exception(e.getErrorMessage(),e.getErrorType(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
+	}
+}
+
 void RelationshipWidget::editObject(int row)
 {
 	ObjectType obj_type=OBJ_COLUMN;
@@ -765,7 +795,7 @@ void RelationshipWidget::editObject(int row)
 
 void RelationshipWidget::showObjectData(TableObject *object, int row)
 {
-	ObjectTableWidget *tab=nullptr;
+	ObjectsTableWidget *tab=nullptr;
 
 	if(object->getObjectType()==OBJ_COLUMN)
 	{
