@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2016 - Raphael Araújo e Silva <raphael@pgmodeler.com.br>
+# Copyright 2006-2018 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -22,17 +22,38 @@
 #include "snippetsconfigwidget.h"
 #include "pgmodeleruins.h"
 #include "plaintextitemdelegate.h"
+#include "datamanipulationform.h"
+
+map<QString, QString> SQLExecutionWidget::cmd_history;
+
+int SQLExecutionWidget::cmd_history_max_len = 1000;
+const QString SQLExecutionWidget::COLUMN_NULL_VALUE = QString("␀");
 
 SQLExecutionWidget::SQLExecutionWidget(QWidget * parent) : QWidget(parent)
 {
 	setupUi(this);
 
+	result_model = nullptr;
+
 	sql_cmd_txt=PgModelerUiNS::createNumberedTextEditor(sql_cmd_wgt);
+	cmd_history_txt=PgModelerUiNS::createNumberedTextEditor(cmd_history_parent);
+	cmd_history_txt->setCustomContextMenuEnabled(false);
 
 	cmd_history_txt->setTabStopWidth(sql_cmd_txt->getTabWidth());
 	cmd_history_txt->setContextMenuPolicy(Qt::CustomContextMenu);
+	cmd_history_txt->setReadOnly(true);
 	cmd_history_txt->installEventFilter(this);
 
+	output_tbw->widget(2)->installEventFilter(this);
+
+	find_history_wgt = new FindReplaceWidget(cmd_history_txt, find_history_parent);
+	QVBoxLayout *layout = new QVBoxLayout;
+	layout->setContentsMargins(0,0,0,0);
+	layout->addWidget(find_history_wgt);
+	find_history_parent->setLayout(layout);
+	find_history_parent->setVisible(false);
+	connect(find_history_wgt->hide_tb, SIGNAL(clicked(bool)), find_history_parent, SLOT(hide()));
+
 	sql_cmd_hl=new SyntaxHighlighter(sql_cmd_txt, false);
 	sql_cmd_hl->loadConfiguration(GlobalAttributes::SQL_HIGHLIGHT_CONF_PATH);
 
@@ -48,65 +69,104 @@ SQLExecutionWidget::SQLExecutionWidget(QWidget * parent) : QWidget(parent)
 	sql_file_dlg.setModal(true);
 
 	snippets_tb->setMenu(&snippets_menu);
-	code_compl_wgt=new CodeCompletionWidget(sql_cmd_txt);
+	code_compl_wgt=new CodeCompletionWidget(sql_cmd_txt, true);
 
 	find_replace_wgt=new FindReplaceWidget(sql_cmd_txt, find_wgt_parent);
 	QHBoxLayout *hbox=new QHBoxLayout(find_wgt_parent);
 	hbox->setContentsMargins(0,0,0,0);
 	hbox->addWidget(find_replace_wgt);
 	find_wgt_parent->setVisible(false);
+	connect(find_replace_wgt->hide_tb, SIGNAL(clicked(bool)), find_tb, SLOT(toggle()));
 
 	run_sql_tb->setToolTip(run_sql_tb->toolTip() + QString(" (%1)").arg(run_sql_tb->shortcut().toString()));
 	export_tb->setToolTip(export_tb->toolTip() + QString(" (%1)").arg(export_tb->shortcut().toString()));
-	load_tb->setToolTip(load_tb->toolTip() + QString(" (%1)").arg(load_tb->shortcut().toString()));
-	save_tb->setToolTip(save_tb->toolTip() + QString(" (%1)").arg(save_tb->shortcut().toString()));
+	file_tb->setToolTip(file_tb->toolTip() + QString(" (%1)").arg(file_tb->shortcut().toString()));
 	output_tb->setToolTip(output_tb->toolTip() + QString(" (%1)").arg(output_tb->shortcut().toString()));
 	find_tb->setToolTip(find_tb->toolTip() + QString(" (%1)").arg(find_tb->shortcut().toString()));
 
 	results_tbw->setItemDelegate(new PlainTextItemDelegate(this, true));
 
+	action_load=new QAction(QIcon(PgModelerUiNS::getIconPath("abrir")), trUtf8("Load"), this);
+	action_save=new QAction(QIcon(PgModelerUiNS::getIconPath("salvar")), trUtf8("Save"), this);
+	action_save_as=new QAction(QIcon(PgModelerUiNS::getIconPath("salvar_como")), trUtf8("Save as"), this);
+
+	file_menu.addAction(action_load);
+	file_menu.addAction(action_save);
+	file_menu.addAction(action_save_as);
+	file_tb->setMenu(&file_menu);
+
+	connect(action_load, SIGNAL(triggered(bool)), this, SLOT(loadCommands()));
+	connect(action_save, SIGNAL(triggered(bool)), this, SLOT(saveCommands()));
+	connect(action_save_as, SIGNAL(triggered(bool)), this, SLOT(saveCommands()));
+
 	connect(clear_btn, SIGNAL(clicked(void)), this, SLOT(clearAll(void)));
 	connect(sql_cmd_txt, SIGNAL(textChanged(void)), this, SLOT(enableCommandButtons(void)));
 	connect(run_sql_tb, SIGNAL(clicked(void)), this, SLOT(runSQLCommand(void)));
-	connect(save_tb, SIGNAL(clicked(void)), this, SLOT(saveCommands(void)));
-	connect(load_tb, SIGNAL(clicked(void)), this, SLOT(loadCommands(void)));
 	connect(find_tb, SIGNAL(toggled(bool)), find_wgt_parent, SLOT(setVisible(bool)));
 	connect(output_tb, SIGNAL(toggled(bool)), this, SLOT(toggleOutputPane(bool)));
 
 	//Signal handling with C++11 lambdas Slots
-	connect(results_tbw, &QTableWidget::itemPressed,
-			[=](){ SQLExecutionWidget::copySelection(results_tbw); });
+	connect(results_tbw, &QTableView::pressed,
+			[&](){ SQLExecutionWidget::copySelection(results_tbw); });
 
 	connect(export_tb, &QToolButton::clicked,
-			[=](){ SQLExecutionWidget::exportResults(results_tbw); });
+			[&](){ SQLExecutionWidget::exportResults(results_tbw); });
+
+	connect(close_file_tb, &QToolButton::clicked,
+	[&](){
+			if(clearAll() == QDialog::Accepted)
+			{
+				filename_edt->clear();
+				filename_wgt->setVisible(false);
+			}
+	});
 
 	connect(&snippets_menu, SIGNAL(triggered(QAction*)), this, SLOT(selectSnippet(QAction *)));
-
-	connect(code_compl_wgt, SIGNAL(s_wordSelected(QString)), this, SLOT(handleSelectedWord(QString)));
-
 	connect(cmd_history_txt, SIGNAL(customContextMenuRequested(QPoint)), this, SLOT(showHistoryContextMenu()));
 
 	configureSnippets();
 	toggleOutputPane(false);
+	filename_wgt->setVisible(false);
 	v_splitter->handle(1)->installEventFilter(this);
 }
 
+SQLExecutionWidget::~SQLExecutionWidget(void)
+{
+	if(result_model)
+	{
+		results_tbw->blockSignals(true);
+		results_tbw->setModel(nullptr);
+		delete(result_model);
+	}
+}
+
 bool SQLExecutionWidget::eventFilter(QObject *object, QEvent *event)
 {
-	if(event->type() == QEvent::MouseButtonDblClick &&
-			qobject_cast<QSplitterHandle *>(object) == v_splitter->handle(1))
+	if(event->type() == QEvent::MouseButtonDblClick && object == v_splitter->handle(1))
 	{
 		output_tb->setChecked(!v_splitter->handle(1)->isEnabled());
 		return(true);
 	}
 	else if(event->type()== QEvent::MouseButtonPress &&
 					dynamic_cast<QMouseEvent *>(event)->button()==Qt::MiddleButton &&
-					qobject_cast<QPlainTextEdit *>(object)==cmd_history_txt &&
+					object == cmd_history_txt &&
 					cmd_history_txt->textCursor().hasSelection())
 	{
 		sql_cmd_txt->appendPlainText(cmd_history_txt->textCursor().selectedText());
 		return(true);
 	}
+	else if(event->type() == QEvent::Show && object == output_tbw->widget(2))
+	{
+		if(cmd_history_txt->toPlainText().count(QChar('\n')) !=
+			 cmd_history[sql_cmd_conn.getConnectionId(true,true)].count(QChar('\n')))
+		{
+			cmd_history_txt->clear();
+			cmd_history_txt->appendPlainText(cmd_history[sql_cmd_conn.getConnectionId(true,true)]);
+			cmd_history_txt->updateLineNumbers();
+		}
+
+		return(true);
+	}
 
 	return(QWidget::eventFilter(object, event));
 }
@@ -126,7 +186,6 @@ void SQLExecutionWidget::enableCommandButtons(void)
 	run_sql_tb->setEnabled(!sql_cmd_txt->toPlainText().isEmpty());
 	find_tb->setEnabled(!sql_cmd_txt->toPlainText().isEmpty());
 	clear_btn->setEnabled(run_sql_tb->isEnabled());
-	save_tb->setEnabled(run_sql_tb->isEnabled());
 }
 
 void SQLExecutionWidget::fillResultsTable(ResultSet &res)
@@ -139,7 +198,21 @@ void SQLExecutionWidget::fillResultsTable(ResultSet &res)
 		aux_conn.setConnectionParams(sql_cmd_conn.getConnectionParams());
 		export_tb->setEnabled(res.getTupleCount() > 0);
 		catalog.setConnection(aux_conn);
-		fillResultsTable(catalog, res, results_tbw);
+
+		results_tbw->setSortingEnabled(false);
+		results_tbw->blockSignals(true);
+		results_tbw->setUpdatesEnabled(false);
+		results_tbw->setModel(nullptr);
+
+		if(result_model)
+			delete(result_model);
+
+		result_model = new ResultSetModel(res, catalog);
+
+		results_tbw->setModel(result_model);
+		results_tbw->resizeColumnsToContents();
+		results_tbw->setUpdatesEnabled(true);
+		results_tbw->blockSignals(false);
 	}
 	catch(Exception &e)
 	{
@@ -160,10 +233,9 @@ void SQLExecutionWidget::resizeEvent(QResizeEvent *event)
 		style=Qt::ToolButtonIconOnly;
 
 
-	if(load_tb->toolButtonStyle()!=style)
+	if(file_tb->toolButtonStyle()!=style)
 	{
-		load_tb->setToolButtonStyle(style);
-		save_tb->setToolButtonStyle(style);
+		file_tb->setToolButtonStyle(style);
 		run_sql_tb->setToolButtonStyle(style);
 		clear_btn->setToolButtonStyle(style);
 		find_tb->setToolButtonStyle(style);
@@ -191,7 +263,9 @@ void SQLExecutionWidget::fillResultsTable(Catalog &catalog, ResultSet &res, QTab
 		results_tbw->setRowCount(0);
 		results_tbw->setColumnCount(col_cnt);
 		results_tbw->verticalHeader()->setVisible(true);
+		results_tbw->setSortingEnabled(false);
 		results_tbw->blockSignals(true);
+		results_tbw->setUpdatesEnabled(false);
 
 		//Configuring the grid columns with the names of retrived table columns
 		for(col=0; col < col_cnt; col++)
@@ -219,7 +293,7 @@ void SQLExecutionWidget::fillResultsTable(Catalog &catalog, ResultSet &res, QTab
 		for(col=0; col < col_cnt; col++)
 		{
 			item=results_tbw->horizontalHeaderItem(col);
-			item->setToolTip(res.getColumnName(col) + QString(" [%1]").arg(type_names[res.getColumnTypeId(col)]));
+			item->setToolTip(type_names[res.getColumnTypeId(col)]);
 			item->setData(Qt::UserRole, type_names[res.getColumnTypeId(col)]);
 		}
 
@@ -244,8 +318,10 @@ void SQLExecutionWidget::fillResultsTable(Catalog &catalog, ResultSet &res, QTab
 					{
 						item->setText(res.getColumnValue(col));
 
+						/* When storing column values in the QTableWidget items we need distinguish empty from null values
+						 * Since it may affect the generation of SQL like delete when the field value is used somehow (see DataManipulationForm::getDMLCommand) */
 						if(store_data)
-							item->setData(Qt::UserRole, item->text());
+							item->setData(Qt::UserRole, res.isColumnValueNull(col) ? COLUMN_NULL_VALUE : item->text());
 					}
 
 					results_tbw->setItem(row, col, item);
@@ -258,6 +334,7 @@ void SQLExecutionWidget::fillResultsTable(Catalog &catalog, ResultSet &res, QTab
 			while(res.accessTuple(ResultSet::NEXT_TUPLE));
 		}
 
+		results_tbw->setUpdatesEnabled(true);
 		results_tbw->blockSignals(false);
 		results_tbw->resizeColumnsToContents();
 		results_tbw->resizeRowsToContents();
@@ -276,14 +353,14 @@ void SQLExecutionWidget::showError(Exception &e)
 
 	PgModelerUiNS::createOutputListItem(msgoutput_lst,
 																			QString("%1 %2").arg(time_str).arg(e.getErrorMessage()),
-																			QPixmap(QString(":/icones/icones/msgbox_erro.png")), false);
+																			QPixmap(PgModelerUiNS::getIconPath("msgbox_erro")), false);
 
 	if(e.getErrorType()==ERR_CONNECTION_TIMEOUT ||
 		 e.getErrorType()==ERR_CONNECTION_BROKEN)
 	{
 		PgModelerUiNS::createOutputListItem(msgoutput_lst,
 																				QString("%1 %2").arg(time_str).arg(trUtf8("No results retrieved or changes done due to the error above.")),
-																				QPixmap(QString(":/icones/icones/msgbox_alerta.png")), false);
+																				QPixmap(PgModelerUiNS::getIconPath("msgbox_alerta")), false);
 	}
 
 	msgoutput_lst->setVisible(true);
@@ -296,26 +373,63 @@ void SQLExecutionWidget::showError(Exception &e)
 	output_tbw->setTabEnabled(0, false);
 }
 
-void SQLExecutionWidget::registerSQLCommand(const QString &cmd, unsigned rows, const QString &error)
+void SQLExecutionWidget::addToSQLHistory(const QString &cmd, unsigned rows, const QString &error)
 {
 	if(!cmd.isEmpty())
 	{
 		QString fmt_cmd;
-		fmt_cmd=QString("--\n-- Executed at [%1] -- \n").arg(QTime::currentTime().toString(QString("hh:mm:ss.zzz")));
-		fmt_cmd+=cmd;
-		fmt_cmd+=QChar('\n');
+
+		if(!cmd_history_txt->toPlainText().isEmpty())
+			fmt_cmd += QString("\n");
+
+		fmt_cmd += QString("-- Executed at [%1] -- \n").arg(QDateTime::currentDateTime().toString(QString("yyyy-MM-dd hh:mm:ss.zzz")));
+		fmt_cmd += cmd;
+		fmt_cmd += QChar('\n');
 
 		if(!error.isEmpty())
-			fmt_cmd+=QString("/*\n%1\n*/\n").arg(error);
+		{
+			fmt_cmd += QString("-- Query failed --\n");
+			fmt_cmd += QString("/*\n%1\n*/\n").arg(error);
+		}
 		else
-			fmt_cmd+=QString("-- Rows retrieved: %1\n").arg(rows);
+			fmt_cmd += QString("-- Rows retrieved: %1\n").arg(rows);
 
-		fmt_cmd+=QString("--\n");
+		if(!fmt_cmd.trimmed().endsWith(ParsersAttributes::DDL_END_TOKEN))
+			fmt_cmd += ParsersAttributes::DDL_END_TOKEN + QChar('\n');
 
-		cmd_history_txt->appendPlainText(fmt_cmd);
+		SQLExecutionWidget::validateSQLHistoryLength(sql_cmd_conn.getConnectionId(true,true), fmt_cmd, cmd_history_txt);
 	}
 }
 
+void SQLExecutionWidget::validateSQLHistoryLength(const QString &conn_id, const QString &fmt_cmd, NumberedTextEditor *cmd_history_txt)
+{
+	QString cmds;
+	int ln_count = 0;
+
+	cmds = cmd_history[conn_id];
+	ln_count = cmds.count(QChar('\n'));
+	ln_count += fmt_cmd.count(QChar('\n'));
+
+	if(ln_count > cmd_history_max_len)
+	{
+		QStringList buffer = cmds.split(QChar('\n'));
+		cmds = buffer.mid(buffer.size()/2).join(QChar('\n'));
+		cmds = cmds.mid(cmds.indexOf(ParsersAttributes::DDL_END_TOKEN) + ParsersAttributes::DDL_END_TOKEN.length());
+		cmd_history[conn_id] = cmds.trimmed();
+
+		if(cmd_history_txt)
+		{
+			cmd_history_txt->clear();
+			cmd_history_txt->appendPlainText(cmds);
+		}
+	}
+
+	cmd_history[conn_id].append(fmt_cmd);
+
+	if(cmd_history_txt)
+		cmd_history_txt->appendPlainText(fmt_cmd);
+}
+
 void SQLExecutionWidget::runSQLCommand(void)
 {
 	QString cmd=sql_cmd_txt->textCursor().selectedText();
@@ -324,6 +438,7 @@ void SQLExecutionWidget::runSQLCommand(void)
 	{
 		ResultSet res;
 		QStringList conn_notices;
+		qint64 start_exec=0, end_exec=0, total_exec = 0;
 
 		output_tb->setChecked(true);
 
@@ -344,10 +459,12 @@ void SQLExecutionWidget::runSQLCommand(void)
 		}
 
 		QApplication::setOverrideCursor(Qt::WaitCursor);
-		sql_cmd_conn.executeDMLCommand(cmd, res);
-		conn_notices=sql_cmd_conn.getNotices();
 
-		registerSQLCommand(cmd, res.getTupleCount());
+		start_exec=QDateTime::currentDateTime().toMSecsSinceEpoch();
+		sql_cmd_conn.executeDMLCommand(cmd, res);
+
+		conn_notices=sql_cmd_conn.getNotices();
+		addToSQLHistory(cmd, res.getTupleCount());
 
 		output_tbw->setTabEnabled(0, !res.isEmpty());
 		results_parent->setVisible(!res.isEmpty());
@@ -356,7 +473,7 @@ void SQLExecutionWidget::runSQLCommand(void)
 		if(!res.isEmpty())
 		{
 			fillResultsTable(res);
-			output_tbw->setTabText(0, trUtf8("Results (%1)").arg(res.getTupleCount()));
+			output_tbw->setTabText(0, trUtf8("Results (%1)").arg(results_tbw->model()->rowCount()));
 			output_tbw->setCurrentIndex(0);
 		}
 		else
@@ -371,15 +488,19 @@ void SQLExecutionWidget::runSQLCommand(void)
 		{
 			PgModelerUiNS::createOutputListItem(msgoutput_lst,
 																					PgModelerUiNS::formatMessage(QString("[%1]: %2").arg(QTime::currentTime().toString(QString("hh:mm:ss.zzz"))).arg(notice)),
-																					QPixmap(QString(":/icones/icones/msgbox_alerta.png")));
+																					QPixmap(PgModelerUiNS::getIconPath("msgbox_alerta")));
 		}
 
+		end_exec=QDateTime::currentDateTime().toMSecsSinceEpoch();
+		total_exec = end_exec - start_exec;
+
 		PgModelerUiNS::createOutputListItem(msgoutput_lst,
-																				PgModelerUiNS::formatMessage(trUtf8("[%1]: SQL command successfully executed. <em>%2 <strong>%3</strong></em>")
+																				PgModelerUiNS::formatMessage(trUtf8("[%1]: SQL command successfully executed in <em><strong>%2</strong></em>. <em>%3 <strong>%4</strong></em>")
 																																		 .arg(QTime::currentTime().toString(QString("hh:mm:ss.zzz")))
+																																		 .arg(total_exec >= 1000 ? QString("%1 s").arg(total_exec/1000) : QString("%1 ms").arg(total_exec))
 																																		 .arg(res.isEmpty() ? trUtf8("Rows affected") :  trUtf8("Rows retrieved"))
 																																		 .arg(res.getTupleCount())),
-																				QPixmap(QString(":/icones/icones/msgbox_info.png")));
+																				QPixmap(PgModelerUiNS::getIconPath("msgbox_info")));
 
 		output_tbw->setTabText(1, trUtf8("Messages (%1)").arg(msgoutput_lst->count()));
 
@@ -387,7 +508,7 @@ void SQLExecutionWidget::runSQLCommand(void)
 	}
 	catch(Exception &e)
 	{
-		registerSQLCommand(cmd, 0, e.getErrorMessage());
+		addToSQLHistory(cmd, 0, e.getErrorMessage());
 		QApplication::restoreOverrideCursor();
 		sql_cmd_conn.close();
 		showError(e);
@@ -396,22 +517,35 @@ void SQLExecutionWidget::runSQLCommand(void)
 
 void SQLExecutionWidget::saveCommands(void)
 {
-	sql_file_dlg.setWindowTitle(trUtf8("Save SQL commands"));
-	sql_file_dlg.setAcceptMode(QFileDialog::AcceptSave);
-	sql_file_dlg.exec();
+	bool browse_file = (sender() == action_save_as || filename_edt->text().isEmpty());
+	QString filename = filename_edt->text();
 
-	if(sql_file_dlg.result()==QDialog::Accepted)
+	if(browse_file)
+	{
+		sql_file_dlg.setWindowTitle(trUtf8("Save SQL commands"));
+		sql_file_dlg.setAcceptMode(QFileDialog::AcceptSave);
+		sql_file_dlg.exec();
+
+		if(sql_file_dlg.result()==QDialog::Accepted)
+			filename = sql_file_dlg.selectedFiles().at(0);
+	}
+	else
+		filename = filename_edt->text();
+
+	if(!filename.isEmpty())
 	{
 		QFile file;
-		file.setFileName(sql_file_dlg.selectedFiles().at(0));
+		file.setFileName(filename);
 
 		if(!file.open(QFile::WriteOnly))
-			throw Exception(Exception::getErrorMessage(ERR_FILE_DIR_NOT_ACCESSED)
-							.arg(sql_file_dlg.selectedFiles().at(0))
-							, ERR_FILE_DIR_NOT_ACCESSED ,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+			throw Exception(Exception::getErrorMessage(ERR_FILE_DIR_NOT_ACCESSED).arg(filename),
+											ERR_FILE_DIR_NOT_ACCESSED ,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 
 		file.write(sql_cmd_txt->toPlainText().toUtf8());
 		file.close();
+
+		filename_edt->setText(filename);
+		filename_wgt->setVisible(true);
 	}
 }
 
@@ -434,10 +568,13 @@ void SQLExecutionWidget::loadCommands(void)
 		sql_cmd_txt->clear();
 		sql_cmd_txt->setPlainText(file.readAll());
 		file.close();
+
+		filename_edt->setText(sql_file_dlg.selectedFiles().at(0));
+		filename_wgt->setVisible(true);
 	}
 }
 
-void SQLExecutionWidget::exportResults(QTableWidget *results_tbw)
+void SQLExecutionWidget::exportResults(QTableView *results_tbw)
 {
 	if(!results_tbw)
 		throw Exception(ERR_OPR_NOT_ALOC_OBJECT ,__PRETTY_FUNCTION__,__FILE__,__LINE__);
@@ -463,60 +600,32 @@ void SQLExecutionWidget::exportResults(QTableWidget *results_tbw)
 							.arg(csv_file_dlg.selectedFiles().at(0))
 							, ERR_FILE_DIR_NOT_ACCESSED ,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 
-		file.write(generateCSVBuffer(results_tbw, 0, 0, results_tbw->rowCount(), results_tbw->columnCount()));
+		QApplication::setOverrideCursor(Qt::WaitCursor);
+		results_tbw->setUpdatesEnabled(false);
+		results_tbw->blockSignals(true);
+		results_tbw->selectAll();
+
+		file.write(generateCSVBuffer(results_tbw));
 		file.close();
+
+		results_tbw->clearSelection();
+		results_tbw->blockSignals(false);
+		results_tbw->setUpdatesEnabled(true);
+		QApplication::restoreOverrideCursor();
 	}
 }
 
-QByteArray SQLExecutionWidget::generateCSVBuffer(QTableWidget *results_tbw, int start_row, int start_col, int row_cnt, int col_cnt)
-{
-	if(!results_tbw)
-		throw Exception(ERR_OPR_NOT_ALOC_OBJECT ,__PRETTY_FUNCTION__,__FILE__,__LINE__);
-
-	QByteArray buf;
-
-	//If the selection interval is valid
-	if(start_row >=0 && start_col >=0 &&
-			start_row + row_cnt <= results_tbw->rowCount() &&
-			start_col + col_cnt <= results_tbw->columnCount())
-	{
-		int col=0, row=0,
-				max_col=start_col + col_cnt,
-				max_row=start_row + row_cnt;
-
-		//Creating the header of csv
-		for(col=start_col; col < max_col; col++)
-		{
-			buf.append(QString("\"%1\"").arg(results_tbw->horizontalHeaderItem(col)->text()));
-			buf.append(';');
-		}
-
-		buf.append('\n');
-
-		//Creating the content
-		for(row=start_row; row < max_row; row++)
-		{
-			for(col=start_col; col < max_col; col++)
-			{
-				buf.append(QString("\"%1\"").arg(results_tbw->item(row, col)->text()));
-				buf.append(';');
-			}
-
-			buf.append('\n');
-		}
-	}
-
-	return(buf);
-}
-
-void SQLExecutionWidget::clearAll(void)
+int SQLExecutionWidget::clearAll(void)
 {
 	Messagebox msg_box;
+	int res = 0;
 
 	msg_box.show(trUtf8("The SQL input field and the results grid will be cleared! Want to proceed?"),
-				 Messagebox::CONFIRM_ICON, Messagebox::YES_NO_BUTTONS);
+							 Messagebox::CONFIRM_ICON, Messagebox::YES_NO_BUTTONS);
 
-	if(msg_box.result()==QDialog::Accepted)
+	res = msg_box.result();
+
+	if(res==QDialog::Accepted)
 	{
 		sql_cmd_txt->setPlainText(QString());
 		msgoutput_lst->clear();
@@ -524,30 +633,115 @@ void SQLExecutionWidget::clearAll(void)
 		results_parent->setVisible(false);
 		export_tb->setEnabled(false);
 	}
+
+	return(res);
 }
 
-void SQLExecutionWidget::copySelection(QTableWidget *results_tbw, bool use_popup)
+QByteArray SQLExecutionWidget::generateCSVBuffer(QTableView *results_tbw)
+{
+	return(generateBuffer(results_tbw, QChar(';'), true, true));
+}
+
+QByteArray SQLExecutionWidget::generateTextBuffer(QTableView *results_tbw)
+{
+	return(generateBuffer(results_tbw, QChar('\t'), false, false));
+}
+
+QByteArray SQLExecutionWidget::generateBuffer(QTableView *results_tbw, QChar separator, bool incl_col_names, bool use_quotes)
 {
 	if(!results_tbw)
 		throw Exception(ERR_OPR_NOT_ALOC_OBJECT ,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 
-	QList<QTableWidgetSelectionRange> sel_ranges=results_tbw->selectedRanges();
+	if(!results_tbw->selectionModel())
+		return (QByteArray());
 
-	if(sel_ranges.count()==1 && (!use_popup || (use_popup && QApplication::mouseButtons()==Qt::RightButton)))
+	QAbstractItemModel *model = results_tbw->model();
+	QModelIndexList sel_indexes = results_tbw->selectionModel()->selectedIndexes();
+	QByteArray buf;
+	QStringList line;
+	QModelIndex index;
+	QString str_pattern = use_quotes ? QString("\"%1\"") : QString("%1");
+	int start_row = -1, start_col = -1,
+			row_cnt = 0, col_cnt = 0;
+
+	start_row = sel_indexes.at(0).row();
+	start_col = sel_indexes.at(0).column();
+	row_cnt = (sel_indexes.last().row() - start_row) + 1;
+	col_cnt = (sel_indexes.last().column() - start_col) + 1;
+
+	int col=0, row=0,
+			max_col=start_col + col_cnt,
+			max_row=start_row + row_cnt;
+
+	if(incl_col_names)
 	{
-		QMenu copy_menu;
+		//Creating the header
+		for(col=start_col; col < max_col; col++)
+			line.append(str_pattern.arg(model->headerData(col, Qt::Horizontal).toString()));
+
+		buf.append(line.join(separator));
+		buf.append('\n');
+		line.clear();
+	}
+
+	//Creating the content
+	for(row=start_row; row < max_row; row++)
+	{
+		for(col=start_col; col < max_col; col++)
+		{
+			index = model->index(row, col);
+			line.append(str_pattern.arg(index.data().toString()));
+		}
+
+		buf.append(line.join(separator));
+		line.clear();
+		buf.append('\n');
+	}
+
+	return(buf);
+}
+
+void SQLExecutionWidget::copySelection(QTableView *results_tbw, bool use_popup, bool csv_is_default)
+{
+	if(!results_tbw)
+		throw Exception(ERR_OPR_NOT_ALOC_OBJECT ,__PRETTY_FUNCTION__,__FILE__,__LINE__);
+
+	QItemSelectionModel *selection = results_tbw->selectionModel();
+
+	if(selection && (!use_popup || (use_popup && QApplication::mouseButtons()==Qt::RightButton)))
+	{
+		QModelIndexList sel_indexes = selection->selectedIndexes();
+		QMenu copy_menu, copy_mode_menu;
+		QAction *act = nullptr, *act_csv = nullptr, *act_txt = nullptr;
 
 		if(use_popup)
-			copy_menu.addAction(trUtf8("Copy selection"));
-
-		if(!use_popup || (use_popup && copy_menu.exec(QCursor::pos())))
 		{
-			QTableWidgetSelectionRange selection=sel_ranges.at(0);
+			act = copy_menu.addAction(trUtf8("Copy selection"));
+			act_txt = copy_mode_menu.addAction(trUtf8("Plain format"));
+			act_csv = copy_mode_menu.addAction(trUtf8("CVS format"));
+			act->setMenu(&copy_mode_menu);
+			act = copy_menu.exec(QCursor::pos());
+		}
+
+		if(!use_popup || act)
+		{
+			//QTableWidgetSelectionRange selection=sel_ranges.at(0);
+			QByteArray buf;
+
+			if((use_popup && act == act_csv) || (!use_popup && csv_is_default))
+			{
+				//Generates the csv buffer and assigns it to application's clipboard
+				buf=generateCSVBuffer(results_tbw);
+
+				/* Making DataManipulationForm instances know that the clipboard has csv buffer
+				 * in order to paste the contents properly */
+				DataManipulationForm::setHasCsvClipboard(true);
+			}
+			else if((use_popup && act == act_txt) || (!use_popup && !csv_is_default))
+			{
+				buf=generateTextBuffer(results_tbw);
+			}
 
-			//Generates the csv buffer and assigns it to application's clipboard
-			QByteArray buf=generateCSVBuffer(results_tbw,
-																			 selection.topRow(), selection.leftColumn(),
-																			 selection.rowCount(), selection.columnCount());
 			qApp->clipboard()->setText(buf);
 		}
 	}
@@ -562,17 +756,6 @@ void SQLExecutionWidget::selectSnippet(QAction *act)
 	sql_cmd_txt->setTextCursor(cursor);
 }
 
-void SQLExecutionWidget::handleSelectedWord(QString word)
-{
-	if(SnippetsConfigWidget::isSnippetExists(word))
-	{
-		QTextCursor tc=sql_cmd_txt->textCursor();
-		tc.movePosition(QTextCursor::PreviousWord, QTextCursor::KeepAnchor);
-		tc.removeSelectedText();
-		tc.insertText(SnippetsConfigWidget::getParsedSnippet(word));
-	}
-}
-
 void SQLExecutionWidget::toggleOutputPane(bool visible)
 {
 	if(!visible)
@@ -599,10 +782,136 @@ void SQLExecutionWidget::configureSnippets(void)
 { 
 	SnippetsConfigWidget::configureSnippetsMenu(&snippets_menu);
 	code_compl_wgt->configureCompletion(nullptr, sql_cmd_hl);
-	code_compl_wgt->clearCustomItems();
-	code_compl_wgt->insertCustomItems(SnippetsConfigWidget::getAllSnippetsAttribute(ParsersAttributes::ID),
-									  SnippetsConfigWidget::getAllSnippetsAttribute(ParsersAttributes::LABEL),
-									  QPixmap(QString(":/icones/icones/codesnippet.png")));
+}
+
+void SQLExecutionWidget::saveSQLHistory(void)
+{
+	try
+	{
+		SchemaParser schparser;
+		attribs_map attribs;
+		QString commands;
+		QByteArray buffer;
+		QFile file;
+
+		for(auto hist : cmd_history)
+		{
+			attribs[ParsersAttributes::CONNECTION] = hist.first;
+			attribs[ParsersAttributes::COMMANDS] = hist.second;
+			schparser.ignoreEmptyAttributes(true);
+			commands += schparser.getCodeDefinition(GlobalAttributes::TMPL_CONFIGURATIONS_DIR +
+																							GlobalAttributes::DIR_SEPARATOR +
+																							GlobalAttributes::SCHEMAS_DIR +
+																							GlobalAttributes::DIR_SEPARATOR +
+																							ParsersAttributes::COMMANDS +
+																							GlobalAttributes::SCHEMA_EXT, attribs);
+		}
+
+		schparser.loadFile(GlobalAttributes::TMPL_CONFIGURATIONS_DIR +
+											 GlobalAttributes::DIR_SEPARATOR +
+											 GlobalAttributes::SCHEMAS_DIR +
+											 GlobalAttributes::DIR_SEPARATOR +
+											 GlobalAttributes::SQL_HISTORY_CONF +
+											 GlobalAttributes::SCHEMA_EXT);
+
+		attribs.clear();
+		attribs[ParsersAttributes::COMMANDS] = commands;
+		buffer.append(schparser.getCodeDefinition(attribs));
+
+
+		file.setFileName(GlobalAttributes::CONFIGURATIONS_DIR +
+										 GlobalAttributes::DIR_SEPARATOR +
+										 GlobalAttributes::SQL_HISTORY_CONF +
+										 GlobalAttributes::CONFIGURATION_EXT);
+
+		if(!file.open(QFile::WriteOnly))
+			throw Exception(Exception::getErrorMessage(ERR_FILE_DIR_NOT_ACCESSED).arg(file.fileName()),
+											ERR_FILE_DIR_NOT_ACCESSED, __PRETTY_FUNCTION__, __FILE__ ,__LINE__);
+
+		file.write(buffer);
+		file.close();
+	}
+	catch(Exception &e)
+	{
+		throw Exception(e.getErrorMessage(), e.getErrorType(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
+	}
+}
+
+void SQLExecutionWidget::loadSQLHistory(void)
+{
+	try
+	{
+		XMLParser xmlparser;
+		attribs_map attribs;
+
+		xmlparser.setDTDFile(GlobalAttributes::TMPL_CONFIGURATIONS_DIR +
+												 GlobalAttributes::DIR_SEPARATOR +
+												 GlobalAttributes::OBJECT_DTD_DIR +
+												 GlobalAttributes::DIR_SEPARATOR +
+												 GlobalAttributes::SQL_HISTORY_CONF +
+												 GlobalAttributes::OBJECT_DTD_EXT,
+												 GlobalAttributes::SQL_HISTORY_CONF);
+
+		xmlparser.loadXMLFile(GlobalAttributes::CONFIGURATIONS_DIR +
+													GlobalAttributes::DIR_SEPARATOR +
+													GlobalAttributes::SQL_HISTORY_CONF +
+													GlobalAttributes::CONFIGURATION_EXT);
+
+		cmd_history.clear();
+
+		if(xmlparser.accessElement(XMLParser::CHILD_ELEMENT))
+		{
+			do
+			{
+				if(xmlparser.getElementName() == ParsersAttributes::COMMANDS)
+				{
+					xmlparser.getElementAttributes(attribs);
+					xmlparser.savePosition();
+
+					if(xmlparser.accessElement(XMLParser::CHILD_ELEMENT))
+						cmd_history[attribs[ParsersAttributes::CONNECTION]].append(xmlparser.getElementContent());
+
+					xmlparser.restorePosition();
+				}
+			}
+			while(xmlparser.accessElement(XMLParser::NEXT_ELEMENT));
+		}
+	}
+	catch(Exception &e)
+	{
+		throw Exception(e.getErrorMessage(), e.getErrorType(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
+	}
+}
+
+void SQLExecutionWidget::destroySQLHistory(void)
+{
+	Messagebox msg_box;
+
+	msg_box.show(trUtf8("This action will wipe out all the SQL commands history for all connections! Do you really want to proceed?"),
+								Messagebox::CONFIRM_ICON, Messagebox::YES_NO_BUTTONS);
+
+	if(msg_box.result() == QDialog::Accepted)
+	{
+		QFile::remove(GlobalAttributes::CONFIGURATIONS_DIR +
+									GlobalAttributes::DIR_SEPARATOR +
+									GlobalAttributes::SQL_HISTORY_CONF +
+									GlobalAttributes::CONFIGURATION_EXT);
+
+		SQLExecutionWidget::cmd_history.clear();
+	}
+}
+
+void SQLExecutionWidget::setSQLHistoryMaxLength(int len)
+{
+	if(len < 100 || len > 2000)
+		len = 1000;
+
+	SQLExecutionWidget::cmd_history_max_len = len;
+}
+
+int SQLExecutionWidget::getSQLHistoryMaxLength(void)
+{
+	return(SQLExecutionWidget::cmd_history_max_len);
 }
 
 void SQLExecutionWidget::enableSQLExecution(bool enable)
@@ -610,9 +919,7 @@ void SQLExecutionWidget::enableSQLExecution(bool enable)
 	try
 	{
 		sql_cmd_txt->setEnabled(enable);
-		load_tb->setEnabled(enable);
 		snippets_tb->setEnabled(enable);
-		save_tb->setEnabled(enable && !sql_cmd_txt->toPlainText().isEmpty());
 		clear_btn->setEnabled(enable && !sql_cmd_txt->toPlainText().isEmpty());
 		run_sql_tb->setEnabled(enable && !sql_cmd_txt->toPlainText().isEmpty());
 		find_tb->setEnabled(enable);
@@ -627,13 +934,50 @@ void SQLExecutionWidget::enableSQLExecution(bool enable)
 void SQLExecutionWidget::showHistoryContextMenu(void)
 {
 	QMenu *ctx_menu=cmd_history_txt->createStandardContextMenu();
-	QAction *act=new QAction(trUtf8("Clear history"), ctx_menu);
+	QAction *action_clear = new QAction(QPixmap(PgModelerUiNS::getIconPath("limpartexto")), trUtf8("Clear history"), ctx_menu),
+			*action_save = new QAction(QPixmap(PgModelerUiNS::getIconPath("salvar")), trUtf8("Save history"), ctx_menu),
+			*action_reload = new QAction(QPixmap(PgModelerUiNS::getIconPath("atualizar")), trUtf8("Reload history"), ctx_menu),
+			*action_toggle_find = nullptr,
+			*exec_act = nullptr;
+
+	if(!find_history_parent->isVisible())
+		action_toggle_find = new QAction(QPixmap(PgModelerUiNS::getIconPath("buscar")), trUtf8("Find in history"), ctx_menu);
+	else
+		action_toggle_find = new QAction(trUtf8("Hide find tool"), ctx_menu);
 
 	ctx_menu->addSeparator();
-	ctx_menu->addAction(act);
+	ctx_menu->addAction(action_toggle_find);
+	ctx_menu->addAction(action_reload);
+	ctx_menu->addAction(action_save);
+	ctx_menu->addSeparator();
+	ctx_menu->addAction(action_clear);
 
-	if(ctx_menu->exec(QCursor::pos())==act)
+	exec_act = ctx_menu->exec(QCursor::pos());
+
+	if(exec_act == action_clear)
+	{
+		Messagebox msg_box;
+
+		msg_box.show(trUtf8("This action will wipe out all the SQL commands history for the current connection! Do you really want to proceed?"),
+									Messagebox::CONFIRM_ICON, Messagebox::YES_NO_BUTTONS);
+
+		if(msg_box.result() == QDialog::Accepted)
+		{
+			cmd_history_txt->clear();
+			cmd_history[sql_cmd_conn.getConnectionId(true,true)].clear();
+		}
+	}
+	else if(exec_act == action_save)
+		SQLExecutionWidget::saveSQLHistory();
+	else if(exec_act == action_reload)
+	{
+		SQLExecutionWidget::loadSQLHistory();
 		cmd_history_txt->clear();
+		cmd_history_txt->appendPlainText(cmd_history[sql_cmd_conn.getConnectionId(true,true)]);
+		cmd_history_hl->rehighlight();
+	}
+	else if(exec_act == action_toggle_find)
+		find_history_parent->setVisible(!find_history_parent->isVisible());
 
 	delete(ctx_menu);
 }
