@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2016 - Raphael Araújo e Silva <raphael@pgmodeler.com.br>
+# Copyright 2006-2018 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -36,9 +36,6 @@ class TablespaceWidget: public BaseObjectWidget, public Ui::TablespaceWidget {
 		TablespaceWidget(QWidget * parent = 0);
 		void setAttributes(DatabaseModel *model, OperationList *op_list, Tablespace *tablespc);
 
-	private slots:
-		void hideEvent(QHideEvent *event);
-
 	public slots:
 		void applyConfiguration(void);
 };
