@@ -1,7 +1,7 @@
 /*
 # PostgreSQL Database Modeler (pgModeler)
 #
-# Copyright 2006-2016 - Raphael Araújo e Silva <raphael@pgmodeler.com.br>
+# Copyright 2006-2018 - Raphael Araújo e Silva <raphael@pgmodeler.io>
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
@@ -19,14 +19,15 @@
 #include "viewwidget.h"
 #include "rulewidget.h"
 #include "triggerwidget.h"
+#include "indexwidget.h"
 #include "baseform.h"
 
 ViewWidget::ViewWidget(QWidget *parent): BaseObjectWidget(parent, OBJ_VIEW)
 {
 	try
 	{
-		ObjectTableWidget *tab=nullptr;
-		ObjectType types[]={ OBJ_TRIGGER, OBJ_RULE };
+		ObjectsTableWidget *tab=nullptr;
+		ObjectType types[]={ OBJ_TRIGGER, OBJ_RULE, OBJ_INDEX };
 		QGridLayout *grid=nullptr;
 		QVBoxLayout *vbox=nullptr;
 
@@ -35,17 +36,20 @@ ViewWidget::ViewWidget(QWidget *parent): BaseObjectWidget(parent, OBJ_VIEW)
 
 		Ui_ViewWidget::setupUi(this);
 
+		expression_txt=new NumberedTextEditor(this, true);
 		expression_hl=new SyntaxHighlighter(expression_txt, false, true);
 		expression_hl->loadConfiguration(GlobalAttributes::SQL_HIGHLIGHT_CONF_PATH);
+		referencias_grid->addWidget(expression_txt, 4, 1, 1, 4);
 
 		code_txt=new NumberedTextEditor(this);
+		code_txt->setReadOnly(true);
 		code_hl=new SyntaxHighlighter(code_txt);
 		code_hl->loadConfiguration(GlobalAttributes::SQL_HIGHLIGHT_CONF_PATH);
 		vbox=new QVBoxLayout(code_prev_tab);
 		vbox->setContentsMargins(4,4,4,4);
 		vbox->addWidget(code_txt);
 
-		cte_expression_txt=new NumberedTextEditor(this);
+		cte_expression_txt=new NumberedTextEditor(this, true);
 		cte_expression_hl=new SyntaxHighlighter(cte_expression_txt);
 		cte_expression_hl->loadConfiguration(GlobalAttributes::SQL_HIGHLIGHT_CONF_PATH);
 		vbox=new QVBoxLayout(cte_tab);
@@ -60,15 +64,15 @@ ViewWidget::ViewWidget(QWidget *parent): BaseObjectWidget(parent, OBJ_VIEW)
 		column_sel=new ObjectSelectorWidget(OBJ_COLUMN, true, this);
 		column_sel->enableObjectCreation(false);
 
-		references_tab=new ObjectTableWidget(ObjectTableWidget::ALL_BUTTONS, true, this);
+		references_tab=new ObjectsTableWidget(ObjectsTableWidget::ALL_BUTTONS, true, this);
 		references_tab->setColumnCount(4);
 		references_tab->setHeaderLabel(trUtf8("Col./Expr."),0);
 		references_tab->setHeaderLabel(trUtf8("Alias"),1);
 		references_tab->setHeaderLabel(trUtf8("Alias Col."),2);
-		references_tab->setHeaderLabel(trUtf8("Flags: SF FW AW VD"),3);
+		references_tab->setHeaderLabel(trUtf8("Flags: SF FW AW EX VD"),3);
 
-		cte_expression_cp=new CodeCompletionWidget(cte_expression_txt);
-		expression_cp=new CodeCompletionWidget(expression_txt);
+		cte_expression_cp=new CodeCompletionWidget(cte_expression_txt, true);
+		expression_cp=new CodeCompletionWidget(expression_txt, true);
 
 		frame_info=generateInformationFrame(trUtf8("To reference all columns in a table (*) just do not fill the field <strong>Column</strong>, this is the same as write <em><strong>[schema].[table].*</strong></em>"));
 
@@ -80,8 +84,8 @@ ViewWidget::ViewWidget(QWidget *parent): BaseObjectWidget(parent, OBJ_VIEW)
 		//Configuring the table objects that stores the triggers and rules
 		for(unsigned i=0, tab_id=1; i < sizeof(types)/sizeof(ObjectType); i++, tab_id++)
 		{
-			tab=new ObjectTableWidget(ObjectTableWidget::ALL_BUTTONS ^
-									  (ObjectTableWidget::UPDATE_BUTTON  | ObjectTableWidget::MOVE_BUTTONS), true, this);
+			tab=new ObjectsTableWidget(ObjectsTableWidget::ALL_BUTTONS ^
+									  (ObjectsTableWidget::UPDATE_BUTTON  | ObjectsTableWidget::MOVE_BUTTONS), true, this);
 
 			objects_tab_map[types[i]]=tab;
 
@@ -94,20 +98,26 @@ ViewWidget::ViewWidget(QWidget *parent): BaseObjectWidget(parent, OBJ_VIEW)
 			connect(tab, SIGNAL(s_rowRemoved(int)), this, SLOT(removeObject(int)));
 			connect(tab, SIGNAL(s_rowAdded(int)), this, SLOT(handleObject(void)));
 			connect(tab, SIGNAL(s_rowEdited(int)), this, SLOT(handleObject(void)));
+			connect(tab, SIGNAL(s_rowDuplicated(int,int)), this, SLOT(duplicateObject(int,int)));
 		}
 
 		objects_tab_map[OBJ_TRIGGER]->setColumnCount(4);
 		objects_tab_map[OBJ_TRIGGER]->setHeaderLabel(trUtf8("Name"), 0);
-		objects_tab_map[OBJ_TRIGGER]->setHeaderIcon(QPixmap(QString(":/icones/icones/uid.png")),0);
+		objects_tab_map[OBJ_TRIGGER]->setHeaderIcon(QPixmap(PgModelerUiNS::getIconPath("uid")),0);
 		objects_tab_map[OBJ_TRIGGER]->setHeaderLabel(trUtf8("Refer. Table"), 1);
-		objects_tab_map[OBJ_TRIGGER]->setHeaderIcon(QPixmap(QString(":/icones/icones/table.png")),1);
+		objects_tab_map[OBJ_TRIGGER]->setHeaderIcon(QPixmap(PgModelerUiNS::getIconPath("table")),1);
 		objects_tab_map[OBJ_TRIGGER]->setHeaderLabel(trUtf8("Firing"), 2);
-		objects_tab_map[OBJ_TRIGGER]->setHeaderIcon(QPixmap(QString(":/icones/icones/trigger.png")),2);
+		objects_tab_map[OBJ_TRIGGER]->setHeaderIcon(QPixmap(PgModelerUiNS::getIconPath("trigger")),2);
 		objects_tab_map[OBJ_TRIGGER]->setHeaderLabel(trUtf8("Events"), 3);
 
+		objects_tab_map[OBJ_INDEX]->setColumnCount(2);
+		objects_tab_map[OBJ_INDEX]->setHeaderLabel(trUtf8("Name"), 0);
+		objects_tab_map[OBJ_INDEX]->setHeaderIcon(QPixmap(PgModelerUiNS::getIconPath("uid")),0);
+		objects_tab_map[OBJ_INDEX]->setHeaderLabel(trUtf8("Indexing"), 1);
+
 		objects_tab_map[OBJ_RULE]->setColumnCount(3);
 		objects_tab_map[OBJ_RULE]->setHeaderLabel(trUtf8("Name"), 0);
-		objects_tab_map[OBJ_RULE]->setHeaderIcon(QPixmap(QString(":/icones/icones/uid.png")),0);
+		objects_tab_map[OBJ_RULE]->setHeaderIcon(QPixmap(PgModelerUiNS::getIconPath("uid")),0);
 		objects_tab_map[OBJ_RULE]->setHeaderLabel(trUtf8("Execution"), 1);
 		objects_tab_map[OBJ_RULE]->setHeaderLabel(trUtf8("Event"), 2);
 
@@ -162,7 +172,7 @@ ViewWidget::ViewWidget(QWidget *parent): BaseObjectWidget(parent, OBJ_VIEW)
 	}
 }
 
-ObjectTableWidget *ViewWidget::getObjectTable(ObjectType obj_type)
+ObjectsTableWidget *ViewWidget::getObjectTable(ObjectType obj_type)
 {
 	if(objects_tab_map.count(obj_type) > 0)
 		return(objects_tab_map[obj_type]);
@@ -180,10 +190,6 @@ int ViewWidget::openEditingForm(TableObject *object)
 														dynamic_cast<Class *>(object));
 	editing_form.setMainWidget(object_wgt);
 
-	//Disabling the apply button if the object is protected
-	if(object)
-		editing_form.apply_ok_btn->setEnabled(!object->isProtected());
-
 	return(editing_form.exec());
 }
 
@@ -191,7 +197,7 @@ void ViewWidget::handleObject(void)
 {
 	ObjectType obj_type=BASE_OBJECT;
 	TableObject *object=nullptr;
-	ObjectTableWidget *obj_table=nullptr;
+	ObjectsTableWidget *obj_table=nullptr;
 
 	try
 	{
@@ -203,6 +209,8 @@ void ViewWidget::handleObject(void)
 
 		if(obj_type==OBJ_TRIGGER)
 			openEditingForm<Trigger,TriggerWidget>(object);
+		else if(obj_type==OBJ_INDEX)
+			openEditingForm<Index,IndexWidget>(object);
 		else
 			openEditingForm<Rule,RuleWidget>(object);
 
@@ -215,6 +223,49 @@ void ViewWidget::handleObject(void)
 	}
 }
 
+void ViewWidget::duplicateObject(int curr_row, int new_row)
+{
+	ObjectType obj_type=BASE_OBJECT;
+	BaseObject *object=nullptr, *dup_object=nullptr;
+	ObjectsTableWidget *obj_table=nullptr;
+	View *view = dynamic_cast<View *>(this->object);
+	int op_id = -1;
+
+	try
+	{
+		obj_type=getObjectType(sender());
+
+		//Selects the object table based upon the passed object type
+		obj_table=getObjectTable(obj_type);
+
+		//Gets the object reference if there is an item select on table
+		if(curr_row >= 0)
+			object = reinterpret_cast<BaseObject *>(obj_table->getRowData(curr_row).value<void *>());
+
+		PgModelerNS::copyObject(&dup_object, object, obj_type);
+		dup_object->setName(PgModelerNS::generateUniqueName(dup_object, *view->getObjectList(obj_type), false, QString("_cp")));
+
+		op_id=op_list->registerObject(dup_object, Operation::OBJECT_CREATED, new_row, this->object);
+
+		view->addObject(dup_object);
+		view->setModified(true);
+		listObjects(obj_type);
+	}
+	catch(Exception &e)
+	{
+		//If operation was registered
+		if(op_id >= 0)
+		{
+			op_list->ignoreOperationChain(true);
+			op_list->removeLastOperation();
+			op_list->ignoreOperationChain(false);
+		}
+
+		listObjects(obj_type);
+		throw Exception(e.getErrorMessage(),e.getErrorType(),__PRETTY_FUNCTION__,__FILE__,__LINE__, &e);
+	}
+}
+
 void ViewWidget::removeObjects(void)
 {
 	View *view=nullptr;
@@ -281,7 +332,7 @@ ObjectType ViewWidget::getObjectType(QObject *sender)
 
 	if(sender)
 	{
-		map<ObjectType, ObjectTableWidget *>::iterator itr, itr_end;
+		map<ObjectType, ObjectsTableWidget *>::iterator itr, itr_end;
 
 		itr=objects_tab_map.begin();
 		itr_end=objects_tab_map.end();
@@ -300,9 +351,10 @@ ObjectType ViewWidget::getObjectType(QObject *sender)
 
 void ViewWidget::showObjectData(TableObject *object, int row)
 {
-	ObjectTableWidget *tab=nullptr;
+	ObjectsTableWidget *tab=nullptr;
 	Trigger *trigger=nullptr;
 	Rule *rule=nullptr;
+	Index *index=nullptr;
 	ObjectType obj_type;
 	QString str_aux;
 	unsigned i;
@@ -347,13 +399,20 @@ void ViewWidget::showObjectData(TableObject *object, int row)
 		//Column 2: Rule event type
 		tab->setCellText(~rule->getEventType(),row,2);
 	}
+	else
+	{
+		index=dynamic_cast<Index *>(object);
+
+		//Column 1: Indexing type
+		tab->setCellText(~index->getIndexingType(),row,1);
+	}
 
 	tab->setRowData(QVariant::fromValue<void *>(object), row);
 }
 
 void ViewWidget::listObjects(ObjectType obj_type)
 {
-	ObjectTableWidget *tab=nullptr;
+	ObjectsTableWidget *tab=nullptr;
 	unsigned count, i;
 	View *view=nullptr;
 
@@ -381,30 +440,6 @@ void ViewWidget::listObjects(ObjectType obj_type)
 	}
 }
 
-void ViewWidget::hideEvent(QHideEvent *evento)
-{
-	//View *view=dynamic_cast<View *>(this->object);
-	ObjectType types[]={ OBJ_TRIGGER, OBJ_RULE };
-
-	references_tab->removeRows();
-	tabWidget->setCurrentIndex(0);
-	cte_expression_txt->clear();
-
-	clearReferenceForm();
-
-	for(unsigned i=0; i < sizeof(types)/sizeof(ObjectType); i++)
-	{
-		objects_tab_map[types[i]]->blockSignals(true);
-		objects_tab_map[types[i]]->removeRows();
-		objects_tab_map[types[i]]->blockSignals(false);
-	}
-
-	if(this->new_object)// && !view->isModified())
-		this->cancelConfiguration();
-
-	BaseObjectWidget::hideEvent(evento);
-}
-
 void ViewWidget::clearReferenceForm(void)
 {
 	column_sel->clearSelector();
@@ -451,25 +486,22 @@ void ViewWidget::handleReference(int ref_idx)
 		if(static_cast<unsigned>(ref_type_cmb->currentIndex())==Reference::REFER_COLUMN)
 		{
 			ref=Reference(dynamic_cast<Table *>(table_sel->getSelectedObject()),
-						  dynamic_cast<Column *>(column_sel->getSelectedObject()),
-						  tab_alias_edt->text().toUtf8(), col_alias_edt->text().toUtf8());
+										dynamic_cast<Column *>(column_sel->getSelectedObject()),
+										tab_alias_edt->text().toUtf8(), col_alias_edt->text().toUtf8());
 		}
 		//Creating a reference to an expression
 		else
 		{
-			ref=Reference(expression_txt->toPlainText(),
-						  expr_alias_edt->text().toUtf8());
+			ref=Reference(expression_txt->toPlainText(), expr_alias_edt->text().toUtf8());
 		}
 
 		/* The reference must have an SQL application (be between SELECT-FROM, FROM-WHERE or after WHERE),
 			 if the user do not check some of these attributes raises an error */
-		if(!select_from_chk->isChecked() &&
-				!from_where_chk->isChecked() &&
-				!after_where_chk->isChecked() &&
+		if(!select_from_chk->isChecked() &&	!from_where_chk->isChecked() &&
+				!after_where_chk->isChecked() && !end_expr_chk->isChecked() &&
 				!view_def_chk->isChecked())
 			throw Exception(ERR_SQL_SCOPE_INV_VIEW_REF,__PRETTY_FUNCTION__,__FILE__,__LINE__);
 
-
 		if(view_def_chk->isChecked())
 		{
 			select_from_chk->setChecked(false);
@@ -478,7 +510,7 @@ void ViewWidget::handleReference(int ref_idx)
 		}
 
 		showReferenceData(ref, select_from_chk->isChecked(), from_where_chk->isChecked(),
-						  after_where_chk->isChecked(), view_def_chk->isChecked() ,ref_idx);
+											after_where_chk->isChecked(), end_expr_chk->isChecked(), view_def_chk->isChecked() ,ref_idx);
 
 		clearReferenceForm();
 		references_tab->clearSelection();
@@ -521,7 +553,8 @@ void ViewWidget::editReference(int ref_idx)
 	select_from_chk->setChecked(str_aux[0]=='1');
 	from_where_chk->setChecked(str_aux[1]=='1');
 	after_where_chk->setChecked(str_aux[2]=='1');
-	view_def_chk->setChecked(str_aux[3]=='1');
+	end_expr_chk->setChecked(str_aux[3]=='1');
+	view_def_chk->setChecked(str_aux[4]=='1');
 }
 
 void ViewWidget::showObjectName(void)
@@ -554,7 +587,7 @@ void ViewWidget::showObjectName(void)
 	}
 }
 
-void ViewWidget::showReferenceData(Reference refer, bool selec_from, bool from_where, bool after_where, bool view_def, unsigned row)
+void ViewWidget::showReferenceData(Reference refer, bool selec_from, bool from_where, bool after_where, bool end_expr, bool view_def, unsigned row)
 {
 	Table *tab=nullptr;
 	Column *col=nullptr;
@@ -589,6 +622,7 @@ void ViewWidget::showReferenceData(Reference refer, bool selec_from, bool from_w
 	str_aux+=(selec_from ? QString("1") : QString("0"));
 	str_aux+=(from_where ? QString("1") : QString("0"));
 	str_aux+=(after_where ? QString("1") : QString("0"));
+	str_aux+=(end_expr ? QString("1") : QString("0"));
 	str_aux+=(view_def ? QString("1") : QString("0"));
 	references_tab->setCellText(str_aux,row,3);
 
@@ -606,10 +640,12 @@ void ViewWidget::updateCodePreview(void)
 			Reference refer;
 			QString str_aux;
 			TableObject *tab_obj=nullptr;
-			map<ObjectType, ObjectTableWidget *>::iterator itr, itr_end;
-			unsigned i, count, i1, expr_type[]={Reference::SQL_REFER_SELECT,
+			map<ObjectType, ObjectsTableWidget *>::iterator itr, itr_end;
+			unsigned i, count, i1, expr_type[]={
+												Reference::SQL_REFER_SELECT,
 												Reference::SQL_REFER_FROM,
 												Reference::SQL_REFER_WHERE,
+												Reference::SQL_REFER_END_EXPR,
 												Reference::SQL_VIEW_DEFINITION};
 
 			aux_view.BaseObject::setName(name_edt->text().toUtf8());
@@ -680,7 +716,7 @@ void ViewWidget::updateCodePreview(void)
 void ViewWidget::setAttributes(DatabaseModel *model, OperationList *op_list, Schema *schema, View *view, double px, double py)
 {
 	unsigned i, count;
-	bool sel_from, from_where, after_where, view_def;
+	bool sel_from = false, from_where = false, after_where = false, view_def = false, end_expr = false;
 	Reference refer;
 
 	if(!view)
@@ -726,9 +762,10 @@ void ViewWidget::setAttributes(DatabaseModel *model, OperationList *op_list, Sch
 		sel_from=(view->getReferenceIndex(refer,Reference::SQL_REFER_SELECT) >= 0);
 		from_where=(view->getReferenceIndex(refer,Reference::SQL_REFER_FROM) >= 0);
 		after_where=(view->getReferenceIndex(refer,Reference::SQL_REFER_WHERE)>= 0);
+		end_expr=(view->getReferenceIndex(refer,Reference::SQL_REFER_END_EXPR)>= 0);
 		view_def=(view->getReferenceIndex(refer,Reference::SQL_VIEW_DEFINITION)>= 0);
 
-		showReferenceData(refer, sel_from, from_where, after_where, view_def, i);
+		showReferenceData(refer, sel_from, from_where, after_where, end_expr, view_def, i);
 	}
 
 	references_tab->blockSignals(false);
@@ -736,7 +773,7 @@ void ViewWidget::setAttributes(DatabaseModel *model, OperationList *op_list, Sch
 
 	listObjects(OBJ_TRIGGER);
 	listObjects(OBJ_RULE);
-
+	listObjects(OBJ_INDEX);
 }
 
 void ViewWidget::applyConfiguration(void)
@@ -744,11 +781,12 @@ void ViewWidget::applyConfiguration(void)
 	try
 	{
 		View *view=nullptr;
-		ObjectType types[]={ OBJ_TRIGGER, OBJ_RULE };
+		ObjectType types[]={ OBJ_TRIGGER, OBJ_RULE, OBJ_INDEX };
 		unsigned expr_type[]={ Reference::SQL_REFER_SELECT,
-							   Reference::SQL_REFER_FROM,
-							   Reference::SQL_REFER_WHERE,
-							   Reference::SQL_VIEW_DEFINITION};
+													 Reference::SQL_REFER_FROM,
+													 Reference::SQL_REFER_WHERE,
+													 Reference::SQL_REFER_END_EXPR,
+													 Reference::SQL_VIEW_DEFINITION};
 		Reference refer;
 		QString str_aux;
 
